/*
	$Id: book.c,v 1.82 2012/04/03 03:51:40 alex Exp alex $
	book.c--- make book
*/

#ifndef	lint
static char *rcsid __attribute__((unused)) = "$Id: book.c,v 1.82 2012/04/03 03:51:40 alex Exp alex $";
#endif

#include	<stdio.h>
#include	<stdlib.h>
#include	<stdarg.h>
#include	<unistd.h>
#include	<sys/types.h>
#include	<ctype.h>
#include	<string.h>
#include	<signal.h>
#include	"edb/edb.h"
#include	"edb/html.h"
#include	"edb/publish.h"


int
edb_book_print_not_implement(EdbPrint ep, EdbTupleInfo tuple)
{
	register Doc	doc = ep->doc;
	Caption	cap;

	edb_print_tuple_prefix(ep, tuple);
	cap = edb_make_caption(ep->ec, TupleEID(tuple), CAP_long);
	doc_text_puts3(doc,  "", (char *)cap->c_main, "");
	doc_text_puts3(doc,  " (", (char *)cap->c_sub, ")\n");
	edb_caption_free(ep->ec, cap);
	if(EIDisVALID(TupleOwner(tuple))) {
		cap = edb_make_caption(ep->ec, TupleOwner(tuple), 0);
		doc_text_puts3(doc,  "　　　　[OWNER] ", (char *)cap->c_main, "");
		edb_caption_free(ep->ec, cap);
	}
	return TRUE;
}

int
edb_book_print_AND(EdbPrint ep)
{
	register Doc	doc = ep->doc;

	HTML(doc) doc_puts(doc, " <i>and</i> ");
	TeX(doc) doc_puts(doc, "\\EdbAND ");
	PLAIN(doc) doc_puts(doc, " and ");
	return TRUE;
}

int
edb_book_tuple_column_match_eid(EdbContext ec, EdbTupleInfo tuple, char *column, eid_t eid)
{
	EdbTupleColumn	tc;
	EdbDatum d;

	tc = edb_tuple_seek_column(ec, tuple, (utf8_t *)column);
	ForTCDatum(d, tc) {
		if(DatumEID(d) == eid) return TRUE;
	}
	return FALSE;
}

void
edb_book_print_delimiter(EdbPrint ep, int put, char *delim)
{
	register Doc	doc = ep->doc;

	CSV(ep) {
		doc_puts(doc, "\t");
	} else LaTeX(ep) {
	} else if(put) {
		doc_puts(doc, delim);
	}
}

char *
edb_book_int2numstr(char *str, int n, int mode)
{

	strcpy(str, "");
	switch(mode) {
	case NUM_Roman:
		if(n>=100) {
			sprintf(str, "%d", n);
			break;
		} else if(n>=10) {
			switch(n/10) {
			case 1: strcat(str, "X");	break;
			case 2: strcat(str, "XX");	break;
			case 3: strcat(str, "XXX");	break;
			case 4: strcat(str, "XL");	break;
			case 5: strcat(str, "L");	break;
			case 6: strcat(str, "LX");	break;
			case 7: strcat(str, "LXX");	break;
			case 8: strcat(str, "LXXX");	break;
			case 9: strcat(str, "XC");	break;
			}
			edb_book_int2numstr(&str[strlen(str)], n%10, mode);
		} else {
			switch(n) {
			case 0: strcat(str, "");	break;
			case 1: strcat(str, "I");	break;
			case 2: strcat(str, "II");	break;
			case 3: strcat(str, "III");	break;
			case 4: strcat(str, "IV");	break;
			case 5: strcat(str, "V");	break;
			case 6: strcat(str, "VI");	break;
			case 7: strcat(str, "VII");	break;
			case 8: strcat(str, "VIII");	break;
			case 9: strcat(str, "IX");	break;
			}
		}
		break;
	case NUM_roman:
		edb_book_int2numstr(&str[strlen(str)], n%10, NUM_Roman);
		str_minuscule(str, str);
		break;
	case NUM_arabic:
	default:
		sprintf(str, "%d", n);
		break;
	}
	return str;
}

Link
edb_book_get_eid_list(EdbContext ec, eid_t teid, char *constraint)
{
	char	*query;
	char	*order;
	EdbTableInfo	ti;
	int	n;
	Link	l_eid;

	ti = edb_table_get(ec, teid);
	if(ti == NULL) return NULL;
	order = (char *)edb_table_make_order(ec, ti, (utf8_t *)"t1");
	asprintf(&query,
		"SELECT %s FROM %s AS t1 WHERE t1.%s %s %s %s %s",
		EDB_BASE_EID,
		TABLE_SN(ti),
		EDB_BASE_AVAILABLE,
		constraint ? "AND" : "",
		constraint ? constraint : "",
		STRisVALID(order) ? "ORDER BY" : "", order);
	FREE(order);
	l_eid = edb_pgsql_query_one_field(ec, (utf8_t *)query, &n);
	FREE(query);
	return l_eid;
}

Link
edb_book_get_tuple_list(EdbContext ec, EdbTableInfo table_ti, char *constraint)
{
	Link	l, l_eid, l_tuple;
	EdbTupleInfo	tuple;

	l_eid = edb_book_get_eid_list(ec, TABLE_EID(table_ti), constraint);
	edb_table_prefetch_tuples(ec, table_ti);
	l_tuple = NULL;
	ForLink(l, l_eid) {
		tuple = edb_tuple_get(ec, ATOI((char *)l->d), TEMPLATE(table_ti), 0);
		if(tuple==NULL) continue;
		LinkAppend(l_tuple, link_atom_new(tuple, NULL));
	}
	link_free(l_eid);
	return l_tuple;
}

static
int	/* output something */
book_alert(EdbPrint ep, char *s, int alert)
{

	return edb_print_alert(ep, alert ? EP_ALERT : EP_WARNING, "%s", s);
}

void
edb_book_begin_HTML(EdbPrint ep, Book book, char *title)
{
	register Doc	doc = ep->doc;

	doc_puts(doc, "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n");
	doc_puts(doc, "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n");
	doc_puts(doc, "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"ja\" lang=\"ja\">\n");
	doc_puts(doc, "	<head>\n");
	doc_puts(doc, "		<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />\n");
	doc_puts(doc, "		<meta http-equiv=\"Content-Style-Type\" content=\"text/css; charset=utf-8\" />\n");
	doc_puts(doc, "		<meta http-equiv=\"Pragma\" content=\"no-cache\" />\n");
	doc_puts(doc, "		<meta http-equiv=\"Cache-Control\" content=\"no-cache\" />\n");
	doc_puts(doc, "		<meta http-equiv=\"Expires\" content=\"Thu, 01 Dec 1994 16:00:00 GMT\" />\n");
	doc_puts(doc, "		<meta http-equiv=\"X-UA-Compatible\" content=\"IE=10\" />\n");
#if defined(EDB_READER)
	doc_puts(doc, "		<link rel=\"stylesheet\" type=\"text/css\" href=\"" WEB_PORTAL_URL "/edb-css/edb.css\" />\n");
#else
	doc_puts(doc, "		<link rel=\"stylesheet\" type=\"text/css\" href=\"/edb-css/edb.css\" />\n");
#endif
	doc_printf(doc, "<title>%s</title>\n", title);
	doc_puts(doc, "</head>\n");
	doc_printf(doc, "<body text=\"black\" style=\"background-color:%s;\">\n", BGCOLOR_COOL);
	html_file_msg_put(ep->ec, "cool.msg", doc->fp);
	doc_puts(doc, "<div id=\"container\">\n");
	doc_puts(doc, "	<div id=\"header\">\n");
	doc_text_puts3(doc, "		<h1>", title, "</h1>\n");
	doc_puts(doc, "	</div>\n\n");
}

void
edb_book_end_HTML(EdbPrint ep, Book book)
{
	register Doc	doc = ep->doc;

	doc_puts(doc, "	<div id=\"footer\">\n");
	html_print_postscript(ep->ec, doc->fp);
	doc_puts(doc, "	</div>\n");
	doc_puts(doc, "</div>\n");
	doc_puts(doc, "</body>\n");
	doc_puts(doc, "</html>\n");
}

#define	TCDatumText(tc, tag)	(((tc)->datum)->tag)

static
int
edb_book_print_column1(EdbPrint ep, EdbTupleInfo tuple, char *column, int mode, int alert)
{
	register Doc	doc = ep->doc;
	EdbTupleColumn	tc;
	int	status = FALSE;

	tc = edb_tuple_seek_column(ep->ec, tuple, (utf8_t *)column);
	if(tc==NULL) return status;

	edb_print_push(ep);
	ep->ct->alert_level = alert;
	ep->ct->tcd_idx_prefix = NULL;
	ep->ct->tcd_idx = 1;
	status |= edb_print_tcd(ep, tuple, tc, tc->datum);
	edb_print_pop(ep);

	return status;
}

int
edb_book_print_named_column(EdbPrint ep, EdbTupleInfo tuple,
	char *prefix, char *dprefix, char *tcname, char *dpostfix, char *postfix,
	char *substitute,
	int mode,
	int alert)
{
	register Doc	doc = ep->doc;
	int	status = FALSE;
	EdbTupleColumn tc;
	EdbColumnInfo ci;
	EdbDatum	datum;

	tc = edb_tuple_seek_column(ep->ec, tuple, (utf8_t *)tcname);
	if(tc) {
		ci = tc->ci;
	} else {
		ci = edb_table_seek_column(ep->ec, TupleTable(tuple), (utf8_t *)tcname);
	}
	if(ci==NULL) return FALSE;

	edb_print_push(ep);
	ep->ct->alert_level = alert;
	ep->ct->tcd_mode = mode;
	ep->ct->tcd_idx_prefix = NULL;
	if(tc && tc->datum) {
		NotLaTeX(ep) if(STRisVALID(prefix)) doc_puts(doc, prefix);
		ep->ct->tcd_idx=1;
		ForTCDatum(datum, tc) {
			int	st;

			NotLaTeX(ep) if(tc->datum!=datum && STRisVALID(dprefix)) doc_puts(doc, dprefix);
			st = edb_print_tcd(ep, tuple, tc, datum);
			NotLaTeX(ep) if(TCDatumNext(tc, datum) && STRisVALID(dpostfix)) doc_puts(doc, dpostfix);
			++ep->ct->tcd_idx;
			status = status || st;
		}
		NotLaTeX(ep) if(STRisVALID(postfix)) doc_puts(doc, postfix);
	} else {
		if(substitute) {
			doc_puts(doc, substitute);
		} else if(alert) {
			edb_print_alert_column(ep, alert, ci);
		}
	}
	edb_print_pop(ep);

	return status;
}

int
edb_book_print_named_column_AND(EdbPrint ep, EdbTupleInfo tuple,
	char *prefix, char *dprefix, char *tcname, char *dpostfix, char *postfix,
	char *substitute,
	int mode,
	int alert)
{
	register Doc	doc = ep->doc;
	int	status = FALSE;
	EdbTupleColumn tc;
	EdbColumnInfo ci;
	EdbDatum	datum;

	tc = edb_tuple_seek_column(ep->ec, tuple, (utf8_t *)tcname);
	if(tc) {
		ci = tc->ci;
	} else {
		ci = edb_table_seek_column(ep->ec, TupleTable(tuple), (utf8_t *)tcname);
	}
	if(ci==NULL) return FALSE;

	edb_print_push(ep);
	ep->ct->alert_level = alert;
	ep->ct->tcd_mode = mode;
	ep->ct->tcd_idx_prefix = NULL;
	if(tc && tc->datum) {
		NotLaTeX(ep) if(STRisVALID(prefix)) doc_puts(doc, prefix);
		ep->ct->tcd_idx=1;
		ForTCDatum(datum, tc) {
			int	st;

			NotLaTeX(ep) if(tc->datum!=datum && STRisVALID(dprefix)) doc_puts(doc, dprefix);
			st = edb_print_tcd(ep, tuple, tc, datum);
			NotLaTeX(ep) {
				if(TCDatumNext(tc, datum)) {
					if(LangIsEnglish(ep) && !TCDatumNext(tc, TCDatumNext(tc, datum))) {
						edb_book_print_AND(ep);
					} else if(STRisVALID(dpostfix)) {
						doc_puts(doc, dpostfix);
					}
				}
			}
			++ep->ct->tcd_idx;
			status = status || st;
		}
		NotLaTeX(ep) if(STRisVALID(postfix)) doc_puts(doc, postfix);
	} else {
		if(substitute) {
			doc_puts(doc, substitute);
		} else if(alert) {
			edb_print_alert_column(ep, alert, ci);
		}
	}
	edb_print_pop(ep);

	return status;
}

int
edb_book_print_text(EdbPrint ep, EdbTupleInfo tuple, char *column, int idx, int mode, int alert)
{
	register Doc	doc = ep->doc;
	EdbTupleColumn	tc;
	EdbDatum	datum;
	int	status;

	if(tuple == NULL) return edb_print_alert(ep, EP_ALERT, "!!!");
	tc = edb_tuple_seek_column(ep->ec, tuple, (utf8_t *)column);
	if(tc == NULL) return edb_print_alert(ep, EP_ALERT, "%s", column);
	datum = edb_tcdatum_get_datum(ep->ec, tc, idx);
	if(datum==NULL || !DatumAvail(datum)) return edb_print_alert_column(ep, alert, tc->ci);

	edb_print_push(ep);
	ep->ct->alert_level = alert;
	ep->ct->tcd_mode = mode;
	ep->ct->tcd_idx_prefix = NULL;
	ep->ct->tcd_idx = idx+1;
	status = edb_print_tcd(ep, tuple, tc, datum);
	edb_print_pop(ep);
	return status;
}

int
edb_book_print_column(EdbPrint ep, EdbTupleInfo tuple, utf8_t *column, int idx, int japanese, int mode, int alert)
{
	register Doc	doc = ep->doc;
	EdbTupleColumn	tc;
	int	status = FALSE;

	tc = edb_tuple_seek_column(ep->ec, tuple, (utf8_t *)column);
	if(tc==NULL) return FALSE;

	edb_print_push(ep);
	ep->ct->alert_level = alert;
	EpLang(ep) = japanese ? EP_LANG_Japanese : EP_LANG_English;
	ep->ct->tcd_mode = mode;
	ep->ct->tcd_idx_prefix = NULL;
	ep->ct->tcd_idx = idx+1;
	status |= edb_print_tcd(ep, tuple, tc, idx>=0 ? edb_tcdatum_get_datum(ep->ec, tc, idx) : NULL);
	edb_print_pop(ep);

	ITEMIZE(ep) return TRUE;
	return status;
}

int
edb_book_item_japanese(EdbPrint ep, EdbTupleInfo tuple, char *column)
{
	register EdbTupleColumn	tc;
	register EdbDatum	datum;

	tc = edb_tuple_seek_column(ep->ec, tuple, (utf8_t *)column);
	if(tc==NULL) return FALSE;
	datum = TCDatumFirst(tc);
	if(DatumJapaneseIsValid(datum)) return TRUE;
	return FALSE;
}

int
edb_book_print_person_info_name(EdbPrint ep, EdbTupleInfo tuple, int japanese, int mode)
{
	register Doc	doc = ep->doc;
	EdbTupleColumn	tc1, tc2, tc3;
	int	status = EP_STATUS_NORMAL;
	EdbContext	ec = ep->ec;
	int	underline = FALSE;
	EdbDatum	datum;

	tc1 = TupleColumn_person_surname(ec, tuple);
	tc2 = TupleColumn_person_givenname(ec, tuple);
	tc3 = TupleColumn_person_middlename(ec, tuple);
#if 0
	if(edb_print_eoi_check(ep, TupleEID(tuple)) && !ep->ct->nounderline) underline = TRUE;
#endif
	if(underline) NotLaTeX(ep) doc_underline_begin(ep->doc);
	if(japanese && (tc1 && (datum=TCDatumFirst(tc1)) && DatumJapaneseIsValid(datum))) {
		TeX(doc) doc_puts(doc, "\\EdbSMG{");
		if(tc1 && (datum=TCDatumFirst(tc1)) && DatumJapaneseIsValid(datum)) {
			doc_text_puts(doc, (char *)DatumJapanese(datum));
		} else {
			edb_print_alert(ep, EP_ALERT, "姓");
			status |= EP_STATUS_ALERT;
		}
		HTML(doc) doc_putc(doc, ' ');
		TeX(doc) doc_puts(doc, "}{");
		PLAIN(doc) doc_putc(doc, ' ');
		if(tc3 && (datum=TCDatumFirst(tc3)) && DatumJapaneseIsValid(datum)) {
			doc_text_puts(doc, (char *)DatumJapanese(datum));
			HTML(doc) doc_putc(doc, ' ');
			PLAIN(doc) doc_putc(doc, ' ');
		}
		TeX(doc) doc_puts(doc, "}{");
		if(tc2 && (datum=TCDatumFirst(tc2)) && DatumJapaneseIsValid(datum)) {
			doc_text_puts(doc, (char *)DatumJapanese(datum));
		} else {
			edb_print_alert(ep, EP_ALERT, "名");
			status |= EP_STATUS_ALERT;
		}
		TeX(doc) doc_puts(doc, "}");
	} else {
		TeX(doc) doc_puts(doc, "\\EdbGMS{");
		if(tc2 && (datum=TCDatumFirst(tc2)) && DatumEnglishIsValid(datum)) {
			if(ep->ct->person_name_mode==EP_PERSON_FULLNAME) {
				doc_text_puts(doc, (char *)DatumEnglish(datum));
			} else {
				doc_printf(doc, "%c.", DatumEnglish(datum)[0]);
			}
		} else {
			edb_print_alert(ep, EP_ALERT, "名");
			status |= EP_STATUS_ALERT;
		}
		HTML(doc) doc_putc(doc, ' ');
		TeX(doc) doc_puts(doc, "}{");
		PLAIN(doc) doc_putc(doc, ' ');
		if(tc3 && (datum=TCDatumFirst(tc3)) && DatumEnglishIsValid(datum)) {
			if(ep->ct->person_name_mode==EP_PERSON_FULLNAME) {
				doc_text_puts(doc, (char *)DatumEnglish(datum));
			} else {
				doc_printf(doc, "%c.", DatumEnglish(datum)[0]);
			}
			HTML(doc) doc_putc(doc, ' ');
			PLAIN(doc) doc_putc(doc, ' ');
		}
		TeX(doc) doc_puts(doc, "}{");
		if(tc1 && (datum=TCDatumFirst(tc1)) && DatumEnglishIsValid(datum)) {
			doc_text_puts(doc, (char *)DatumEnglish(datum));
		} else {
			edb_print_alert(ep, EP_ALERT, "姓");
			status |= EP_STATUS_ALERT;
		}
		TeX(doc) doc_puts(doc, "}");
	}
	if(underline) NotLaTeX(ep) doc_underline_end(ep->doc);
	return status;
}

static
int
edb_book_print_persons(EdbPrint ep, EdbTupleInfo tuple, char *column, int japanese, int mode)
{
	register Doc	doc = ep->doc;
	EdbTupleColumn	tc;
	int	i, num;
	int	status = FALSE;
	EdbDatum	datum;

	tc = edb_tuple_seek_column(ep->ec, tuple, (utf8_t *)column);
	if(tc==NULL) return status;
	if(tc->datum == NULL) {
		status = book_alert(ep, column, TRUE);
		return status;
	}
	num = edb_tcdatum_num(ep->ec, tc);
	i = 0;
	ForTCDatum(datum, tc) {
		edb_print_push(ep);
		EpLang(ep) = japanese ? EP_LANG_Japanese : EP_LANG_English;
		ep->ct->tcd_mode = mode;
		ep->ct->tcd_idx_prefix = NULL;
		ep->ct->tcd_idx = i+1;
		edb_print_tcd(ep, tuple, tc, datum);
		edb_print_pop(ep);
		ITEMIZE(ep) {
		} else {
			if(japanese) {
				if(i+1<num) doc_puts(doc, ", ");
			} else {
				if(i+2<num) {
					doc_puts(doc, ", ");
				} else if(i+1<num) {
					edb_book_print_AND(ep);
				}
			}
		}
		status = TRUE;
		++i;
	}
	return status;
}

static
eid_t
edb_book_get_organ_eid(EdbContext ec, eid_t peid, char *name)
{
	EdbTableInfo	ti;
	EdbColumnInfo	ci;
	char	*constraint;
	Link	l_eid;
	eid_t	eid;
	char	*s;

	if(name==NULL || strlen(name)==0) return peid;
	name = STRDUP(name);
	if((s=index(name, '.'))!=NULL) {
		*s++ = '\0';
		eid = edb_book_get_organ_eid(ec, edb_book_get_organ_eid(ec, peid, name), s);
	} else {
		ti = edb_table_get_by_name(ec, (utf8_t *)EDB_TABLE_ORGANIZATION_XN);
		if(ti==NULL) return 0;
		ci = edb_table_seek_column(ec, ti, (utf8_t *)"@.name");
		if(ci==NULL) return 0;
		constraint = NULL;
		if((ci = edb_table_seek_column(ec, ti, (utf8_t *)"@.upper"))!=NULL) {
			if(peid) {
				asprintf(&constraint, "text(%s)='%s' AND %s*=%d", ColumnSN(ci), name, ColumnSN(ci), peid);
			} else {
				asprintf(&constraint, "text(%s)='%s' AND %s ISNULL", ColumnSN(ci), name, ColumnSN(ci));
			}
		} else {
			asprintf(&constraint, "text(%s)='%s'", ColumnSN(ci), name);
		}
		l_eid = edb_book_get_eid_list(ec, TABLE_EID(ti), constraint);
		FREE(constraint);
		if(link_atoms(l_eid) == 0) {
			fprintf(stderr, "no EID of %s\n", name);
			return 0;
		} else if(link_atoms(l_eid) != 1) {
			fprintf(stderr, "duplicate EID of %s\n", name);
		}
		eid = ATOI((char *)l_eid->d);
		link_free(l_eid);
	}
	if(eid==0) {
		fprintf(stderr, "can't get EID of %s\n", name);
	}
	return eid;
}

static
Link
edb_book_get_organ_lower_eids_sub(EdbContext ec, eid_t peid, int iter)
{
	EdbTupleInfo	tuple;
	EdbTupleColumn	tc;
	char	*constraint;
	Link	l, l_eid;
	Link	l_child;

	if(iter<=0) return NULL;
	tuple = edb_tuple_get(ec, peid, 0);
	if(tuple==NULL) return NULL;

	tc = edb_tuple_seek_column(ec, tuple, (utf8_t *)"@.upper");
	if(tc==NULL) return 0;
	asprintf(&constraint, "%s *= %d", ColumnSN(tc->ci), peid);
	l_eid = edb_book_get_eid_list(ec, TABLE_EID(TupleTable(tuple)), constraint);
	FREE(constraint);
	l_child = NULL;
	ForLink(l, l_eid) {
		LinkAppend(l_child, edb_book_get_organ_lower_eids_sub(ec, ATOI((char *)l->d), iter-1));
	}
	return link_append(l_eid, l_child);
}

Link
edb_book_get_organ_lower_eids(EdbContext ec, eid_t eid)
{
	EdbTupleInfo	tuple;
	Link	l_eid;

	l_eid = link_atom_new_eid2s(eid);
	tuple = edb_tuple_get(ec, eid, 0);
	if(tuple==NULL) return l_eid;
	LinkAppend(l_eid, edb_book_get_organ_lower_eids_sub(ec, eid, EP_RECURSION_DEPTH));
	return l_eid;
}

Link
edb_book_classify_into_organization(EdbContext ec, Book book, Link l_eid, char *column, Section org)
{
	Link	l, lo, l_remain=NULL;
	int	i;

	ForLink(l, l_eid) {
		EdbTupleInfo	tuple;
		EdbTupleColumn	tc;
		int	match = FALSE;
		Link	ld;

		tuple = edb_tuple_get(ec, ATOI((char *)l->d), 0);
		if(tuple==NULL) continue;
		tc = edb_tuple_seek_column(ec, tuple, (utf8_t *)column);
		if(tc==NULL) continue;

		for(i=0; org[i].title && org[i].organ_eid != 0; ++i) {
			EdbDatum	d;
			int	omatch = FALSE;

			ForTCDatum(d, tc) {
				eid_t	oeid1;

				oeid1 = DatumEID(d);
				if(oeid1==0) continue;
				for(lo=org[i].organ_eids; lo && !omatch; lo=lo->n) {
					eid_t	oeid2 = ATOI((char *)lo->d);

					if(oeid1 == oeid2) {
						LinkAppend(org[i].l_eid, link_atom_new(l->d, NULL));
						omatch = match = TRUE;
					}
				}
				if(omatch) break;
			}
		}
		if(!match) LinkAppend(l_remain, link_atom_new(l->d, NULL));
	}
	return l_remain;
}

static
int
edb_book_print_tuple_byItemize_column(EdbPrint ep, EdbTupleInfo tuple, char *idx_prefix, Link l_column, int lang)
{
	register Doc	doc = ep->doc;
	int	status = EP_STATUS_NORMAL;
	int	printed = FALSE;

	Link	lc;

	edb_print_push(ep);
	EpLang(ep) = lang!=EP_LANG_English ? EP_LANG_Japanese : EP_LANG_English;
	ep->ct->tcd_mode = 0;
	ep->ct->tcd_idx_prefix = idx_prefix;
	ForLink(lc, l_column) {
		EdbTupleColumn	tc = (EdbTupleColumn)lc->d;
		EdbDatum	datum;
		int	i;

		i = 0;
		ForTCDatum(datum, tc) {
			int	status;

			ep->ct->tcd_idx = i+1;
			status = edb_print_tcd(ep, tuple, tc, datum);
			if(status) printed = TRUE;
			ITEMIZE(ep) {
			} else {
				if(tc->ci->pagenumber) break;
			}
			if(datum->l_child) {
				char	*idxstr = NULL;

				if(STRisVALID(idx_prefix)) {
					asprintf(&idxstr, "%s:%d", idx_prefix, i+1);
				} else {
					asprintf(&idxstr, "%d", i+1);
				}
				CSV(ep) doc_prefix_set(doc, "(");
				if(edb_book_print_tuple_byItemize_column(ep, tuple, idxstr, datum->l_child, lang)) {
					CSV(ep) doc_puts(doc, ")");
				} else {
					doc_prefix_reset(doc);
				}
				FREE(idxstr);
			}
			CSV(ep) if(TCDatumNext(tc, datum)) doc_puts(doc, ", ");
			++i;
		}
		if(i==0 && TCDatumNothing(tc)) {
			int	status;

			ep->ct->tcd_idx = 0;
			status = edb_print_tcd(ep, tuple, tc, NULL);
			if(status) printed = TRUE;
		}
		if(!STRisVALID(idx_prefix)) {
			CSV(ep) if(lc->n) doc_puts(doc, ", ");
		}
	}
	edb_print_pop(ep);

	return printed;
}

static
int
edb_book_print_tuple_byItemize(EdbPrint ep, EdbTupleInfo tuple, Link lcp)
{
	register Doc	doc = ep->doc;
	int	status = EP_STATUS_NORMAL;
	int	default_lang = EpLang(ep);
	EdbContext	ec = ep->ec;

	if(default_lang==EP_LANG_Auto) default_lang = edb_print_tuple_language(ep, tuple);
	edb_print_tuple_prefix(ep, tuple);
	CSV(ep) {
		Link	l;
		int	c;

		ForLink(l, lcp) {
			EdbColumnPrint	cp = (EdbColumnPrint)l->d;
			EdbTupleColumn	tc;
			EdbDatum	d;
			int	i;
			int	lang;

			tc = edb_tuple_seek_column(ec, tuple, cp->xmlname);
			if(tc==NULL) {
				doc_puts(doc, "\t");
				continue;
			}

			lang = cp->lang;
			if(lang == EP_LANG_Auto) lang = default_lang;
			i = 0;
			ForTCDatum(d, tc) {
				int	status;

				status = edb_book_print_column(ep, tuple, ColumnName_ja(tc->ci), i, lang!=EP_LANG_English, 0, EP_WARNING);
				ITEMIZE(ep) {
				} else {
					if(tc->ci->pagenumber) break;
				}
				if(d->l_child) {
					doc_prefix_set(doc, "(");
					if(edb_book_print_tuple_byItemize_column(ep, tuple, NULL, d->l_child, lang)) {
						doc_puts(doc, ")");
					} else {
						doc_prefix_reset(doc);
					}
				}
				if(TCDatumNext(tc, d)) doc_puts(doc, ", ");
				++i;
			}
			if(l->n) doc_puts(doc, "\t");
		}
	} else if(lcp) {
		Link	l;
		int	c;

		ForLink(l, lcp) {
			EdbColumnPrint	cp = (EdbColumnPrint)l->d;
			EdbTupleColumn	tc;
			EdbDatum	d;
			int	i;
			int	lang;

			tc = edb_tuple_seek_column(ec, tuple, cp->xmlname);
			if(tc==NULL) continue;

			lang = cp->lang;
			if(lang == EP_LANG_Auto) lang = default_lang;
			i = 0;
			ForTCDatum(d, tc) {
				int	status;

				status = edb_book_print_column(ep, tuple, ColumnName_ja(tc->ci), i, lang!=EP_LANG_English, 0, EP_WARNING);
				ITEMIZE(ep) {
				} else {
					if(tc->ci->pagenumber) break;
				}
				++i;
			}
			if(i==0 && TCDatumNothing(tc)) edb_book_print_column(ep, tuple, ColumnName_ja(tc->ci), -1, lang!=EP_LANG_English, 0, EP_WARNING);
		}
	} else {
		edb_book_print_tuple_byItemize_column(ep, tuple, NULL, tuple->l_column, default_lang);
	}
	edb_print_tuple_postfix(ep, tuple);
	return status;
}

int
edb_book_print_caption(EdbPrint ep, EdbTupleInfo tuple)
{
	register Doc	doc = ep->doc;
	int	status;

	ITEMIZE(ep) {
		edb_book_print_tuple_byItemize(ep, tuple, NULL);
		return TRUE;
	}
	edb_print_push(ep);
	PrintStyle(ep) = EP_PrintStyle_CAPTION;
	status = edb_print_tuple(ep, tuple);
	edb_print_pop(ep);
	return status;
}

int
edb_book_print_tuple(EdbPrint ep, EdbTupleInfo tuple, Link lcp)
{
	register Doc	doc = ep->doc;
	int	status;

	CSV(ep) {
		if(lcp) {
			edb_book_print_tuple_byItemize(ep, tuple, lcp);
		} else {
			Link	l;

			lcp = NULL;
			ForLink(l, TupleTable(tuple)->l_column) {
				EdbColumnInfo	ci = l->d;
				EdbColumnPrint	cp;

				if(ci) {
					cp = edb_column_print_new(ci->xmlname);
					LinkAppend(lcp, link_atom_new(cp, edb_column_print_free));
				}
			}
			edb_book_print_tuple_byItemize(ep, tuple, lcp);
			link_free(lcp);
		}
		return TRUE;
	}
	ITEMIZE(ep) {
		edb_book_print_tuple_byItemize(ep, tuple, lcp);
		return TRUE;
	}
	if(PrintStyleIsXML(ep)) {
		return edb_print_tuple_XML(ep, tuple);
	}
	edb_print_push(ep);
	PrintStyle(ep) = EP_PrintStyle_STANDARD;
	status = edb_print_tuple(ep, tuple);
	edb_print_pop(ep);
	return status;
}

#include	"module.inc"

void
edb_book_print_initialize(EdbPrint ep)
{
#if 0
static struct {
	char	*xmlname;
	EdbPrintTupleFunc_t	std;
	EdbPrintTupleFunc_t	xml;
	EdbPrintTupleFunc_t	csv;
	EdbPrintTupleFunc_t	item;
	EdbPrintTupleFunc_t	cap;
} info2print[] = {{
	EDB_TABLE_Article_XN,
	edb_book_print_article,
	NULL,
	NULL,
	NULL,
	edb_book_print_article,
#if 0
},{
	EDB_TABLE_ResearchActivity_XN,
	edb_book_print_research,
	NULL,
	NULL,
	NULL,
	edb_book_print_research,
#endif
}};
#endif
	int	i;

#if 0
	for(i=0; i<Ns(info2print); ++i) {
		edb_print_tuple_register_func(ep, info2print[i].xmlname, PrintStyle_STANDARD, info2print[i].std);
		edb_print_tuple_register_func(ep, info2print[i].xmlname, PrintStyle_XML, info2print[i].xml);
		edb_print_tuple_register_func(ep, info2print[i].xmlname, PrintStyle_CSV, info2print[i].csv);
		edb_print_tuple_register_func(ep, info2print[i].xmlname, PrintStyle_ITEMIZE, info2print[i].item);
		edb_print_tuple_register_func(ep, info2print[i].xmlname, PrintStyle_CAPTION, info2print[i].cap);
	}
#endif
	for(i=0; i<Ns(edb_print_default_modules); ++i) {
		struct EDB_PRINT_MODULE_INFO	*info;

		if(edb_print_default_modules[i]==NULL || (info = *edb_print_default_modules[i])==NULL) continue;
		edb_print_tuple_register_language_func(ep, info->xmlname, info->lang);
		edb_print_tuple_register_func(ep, info->xmlname, PrintStyle_STANDARD, info->std);
		edb_print_tuple_register_func(ep, info->xmlname, PrintStyle_XML, info->xml);
		edb_print_tuple_register_func(ep, info->xmlname, PrintStyle_CSV, info->csv);
		edb_print_tuple_register_func(ep, info->xmlname, PrintStyle_ITEMIZE, info->item);
		edb_print_tuple_register_func(ep, info->xmlname, PrintStyle_CAPTION, info->cap);
	}
}

