/*
	$Id: look.c,v 1.44 2008/08/09 09:22:20 alex Exp $
	edb_look--- lookup database
*/

#ifndef	lint
static char *rcsid __attribute__((unused)) = "$Id: look.c,v 1.44 2008/08/09 09:22:20 alex Exp $";
#endif

#include	<stdio.h>
#include	<stdlib.h>
#include	<stdarg.h>
#include	<unistd.h>
#include	<ctype.h>
#include	<sys/types.h>
#include	<string.h>
#include	<signal.h>
#include	"edb/edb.h"
#include	"edb/html.h"
#include	"edb/publish.h"

#define	STATUS_NORMAL	0
#define	STATUS_WARNING	1
#define	STATUS_ERROR	2

/*
	edb_look_show_table
*/
void
edb_look_show_table_show_tuple(EdbPrint ep, EdbTupleInfo tuple, Link l_print)
{
	register EdbContext	ec = ep->ec;
	register Doc		doc = ep->doc;
	Caption	cap;

	if(!edb_book_print_tuple(ep, tuple, l_print)) {
		cap = edb_tuple_make_caption(ec, tuple, 0);
		doc_printf(doc, "%d ", TupleEID(tuple));
		doc_text_puts(doc, (char *)cap->c_main);
		edb_caption_free(ec, cap);
	}
	doc_puts(doc, "\n");
}

static
void
edb_look_split(char *str, char **s1, char *delim, char **s2)
{
	char	*p;

	str = STRDUP(str);
	p = strstr(str, delim);
	if(p) {
		*p = '\0';
		p += strlen(delim);
		*s1 = strdup(str);
		if(*p=='{') {
			++p;
			*s2 = strdup(p);
			p = rindex(*s2, '}');
			if(p) *p = '\0';
		} else {
			*s2 = strdup(p);
		}
	} else {
		*s1 = strdup(str);
		*s2 = strdup("");
	}
}

static
Link
edb_look_analyze_order(EdbContext ec, EdbTableInfo ti, char *order, Doc doc)
{
	Link	l, lstr, lco=NULL;

	if(ti==NULL) return NULL;
	if(order==NULL) return NULL;
	order = STRDUP(order);
	str_oneline(order);
	if(order[0]=='{') {
		char	*s;

		order = STRDUP(&order[1]);
		s = rindex(order, '}');
		if(s) *s = '\0';
	}
	lstr = str_split(order, ' ');
	for(l=lstr; l; l=l->n) {
		EdbColumnInfo	ci;
		EdbColumnOrder	co;
		char	*name = (char *)l->d;
		int	desc = FALSE;

		if(name[0] == '!') {
			desc = TRUE;
			++name;
		}
		ci = edb_table_seek_column(ec, ti, (utf8_t *)name);
		if(ci) {
#if 0
			if(edb_table_column_reldb_elements(ec, ti, NULL, ci)==1) {
				co = edb_column_order_new(ColumnSN(ci));
				co->desc = desc;
				LinkAppend(lco, link_atom_new(co, edb_column_order_free));
			} else {
				doc_comment_printf(doc, "WARNING: Column [%s] is ARRAY.", name);
			}
#else
			co = edb_column_order_new(ColumnSN(ci));
			co->desc = desc;
			LinkAppend(lco, link_atom_new(co, edb_column_order_free));
#endif
		} else {
			doc_comment_printf(doc, "WARNING: Column [%s] is not found.", name);
		}
	}
	link_free(lstr);
	return lco;
}

struct EDB_CONDITION {
	int	type;
	EdbTypeInfo	typeinfo;
#define	CONDITION_TYPE_EID		0
#define	CONDITION_TYPE_INTEGER	1
#define	CONDITION_TYPE_INTEGER2	2
#define	CONDITION_TYPE_REAL		3
#define	CONDITION_TYPE_REAL2		4
#define	CONDITION_TYPE_DATE		5
#define	CONDITION_TYPE_DATE2		6
#define	CONDITION_TYPE_TEXT		7
	eid_t		d_eid;
	int		d_int;
	int		d_int1;
	int		d_int2;
	double	d_double;
	double	d_double1;
	double	d_double2;
	char		*d_text;
};
typedef	struct EDB_CONDITION	*EdbCondition;
static EdbCondition	edb_condition_new	__P((int, ...));
static void			edb_condition_free	__P((EdbCondition));
static
EdbCondition
edb_condition_new(int type, ...)
{
	EdbCondition	cond;
	va_list	ap;

	cond = (EdbCondition)malloc(sizeof(struct EDB_CONDITION));
	if(cond==NULL) return NULL;
	bzero(cond, sizeof(struct EDB_CONDITION));
	cond->type = type;
	va_start(ap, type);
	switch(type) {
	case CONDITION_TYPE_EID:
		cond->d_eid = va_arg(ap, int);
		break;
	case CONDITION_TYPE_INTEGER:
		cond->d_int = va_arg(ap, int);
		break;
	case CONDITION_TYPE_INTEGER2:
		cond->d_int1 = va_arg(ap, int);
		cond->d_int2 = va_arg(ap, int);
		break;
	case CONDITION_TYPE_REAL:
		cond->d_double = va_arg(ap, double);
		break;
	case CONDITION_TYPE_REAL2:
		cond->d_double1 = va_arg(ap, double);
		cond->d_double2 = va_arg(ap, double);
		break;
	case CONDITION_TYPE_DATE:
		cond->d_int = va_arg(ap, int);
		break;
	case CONDITION_TYPE_DATE2:
		cond->d_int1 = va_arg(ap, int);
		cond->d_int2 = va_arg(ap, int);
		break;
	case CONDITION_TYPE_TEXT:
		cond->d_text = va_arg(ap, char *);
		cond->d_text = strdup(cond->d_text);
		break;
	}
	va_end(ap);

	return cond;
}

static
void
edb_condition_free(EdbCondition cond)
{

	FREE(cond->d_text);
	FREE(cond->d_text);
	free(cond);
}

struct EDB_COLUMN_CONDITION {
#define	EDB_COLUMN_CONDITION_NORM	0
#define	EDB_COLUMN_CONDITION_REF	1
#define	EDB_COLUMN_CONDITION_PERM	2
#define	EDB_COLUMN_CONDITION_OWN	3
#define	EDB_COLUMN_CONDITION_MAP	4
	int	type;
	char	*xmlname;
	int	resolved;
	int	cnt;
	char	*condition_edb;
	Link	l_condition;
	char	*condition_sql;
};
typedef	struct EDB_COLUMN_CONDITION	*EdbColumnCondition;
static EdbColumnCondition	edb_column_condition_new	__P((int, char *));
static void				edb_column_condition_free	__P((EdbColumnCondition));

static
EdbColumnCondition
edb_column_condition_new(int type, char *xmlname)
{
	EdbColumnCondition	cc;

	cc = (EdbColumnCondition)malloc(sizeof(struct EDB_COLUMN_CONDITION));
	if(cc==NULL) return NULL;
	bzero(cc, sizeof(struct EDB_COLUMN_CONDITION));
	cc->type = type;
	if(STRisVALID(xmlname)) {
		cc->xmlname = strdup(xmlname);
	} else {
		cc->xmlname = NULL;
	}
	return cc;
}

static
void
edb_column_condition_free(EdbColumnCondition cc)
{

	if(cc->xmlname) free(cc->xmlname);
	if(cc->condition_edb) free(cc->condition_edb);
	if(cc->l_condition) link_free(cc->l_condition);
	if(cc->condition_sql) free(cc->condition_sql);
	free(cc);
}

static
Link
edb_look_post_condition_check(EdbContext ec, Link l_src, Link l_cc)
{
	Link	lc;

	for(lc=l_cc; lc; lc=lc->n) {
		EdbColumnCondition	cc = (EdbColumnCondition)lc->d;
		Link	lt, l_dst;

		if(cc->resolved) continue;
		cc->resolved = TRUE;
		l_dst = NULL;
		for(lt=l_src; lt; lt=lt->n) {
			EdbTupleInfo	tuple = (EdbTupleInfo)lt->d;
			EdbTupleColumn	tc;
			int	ok = FALSE;
			Link	lc2;

			if(cc->type==EDB_COLUMN_CONDITION_NORM) {
				tc = edb_tuple_seek_column_by_xmlname(ec, tuple, (utf8_t *)cc->xmlname);
				if(tc==NULL) continue;
				for(lc2=cc->l_condition; lc2 && !ok; lc2=lc2->n) {
					EdbCondition	cond = (EdbCondition)lc2->d;
					EdbDatum	datum;

					switch(cond->type) {
					case CONDITION_TYPE_EID:
						ForTCDatum(datum, tc) {
							if(DatumEID(datum) == cond->d_eid) {
								ok = TRUE;
								break;
							}
						}
						break;
					default:
						ok = TRUE;
						cc->resolved = FALSE;
						break;
					}
				}
			} else if(cc->type==EDB_COLUMN_CONDITION_REF) {
				EdbIndexInfo	ei;

				ei = edb_index_get(ec, TupleEID(tuple));
				if(ei) {
					for(lc2=cc->l_condition; lc2 && !ok; lc2=lc2->n) {
						EdbCondition	cond = (EdbCondition)lc2->d;
						register eid_t	*eidp;

						switch(cond->type) {
						case CONDITION_TYPE_EID:
#if 0
							if(bitree_search(ei->bt_eid, EDB_INDEX_REF_EID_MARK, cond->d_eid, NULL)) ok = TRUE;
#else
							for(eidp=ei->reids; eidp && *eidp; ++eidp) {
								if(EIDisValid(*eidp) && *eidp==cond->d_eid) ok = TRUE;
							}
#endif
							break;
						default:
							ok = TRUE;
							cc->resolved = FALSE;
							break;
						}
					}
				} else {
					ok = TRUE;
					cc->resolved = FALSE;
				}
			} else if(cc->type==EDB_COLUMN_CONDITION_PERM) {
				for(lc2=cc->l_condition; lc2 && !ok; lc2=lc2->n) {
					EdbCondition	cond = (EdbCondition)lc2->d;

					switch(cond->type) {
					case CONDITION_TYPE_EID:
#ifdef EDB_READER
#else /* !EDB_READER */
						if(edb_auth_writable_by(ec, cond->d_eid, TupleEID(tuple))) ok = TRUE;
#endif /* EDB_READER */
						break;
					default:
						ok = TRUE;
						cc->resolved = FALSE;
						break;
					}
				}
			} else if(cc->type==EDB_COLUMN_CONDITION_OWN) {
				for(lc2=cc->l_condition; lc2 && !ok; lc2=lc2->n) {
					EdbCondition	cond = (EdbCondition)lc2->d;

					switch(cond->type) {
					case CONDITION_TYPE_EID:
						if(TupleOwner(tuple)==cond->d_eid) ok = TRUE;
						break;
					default:
						ok = TRUE;
						cc->resolved = FALSE;
						break;
					}
				}
			} else if(cc->type==EDB_COLUMN_CONDITION_MAP) {
				for(lc2=cc->l_condition; lc2 && !ok; lc2=lc2->n) {
					EdbCondition	cond = (EdbCondition)lc2->d;

					switch(cond->type) {
					case CONDITION_TYPE_EID:
						if(TupleMapto(tuple)==cond->d_eid) ok = TRUE;
						break;
					default:
						ok = TRUE;
						cc->resolved = FALSE;
						break;
					}
				}
			}
			if(ok) LinkAppend(l_dst, link_atom_new(tuple, NULL));
		}
		link_free(l_src);
		l_src = l_dst;
	}

	return l_src;
}

#define	RECURSION_DEPTH	10

static
Link
edb_look_get_hierarchy_organ(EdbContext ec, eid_t upper, char *organ, Doc doc)
{
	char	*p;
	Link	l, l_tuples, l_organ;
	EdbTableInfo	table_ti;
	EdbColumnInfo	ci_upper, ci_name;
	char *organ_sub;

	doc_comment_printf(doc, "edb_look_get_hierarchy_organ: %s", organ);
	if(organ[0] == '{') {
		organ = STRDUP(&organ[1]);
		if((p=rindex(organ, '}'))!=NULL) *p = '\0';
	} else {
		organ = STRDUP(organ);
	}
	p = strstr(organ, "\".\"");
	if(p) {
		++p;
		*p++ = '\0';
		organ_sub = STRDUP(p);
	} else {
		organ_sub = NULL;
	}

	if(organ[0] == '"') {
		organ = STRDUP(&organ[1]);
		if((p=rindex(organ, '"'))!=NULL) *p = '\0';
	}

	table_ti = edb_table_get_by_name(ec, (utf8_t *)EDB_TABLE_ORGANIZATION_XN);
	if(table_ti==NULL) return NULL;

	ci_upper = edb_table_seek_column(ec, table_ti, (utf8_t *)"上位");
	if(ci_upper==NULL) return NULL;

	ci_name = edb_table_seek_column(ec, table_ti, (utf8_t *)"名称");
	if(ci_name==NULL) return NULL;

	if(EIDisVALID(upper)) {
		l_tuples = edb_table_get_tuples(ec, table_ti, "%s AND %s=%d AND %s='%s'::text",
			EDB_BASE_AVAILABLE, ColumnSN(ci_upper), upper, ColumnSN(ci_name), organ);
	} else {
		l_tuples = edb_table_get_tuples(ec, table_ti, "%s AND %s='%s'::text",
			EDB_BASE_AVAILABLE, ColumnSN(ci_name), organ);
	}

	if(organ_sub) {
		l_organ = NULL;
		for(l=l_tuples; l; l=l->n) {
			EdbTupleInfo	rtuple = (EdbTupleInfo)l->d;

			LinkAppend(l_organ, edb_look_get_hierarchy_organ(ec, TupleEID(rtuple), organ_sub, doc));
		}
		link_free(l_tuples);
	} else {
		l_organ = l_tuples;
	}

	return l_organ;
}

static
int
tuple_bitree_search(EdbContext ec, BiTree bt, EdbTupleInfo tuple)
{

	return (bitree_search(bt, tuple, TupleEID(tuple), NULL)!=NULL);
}

static
BiTree
tuple_bitree_append(EdbContext ec, BiTree bt, EdbTupleInfo tuple)
{

	if(!tuple_bitree_search(ec, bt, tuple)) {
		BiTreeAdd(bt, tuple, TupleEID(tuple), NULL, NULL, NULL);
	}
	return bt;
}

static
int
tuple_bitree_to_link_func(register void *o1, register void *o2)
{
	EdbTableInfo ti = o1;
	Link *l_tuples = o2;

	LinkAppend(*l_tuples, link_atom_new(ti, NULL));
	return TRUE;
}

static
Link
tuple_bitree_to_link(EdbContext ec, BiTree bt)
{
	Link	l_tuples = NULL;

	bitree_callback(bt, tuple_bitree_to_link_func, &l_tuples);
	return l_tuples;
}

static
BiTree
tuple_expand(EdbContext ec, BiTree bt, EdbTupleInfo tuple, int iter)
{
	Link	l, l_index;
	EdbTupleInfo	rtuple;

	if(iter<=0) return bt;
	if(tuple==NULL) return bt;
	if(tuple_bitree_search(ec, bt, tuple)) return bt;
	bt = tuple_bitree_append(ec, bt, tuple);

	edb_tuple_expand_check(ec, tuple);
	for(l=tuple->expand_map; l; l=l->n) bt = tuple_expand(ec, bt, (EdbTupleInfo)l->d, iter-1);
	if(EIDisVALID(TupleMapto(tuple))) {
		rtuple = edb_tuple_get(ec, TupleMapto(tuple), 0);
		if(rtuple) bt = tuple_expand(ec, bt, rtuple, iter-1);
	}
	return bt;
}

static
void
get_time_interval(char *value, int *from_date, int *to_date)
{
	int	y1, m1, d1;
	int	y2, m2, d2;
	Link	lstr;
	int	match;
	int	f=0, t=0;

	/* YYYY-MM-DD YYYY-MM-DD */
	lstr = str_split(value, ' ');
	if(link_atoms(lstr)>=1) f = ATOI((char *)lstr->d);
	if(link_atoms(lstr)>=2) {
		t = ATOI((char *)lstr->n->d);
	} else {
		t = f;
	}
	*from_date = f;
	*to_date = t;
}

static
void
eliminateSingleQuote(char *str)
{
	char *s;

	for(s=str; *s; ++s) {
		if(*s!='\'') continue;
		if(*(s+1)=='\'') { ++s; continue; }
		*s = '.';
	}
}

static
EdbColumnCondition
edb_look_column_condition(EdbContext ec, EdbTableInfo ti, int column_type, char *column_sqlname, char *str, char *op, Doc doc)
{
	char	*cond;
	int	count = 0;
	EdbColumnCondition	cc;
	char	condition[EDB_BUFSIZ];
	EdbColumnInfo	ci = NULL;

	if(str == NULL) return NULL;
	str = STRDUP(str);

	if(column_type==EDB_COLUMN_CONDITION_NORM) {
		ci = edb_table_seek_column_by_sqlname(ec, ti, (utf8_t *)column_sqlname);
	}
	cc = edb_column_condition_new(column_type, column_sqlname);
	if(cc==NULL) return NULL;
	cc->condition_edb = strdup(str);

	strcpy(condition, "");
	/* COLUMN=...value... */
	while((cond = str_get_phrase(&str))!=NULL) {
		char	*value, *p;
		char	condstr[EDB_BUFSIZ];
		Link	l, l_tuples = NULL;

		value = STRDUP(cond);
		condstr[0] = '\0';
		if(value[0]=='"') {
			if(ci) {
				if((p = rindex(value, '"'))!=NULL) *p = '\0';
				switch(ci->typeInfo->basicTypeID) {
				case EDB_TYPE_DATE:
					if(edb_table_column_reldb_elements(ec, ti, NULL, ci)==1) {
						int	from_date, to_date;

						get_time_interval(&value[1], &from_date, &to_date);
						sprintf(condstr, "(t1.%s>=%d AND t1.%s<=%d)", ColumnSN(ci), from_date, ColumnSN(ci), to_date);
					} else if(edb_table_column_reldb_elements(ec, ti, NULL, ci)==2) {
						int	from_date, to_date;

						get_time_interval(&value[1], &from_date, &to_date);
						sprintf(condstr, "NOT (t1.%s[2]<%d OR %d<t1.%s[1])", ColumnSN(ci), from_date, to_date, ColumnSN(ci));
					} else {
#if 0
						doc_color_begin(doc, EP_WARNING_COLOR);
						doc_printf(doc, "WARNING: unsupported query!! ");
						doc_printf(doc, "Then, %s %s \"%s\" is ignored. Unnecessary information(s) might be listed!", ColumnName_ja(ci), op, cond);
						doc_color_end(doc);
#else
						int	from_date, to_date;

						get_time_interval(&value[1], &from_date, &to_date);
						sprintf(condstr, "EXISTS (SELECT 1 FROM %s as t2"
							" WHERE t1." EDB_BASE_EID "=t2." EDB_BASE_EID " AND (t2.%s>=%d AND t2.%s<=%d)"
							")",
							ColumnRdbSN(ci),
							ColumnSN(ci), from_date, ColumnSN(ci), to_date);
#endif
					}
					break;
				case EDB_TYPE_DATE2:
					if(edb_table_column_reldb_elements(ec, ti, NULL, ci)==1) {
						int	from_date, to_date;

						get_time_interval(&value[1], &from_date, &to_date);
						sprintf(condstr, "NOT (int4(t1.%s%%int8(100000000))<%d OR %d<int4(t1.%s/int8(100000000)))", ColumnSN(ci), from_date, to_date, ColumnSN(ci));
					} else {
#if 0
						doc_color_begin(doc, EP_WARNING_COLOR);
						doc_printf(doc, "WARNING: unsupported query!! ");
						doc_printf(doc, "Then, %s %s \"%s\" is ignored. Unnecessary information(s) might be listed!", ColumnName_ja(ci), op, cond);
						doc_color_end(doc);
#else
						int	from_date, to_date;

						get_time_interval(&value[1], &from_date, &to_date);
						sprintf(condstr, "EXISTS (SELECT 1 FROM %s as t2"
							" WHERE t1." EDB_BASE_EID "=t2." EDB_BASE_EID " AND NOT (int4(t2.%s%%int8(100000000))<%d OR %d<int4(t2.%s/int8(100000000)))"
							")",
							ColumnRdbSN(ci),
							ColumnSN(ci), from_date, to_date, ColumnSN(ci));
#endif
					}
					break;
				case EDB_TYPE_INT:
					if(edb_table_column_reldb_elements(ec, ti, NULL, ci)==1) {
						int	v1, v2;

						sscanf(&value[1], "%d %d", &v1, &v2);
						sprintf(condstr, "(t1.%s>='%d'::int4 AND t1.%s<='%d'::int4)", ColumnSN(ci), v1, ColumnSN(ci), v2);
					} else {
						int	v1, v2;

						sscanf(&value[1], "%d %d", &v1, &v2);
						sprintf(condstr, "NOT (t1.%s**<'%d'::int4 OR t1.%s**>'%d'::int4)", ColumnSN(ci), v1, ColumnSN(ci), v2);
					}
					break;
				case EDB_TYPE_REAL:
					if(edb_table_column_reldb_elements(ec, ti, NULL, ci)==1) {
						double	v1, v2;

						sscanf(&value[1], "%lf %lf", &v1, &v2);
						sprintf(condstr, "(t1.%s>='%f'::float8 AND t1.%s<='%f'::float8)", ColumnSN(ci), v1, ColumnSN(ci), v2);
					} else {
#if 0
						doc_color_begin(doc, EP_WARNING_COLOR);
						doc_printf(doc, "WARNING: unsupported query!! ");
						doc_printf(doc, "Then, %s %s \"%s\" is ignored. Unnecessary information(s) might be listed!", ColumnName_ja(ci), op, cond);
						doc_color_end(doc);
#else
						double	v1, v2;

						sscanf(&value[1], "%lf %lf", &v1, &v2);
						sprintf(condstr, "EXISTS (SELECT 1 FROM %s as t2"
							" WHERE t1." EDB_BASE_EID "=t2." EDB_BASE_EID " AND (t2.%s>='%f'::float8 AND t2.%s<='%f'::float8)"
							")",
							ColumnRdbSN(ci),
							ColumnSN(ci), v1, ColumnSN(ci), v2);
#endif
					}
					break;
				default:
#if	defined(TEXT_VARIANT_CONVERSION)
					if(!strcmp(op, "~*")) {
						char *vv = (char *)edb_text_variant_conversion((utf8_t *)&value[1], FALSE);
						eliminateSingleQuote(vv);
						if(edb_table_column_reldb_elements(ec, ti, NULL, ci)==1) {
							sprintf(condstr, "t1.%s%s'%s'::text", ColumnSN_VARIANT(ci), op, vv);
						} else if(!strcmp(ci->typeInfo->SQL_type, "text")) {
							sprintf(condstr, "text_array_texticregexeq(t1.%s, '%s'::text)", ColumnSN_VARIANT(ci), vv);
						} else {
							sprintf(condstr, "t1.%s*%s'%s'::text", ColumnSN_VARIANT(ci), op, vv);
						}
						FREE(vv);
					} else {
						eliminateSingleQuote(&value[1]);
						if(edb_table_column_reldb_elements(ec, ti, NULL, ci)==1) {
							sprintf(condstr, "t1.%s%s'%s'::text", ColumnSN(ci), op, &value[1]);
						} else {
							sprintf(condstr, "t1.%s*%s'%s'::text", ColumnSN(ci), op, &value[1]);
						}
					}
#else
					eliminateSingleQuote(&value[1]);
					if(edb_table_column_reldb_elements(ec, ti, NULL, ci)==1) {
						sprintf(condstr, "t1.%s%s'%s'::text", ColumnSN(ci), op, &value[1]);
					} else if(!strcmp(ci->typeInfo->SQL_type, "text") && !strcmp(op, "~*")) {
						sprintf(condstr, "text_array_texticregexeq(t1.%s, '%s'::text)", ColumnSN(ci), &value[1]);
					} else {
						sprintf(condstr, "t1.%s*%s'%s'::text", ColumnSN(ci), op, &value[1]);
					}
#endif	/* TEXT_VARIANT_CONVERSION */
					break;
				}
			} else {
				doc_printf(doc, "Then, %s %s \"%s\" is ignored. Unnecessary information(s) might be listed!", "????", op, cond);
			}
		} else if(!strncasecmp(value,"\\E{", 3)) {
			EdbTupleInfo	rtuple;

			rtuple = edb_tuple_get(ec, refstr2eid(value), 0);
			if(rtuple) l_tuples = link_atom_new(rtuple, NULL);
		} else if(!strncasecmp(value,"\\Organization{", 14)) {
			l_tuples = edb_look_get_hierarchy_organ(ec, 0, &value[13], doc);
		} else if(!strncasecmp(value,"\\組織{", 6)) {
			l_tuples = edb_look_get_hierarchy_organ(ec, 0, &value[5], doc);
		} else {
			l_tuples = edb_look_exec(ec, value, NULL, FALSE, doc);
		}

		if(l_tuples) {
			BiTree	bt = NULL;

			for(l=l_tuples; l; l=l->n) {
				bt = tuple_expand(ec, bt, (EdbTupleInfo)l->d, RECURSION_DEPTH);
			}
			link_free(l_tuples);
			l_tuples = tuple_bitree_to_link(ec, bt);

			for(l=l_tuples; l; l=l->n) {
				EdbTupleInfo	rtuple = (EdbTupleInfo)l->d;

				if(count>0) strcat(condition, " OR ");
				if(ci) {
					if(edb_table_column_reldb_elements(ec, ti, NULL, ci)==1) {
						sprintf(&condition[strlen(condition)], "t1.%s=%d", ColumnSN(ci), TupleEID(rtuple));
					} else {
						sprintf(&condition[strlen(condition)], "t1.%s*=%d", ColumnSN(ci), TupleEID(rtuple));
					}
				} else if(column_type==EDB_COLUMN_CONDITION_REF) {
#if 0
					sprintf(&condition[strlen(condition)], "t2.edb_eid*=%d", TupleEID(rtuple));
#else
					sprintf(&condition[strlen(condition)], "t1.l_eid*=%d", TupleEID(rtuple));
#endif
				} else if(column_type==EDB_COLUMN_CONDITION_PERM) {
					sprintf(&condition[strlen(condition)], "TRUE");
				} else if(column_type==EDB_COLUMN_CONDITION_OWN) {
					sprintf(&condition[strlen(condition)], "t1.owner=%d", TupleEID(rtuple));
				} else if(column_type==EDB_COLUMN_CONDITION_MAP) {
					sprintf(&condition[strlen(condition)], "t1.mapto_eid=%d", TupleEID(rtuple));
				}
				LinkAppend(cc->l_condition, link_atom_new(edb_condition_new(CONDITION_TYPE_EID, TupleEID(rtuple)), edb_condition_free));
				++count;
			}
			bitree_free(bt, NULL);
			link_free(l_tuples);
		}
		if(STRisVALID(condstr)) {
			if(count>0) strcat(condition, " OR ");
			strcat(condition, condstr);
			++count;
		}
	}
	str_oneline(condition);
	doc_comment_printf(doc, "%s", condition);
	cc->condition_sql = strdup(condition);
	cc->cnt = count;
	return cc;
}

#define	TOO_COMPLEX		1024

static
Link
edb_look_condition(EdbContext ec, EdbTableInfo ti, char *str, char *condition, Doc doc)
{
	char	*cond;
	char	*column, *value, *p;
	EdbColumnInfo	ci;
	Link	l;
	char	*op = "=";
	char	*p1, *p2;
	int	count = 0;
	Link	l_cc = NULL;

	if(str == NULL) return NULL;
	str = STRDUP(str);
	doc_comment_printf(doc, "condition : %s", str);

	/* COLUMN=...value... */
	condition[0] = '\0';
	while((cond = str_get_phrase(&str))!=NULL) {
		char	condstr1[EDB_BUFSIZ], condstr2[EDB_BUFSIZ];
		EdbColumnCondition	cc;
		int	column_type;
		char	*column_sqlname = NULL;

		p1 = index(cond, '=');
		p2 = index(cond, '~');
		if(p1 && (!p2 || p1<p2)) {
			edb_look_split(cond, &column, "=", &value);
			op = "=";
		} else {
			edb_look_split(cond, &column, "~", &value);
			op = "~*";
		}
		p = column ; column = STRDUP(column);	free(p);
		p = value ; value = STRDUP(value);	free(p);
		doc_comment_printf(doc, "%s %s %s", column, op, value);
		if(!strcasecmp(column, "\\REF")) {
			column_type = EDB_COLUMN_CONDITION_REF;
		} else if(!strcasecmp(column, "\\PERM")) {
			column_type = EDB_COLUMN_CONDITION_PERM;
		} else if(!strcasecmp(column, "\\OWN")) {
			column_type = EDB_COLUMN_CONDITION_OWN;
		} else if(!strcasecmp(column, "\\MAP")) {
			column_type = EDB_COLUMN_CONDITION_MAP;
		} else {
			column_type = EDB_COLUMN_CONDITION_NORM;
			ci = edb_table_seek_column(ec, ti, (utf8_t *)column);
			if(ci==NULL) {
				doc_printf(doc, "ERROR: column(%s) is not found!\n", column);
				break;
			}
			column_sqlname = (char *)ColumnSN(ci);
			doc_comment_printf(doc, "%s = %s", ColumnSN(ci), value);
#ifdef EDB_READER
#else /* !EDB_READER */
			if(!edb_auth_readable_table_column_for_all_tuples(ec, ti, ci)) {
				doc_color_begin(doc, EP_WARNING_COLOR);
				doc_printf(doc, "WARNING: This column is not readable for you!! ");
				doc_printf(doc, "Then, \"%s\" is ignored.", cond);
				doc_color_end(doc);
				continue;
			}
#endif /* EDB_READER */
		}
		cc = edb_look_column_condition(ec, ti, column_type, column_sqlname, value, op, doc);
		if(cc==NULL) continue;
		LinkAppend(l_cc, link_atom_new(cc, edb_column_condition_free));
		strcpy(condstr1, cc->condition_sql);
		strcpy(condstr2, "");
		if(cc->cnt>1) {
			sprintf(condstr2, "(%s)", condstr1);
		} else if(cc->cnt>0) {
			strcpy(condstr2, condstr1);
		}
		if(cc->cnt>TOO_COMPLEX || cc->type==EDB_COLUMN_CONDITION_PERM) {
			cc->resolved = FALSE;
			strcpy(condition, "TRUE");
		} else if(cc->cnt>0) {
			if(count++>0) {
				strcpy(condstr1, condition);
				sprintf(condition, "%s AND %s", condstr2, condstr1);
			} else {
				strcpy(condition, condstr2);
			}
			cc->resolved = TRUE;
		} else {
			doc_color_begin(doc, EP_WARNING_COLOR);
			doc_comment_printf(doc, "WARNING: QUERY \"%s\" is NOT matched!! ", cond);
			doc_color_end(doc);
			strcpy(condition, "FALSE");
			cc->resolved = TRUE;
			break;
		}
	}
	doc_comment_printf(doc, "%s", condition);
	return l_cc;
}

Link
edb_look_exec(EdbContext ec, char *look, char *order, int top, Doc doc)
{
	eid_t	eid, table_eid;
	char	*p, *table_name, *cond, condition[EDB_BUFSIZ];
	Link	l, l_tuples, l_cc = NULL;
	EdbTableInfo	table_ti;
	EdbTupleInfo	tuple;
	Link	l_order = NULL;
	int	resolved;

	look = STRDUP(look);
	doc_comment_printf(doc, "LOOK=%s", look);

	/* TABLE.{...condition...} */
	edb_look_split(look, &table_name, ".", &cond);
	p = table_name ; table_name = STRDUP(table_name);	free(p);
	p = cond ; cond = STRDUP(cond);	free(p);
	doc_comment_printf(doc, "%s . { %s }", table_name, cond ? cond : "");

	/* table */
	eid = refstr2eid(table_name);
	if(!EIDisVALID(eid)) eid = edb_table_xmlname_to_eid(ec, (utf8_t *)table_name);
	if(!EIDisVALID(eid)) return NULL;

	if(edb_eid_isin_table(ec, eid)!=0) {
		tuple = edb_tuple_get(ec, eid, 0);
		if(tuple && TupleAvail(tuple)) return link_atom_new(tuple, NULL);
	}

	table_eid = eid;
	table_ti = edb_table_get(ec, table_eid);
	if(!table_ti) {
		doc_comment_printf(doc, "ERROR: can't load target table.");
		return NULL;
	}

	if(order) l_order = edb_look_analyze_order(ec, table_ti, order, doc);

	l_cc = edb_look_condition(ec, table_ti, cond, condition, doc);
	resolved = TRUE;
	for(l=l_cc; l; l=l->n) {
		resolved = (resolved && ((EdbColumnCondition)l->d)->resolved);
	}
	doc_comment_printf(doc, "SQL: %s", condition);
	if(STRisVALID(condition)) {
		l_tuples = edb_table_get_tuples_for_look(ec, table_ti, l_order, "t1.%s AND %s", EDB_BASE_AVAILABLE, condition);
	} else {
		l_tuples = edb_table_get_tuples_for_look(ec, table_ti, l_order, "t1.%s", EDB_BASE_AVAILABLE);
	}
	if(!resolved) l_tuples = edb_look_post_condition_check(ec, l_tuples, l_cc);
	for(l=l_cc; l; l=l->n) {
		EdbColumnCondition	cc = (EdbColumnCondition)l->d;

		if(!cc->resolved) {
			Link	lc;

			doc_color_begin(doc, EP_WARNING_COLOR);
			doc_printf(doc, "WARNING: QUERY is too complex!! \n");
			doc_comment_printf(doc, "%s : %s", cc->condition_edb, cc->condition_sql);
			doc_printf(doc, "Then, \"%s\" is ignored. Unnecessary information might be listed!\n", cond);
			doc_color_end(doc);
		}
	}
	link_free(l_order);
	link_free(l_cc);
	return l_tuples;
}


