/*
	$Id: mod_patent.c,v 1.6 2012/04/03 03:51:40 alex Exp alex $
	mod_patent.c---
*/

#ifndef	lint
static char *rcsid __attribute__((unused)) = "$Id: mod_patent.c,v 1.6 2012/04/03 03:51:40 alex Exp alex $";
#endif

#include	<stdio.h>
#include	<stdlib.h>
#include	<stdarg.h>
#include	<unistd.h>
#include	<sys/types.h>
#include	<ctype.h>
#include	<string.h>
#include	"edb/edb.h"
#include	"edb/html.h"
#include	"edb/publish.h"

#define	STUDENT_FGC	"#008000"

static
int
patent_language(EdbPrint ep, EdbTupleInfo tuple)
{
	int	lang = EP_LANG_Japanese;

	if(!edb_book_item_japanese(ep, tuple, "@.name")) lang = EP_LANG_English;
	return lang;
}

static
int
print_date(EdbPrint ep, EdbTupleInfo tuple, EdbDatum super_datum)
{
	register int	status;

	if(super_datum==NULL) return FALSE;
	status = edb_print_tuple_named_column(ep, tuple, super_datum, " (", "", "@.date", NULL, ", ", ")", NULL, EP_ALERT);
	return status;
}

static
int
edb_book_kind_of_patent(EdbPrint ep, EdbTupleInfo tuple)
{

	return edb_book_tuple_column_match_eid(ep->ec, tuple, "@.kind", 10463);
}

static
int
inventor_is_student(EdbContext ec, EdbTupleInfo tuple, EdbTupleColumn tc, EdbDatum datum)
{
	EdbTupleColumn ttc;

	// 学籍番号
	ttc = edb_tuple_seek_TC_by_xmlname(ec, tuple, tc, datum, (utf8_t *)"@.studentcode");
	if(ttc!=NULL && ttc->datum!=NULL && DatumEnglishIsValid(ttc->datum)) return TRUE;

////	// 所属(学生)
////	ttc = edb_tuple_seek_TC_by_xmlname(ec, tuple, tc, datum, (utf8_t *)"@.affiliate");
////	if(ttc==NULL || ttc->datum==NULL || !DatumEIDisValid(ttc->datum)) return FALSE;
////
////	// 所属(肩書)
////	ttc = edb_tuple_seek_TC_by_xmlname(ec, tuple, tc, datum, (utf8_t *)"@.title");
////	if(ttc==NULL || ttc->datum==NULL || !DatumEIDisValid(ttc->datum)) return FALSE;
////	return TRUE;
	return FALSE;
}

static
int
patent_print_inventor(EdbPrint ep, EdbTupleInfo tuple, EdbTupleColumn tc, EdbDatum datum)
{
	int	st = 0;

	edb_print_push(ep);
	edb_print_tcd_register_func(ep, "patent.inventor", PrintStyle(ep), NULL);
	if(!ep->ct->nocolor && !strcmp((char *)ColumnXN(tc->ci), "patent.inventor") && inventor_is_student(ep->ec, tuple, tc, datum)) {
		doc_color_begin(ep->doc, STUDENT_FGC);
		st = edb_print_tcd(ep, tuple, tc, datum);
		doc_color_end(ep->doc);
	} else {
		st = edb_print_tcd(ep, tuple, tc, datum);
	}
	edb_print_pop(ep);
	return st;
}

static
int
edb_book_print_patent(EdbPrint ep, EdbTupleInfo tuple)
{
	register Doc	doc = ep->doc;
	int	patent;
	int	stat;
	int	status = EP_STATUS_NORMAL;

	edb_print_push(ep);
	edb_print_tuple_prefix(ep, tuple);
	if(LangIsAuto(ep)) EpLang(ep) = patent_language(ep, tuple);
	patent = edb_book_kind_of_patent(ep, tuple);
	if(patent) {
		ep->ct->print_patent_number[PrintStyle(ep)] = edb_print_patent_number;
	} else {
		ep->ct->print_patent_number[PrintStyle(ep)] = edb_print_pmodel_number;
	}
	edb_print_type_register_func(ep, EDB_TYPE_DATE, PrintStyle(ep), edb_print_type_DATE_YM);

	CSV(ep) {
	} else {
		LaTeX(ep) if(!LangIsJapanese(ep)) doc_puts(doc, "\\英文\n");
	}

	CSV(ep) {
		status = edb_print_tuple_named_column(ep, tuple, NULL, "", "", "@.kind", NULL, "", "", NULL, EP_ALERT);
		edb_book_print_delimiter(ep, status, " : ");
	}

	if(!ep->ct->noauthor) {
		edb_print_push(ep);
		HTML(ep->doc) edb_print_tcd_register_func(ep, "patent.inventor", PrintStyle(ep), patent_print_inventor);
		ep->ct->column_mode |= EP_COLUMN_MODE_LIST_AND;
		ep->ct->tcd_mode |= EP_TCD_MODE_BOLD;
		status = edb_print_tuple_named_column(ep, tuple, NULL, "", "", "@.inventor", NULL, ", ", "", NULL, EP_ALERT);
		edb_print_pop(ep);
		edb_book_print_delimiter(ep, status, " : ");
	}

	stat = edb_print_tuple_named_column(ep, tuple, NULL, "", "", "@.name", NULL, ", ", "", NULL, EP_ALERT);
	edb_book_print_delimiter(ep, stat, ", ");

	stat = edb_print_tuple_named_column(ep, tuple, NULL, "", "", "@.apply", print_date, ", ", "", NULL, patent ? EP_ALERT : EP_WARNING);
	edb_book_print_delimiter(ep, TRUE, ", ");

	stat = edb_print_tuple_named_column(ep, tuple, NULL, "", "", "@.disclosure", print_date, ", ", "", NULL, EP_WARNING);
	edb_book_print_delimiter(ep, TRUE, ", ");

	stat = edb_print_tuple_named_column(ep, tuple, NULL, "", "", "@.number", NULL, ", ", "", NULL, EP_WARNING);
	edb_book_print_delimiter(ep, FALSE, "");
	if(stat) edb_print_tuple_named_column(ep, tuple, NULL, " (", "", "@.date", NULL, ", ", ")", NULL, EP_ALERT);
	edb_book_print_delimiter(ep, TRUE, ".");

	edb_print_tuple_postfix(ep, tuple);
	edb_print_pop(ep);

	return status;
}


static struct EDB_PRINT_MODULE_INFO module_info = {
/* xmlname */	EDB_TABLE_Patent_XN,
/* language */	patent_language,
/* std */	edb_book_print_patent,
/* xml */	NULL,
/* csv */	NULL,
/* item */	NULL,
/* cap */	edb_book_print_patent
};
DECLARE_PRINT_MODULE(patent, module_info)


