/*
	$Id: print_column.c,v 1.8 2011/04/18 09:45:59 alex Exp $
	print_column.c--- print column
*/

#ifndef	lint
static char *rcsid __attribute__((unused)) = "$Id: print_column.c,v 1.8 2011/04/18 09:45:59 alex Exp $";
#endif

#include	<stdio.h>
#include	<stdarg.h>
#include	<string.h>
#include	"edb/edb.h"
#include	"edb/tex.h"
#include	"edb/publish.h"

int
edb_print_tuple_column(EdbPrint ep, EdbTupleInfo tuple,
		char *prefix, char *dprefix,
		EdbTupleColumn tc,
		EdbPrintTupleColumnCallbackFunc_t print,
		char *dpostfix, char *postfix,
		char *substitute,
		int	alert)
{
	register Doc	doc = ep->doc;
	EdbContext	ec = ep->ec;
	int	status = FALSE;
	Link	l, l_datum;
	EdbDatum	datum;

	l_datum = NULL;
	ForTCDatum(datum, tc) {
		if((ep->ct->column_mode&EP_COLUMN_MODE_AT_ANYTIME) || edb_datum_at_present(ec, datum)) {
			LinkAppend(l_datum, link_atom_new(datum, NULL));
		}
	}
	edb_print_push(ep);
	if(l_datum) {
		int	idx;

		if(STRisVALID(prefix)) {
			doc_puts(doc, prefix);
			status = TRUE;
		}
		idx = 0;
		ForLink(l, l_datum) {
			EdbDatum	datum = l->d;

			if(l->p && STRisVALID(dprefix)) {
				doc_puts(doc, dprefix);
				status = TRUE;
			}
			ep->ct->tcd_idx = idx+1;
			if(edb_print_tcd(ep, tuple, tc, datum)) {
				if(print && tc->ci->l_child) (*print)(ep, tuple, datum);
				status = TRUE;
				if(tc->ci->alias) break;
			}
			if(l->n) {
				if(LangIsEnglish(ep) && !l->n->n && (ep->ct->column_mode&EP_COLUMN_MODE_LIST_AND)) {
					HTML(doc) doc_puts(doc, " <i>and</i> ");
					TeX(doc) doc_puts(doc, "\\EdbAND ");
					PLAIN(doc) doc_puts(doc, " and ");
				} else if(STRisVALID(dpostfix)) {
					doc_puts(doc, dpostfix);
					status = TRUE;
				}
			}
			++idx;
		}
		if(STRisVALID(postfix)) {
			doc_puts(doc, postfix);
			status = TRUE;
		}
		EdbLinkFree2(ec, l_datum);
	} else if(substitute) {
		if(STRisVALID(substitute)) doc_puts(doc, substitute);
	} else if(!TCDatumNothing(tc)) {
		if(STRisVALID(prefix)) doc_prefix_set(doc, prefix);
		status = edb_print_alert_column(ep, alert, tc->ci);
		if(status) {
			if(STRisVALID(postfix)) doc_puts(doc, postfix);
		} else {
			doc_prefix_reset(doc);
		}
	}
	edb_print_pop(ep);
	return status;
}

int
edb_print_tuple_named_column(EdbPrint ep, EdbTupleInfo tuple, EdbDatum super_datum,
	char		*prefix, char *dprefix,
	char		*xn,
	EdbPrintTupleColumnCallbackFunc_t print,
	char		*dpostfix, char *postfix,
	char		*substitute,
	int		alert)
{
	register Doc	doc = ep->doc;
	EdbContext	ec = ep->ec;
	EdbTupleColumn	tc;
	int	status = FALSE;

	tc = edb_tuple_seek_TC_by_xmlname(ec, tuple, super_datum?DatumSuper(super_datum) : NULL, super_datum, (utf8_t *)xn);
	if(tc) {
		status = edb_print_tuple_column(ep, tuple, prefix, dprefix, tc, print, dpostfix, postfix, substitute, alert);
	} else if(substitute) {
		if(STRisVALID(substitute)) doc_puts(doc, substitute);
	} else {
		EdbColumnInfo	ci = NULL;

		if(super_datum==NULL) ci = edb_table_seek_column(ec, TupleTable(tuple), (utf8_t *)xn);
		if(STRisVALID(prefix)) doc_prefix_set(doc, prefix);
		if(ci) {
			status = edb_print_alert_column(ep, alert, ci);
		} else {
			status = edb_print_alert(ep, alert, "%s", xn);
		}
		if(status) {
			if(STRisVALID(postfix)) doc_puts(doc, postfix);
		} else {
			doc_prefix_reset(doc);
		}
	}
	return status;
}


