/*
	$Id: print_table.c,v 1.12 2011/04/18 09:45:59 alex Exp $
	print.c--- print
*/

#ifndef	lint
static char *rcsid __attribute__((unused)) = "$Id: print_table.c,v 1.12 2011/04/18 09:45:59 alex Exp $";
#endif

#include	<stdio.h>
#include	<stdarg.h>
#include	<string.h>
#include	"edb/edb.h"
#include	"edb/publish.h"

/*
	edb_print_table_information
*/
void
edb_print_table_information(EdbPrint ep, EdbTableInfo ti, int mode)
{
	EdbContext	ec = ep->ec;
	Doc	doc = ep->doc;

	if(ti == NULL || doc==NULL) return;
	if(!TABLE_Avail(ti)) doc_color_begin(doc, "#808080");
	edb_base_print_information(ec, TABLE_Base(ti), mode, doc);
	doc_puts(doc, "<ul>\n");
	doc_puts(doc, "<li>名前\n");
	doc_puts(doc, "<ul>\n");
	doc_text_puts3(doc, "<li>（英）", (char *)TABLE_Name_en(ti), "\n");
	doc_text_puts3(doc, "（日）", (char *)TABLE_Name_ja(ti), "\n");
	doc_text_puts3(doc, "（読）", (char *)TABLE_Name_pr(ti), "\n");
	doc_puts(doc, "</ul>\n");
	doc_puts(doc, "<li>説明\n");
	doc_puts(doc, "<ul>\n");
	doc_text_puts3(doc, "<li>（英）", (char *)TABLE_Desc_en(ti), "\n");
	doc_text_puts3(doc, "<li>（日）", (char *)TABLE_Desc_ja(ti), "\n");
	doc_puts(doc, "</ul>\n");
	doc_puts(doc, "<li>解説\n");
	doc_puts(doc, "<ul>\n");
	doc_text_puts3(doc, "<li>（英）", (char *)TABLE_Explain_en(ti), "\n");
	doc_text_puts3(doc, "<li>（日）", (char *)TABLE_Explain_en(ti), "\n");
	doc_puts(doc, "</ul>\n");
	doc_puts(doc, "<li>Tuple: \n");
	doc_text_puts3(doc, "Read: ", (char *)edb_auth_string(ec, TABLE_TupleRead(ti), TRUE), ", ");
	doc_text_puts3(doc, "Create: ", (char *)edb_auth_string(ec, TABLE_TupleCreate(ti), TRUE), ", ");
	doc_text_puts3(doc, "Write: ", (char *)edb_auth_string(ec, TABLE_TupleWrite(ti), TRUE), ", ");
	doc_text_puts3(doc, "Delete: ", (char *)edb_auth_string(ec, TABLE_TupleDelete(ti), TRUE), ".\n");
	doc_puts(doc, "<li>Column: \n");
	doc_text_puts3(doc, "Read: ", (char *)edb_auth_string(ec, TABLE_ColumnRead(ti), TRUE), ", ");
	doc_text_puts3(doc, "Create: ", (char *)edb_auth_string(ec, TABLE_ColumnCreate(ti), TRUE), ", ");
	doc_text_puts3(doc, "Write: ", (char *)edb_auth_string(ec, TABLE_ColumnWrite(ti), TRUE), ", ");
	doc_text_puts3(doc, "Delete: ", (char *)edb_auth_string(ec, TABLE_ColumnDelete(ti), TRUE), ".\n");
	doc_puts(doc, "</ul>\n");
	if(!TABLE_Avail(ti)) doc_color_end(doc);
}

void
edb_print_table_in_xml(EdbPrint ep, EdbTableInfo ti, int mode)
{
	EdbContext	ec = ep->ec;
	Doc			doc = ep->doc;
	char *xml;
	xmlChar	*xmlout;
	xmlDocPtr	xmldoc;
	int	len;
	Link	l, lstr;

	if(ti == NULL || doc==NULL) return;
	xml = (char *)edb_table_make_xml(ec, ti);

	xmldoc = xmlParseMemory(xml, xmlStrlen((xmlChar *)xml));
	if(xmldoc==NULL) return;
#if 1
	xmlDocDumpFormatMemoryEnc(xmldoc, &xmlout, &len, "utf-8", 1);
#else
	xmlDocDumpMemoryEnc(xmldoc, &xmlout, &len, "utf-8");
#endif
	lstr = str_split((char *)xmlout, '\n');
	ForLink(l, lstr) {
		char *s = (char *)l->d;

		while(*s==' ') {
			if(DocIsHTML(doc)) {
				doc_puts(doc, "&nbsp;&nbsp;&nbsp;&nbsp;");
			} else {
				doc_puts(doc, "    ");
			}
			++s;
		}
		if(DocIsHTML(doc)) {
			s = (char *)xml_text_encode(ec, (xmlChar *)s);
			doc_puts(doc, s);
			FREE(s);
			doc_puts(doc, "<br />\n");
		} else {
			doc_puts(doc, s);
			doc_puts(doc, "\n");
		}
	}
	link_free(lstr);

	xmlFreeDoc(xmldoc);
	FREE(xml);
}

static
void
edb_print_table_definition_column_permission(EdbPrint ep, int perm, int mode)
{
	EdbContext	ec = ep->ec;
	Doc			doc = ep->doc;
	char			*bgc = "#ffffff";

	if(perm==EDB_AUTH_PUBLIC) {
		bgc = "#80ffff";
	} else if(perm==EDB_AUTH_UNIVERSITY) {
		bgc = "#ccffff";
	}
	doc_printf(doc, "<td align=\"center\" style=\"background-color:%s; font-size:80%%;\">", bgc);
	doc_printf(doc, "(%s)", (char *)edb_auth_string(ec, perm, mode));
	doc_puts(doc, "</td>\n");
}

static
int
edb_print_table_definition_column_func(EdbContext ec, EdbTableInfo ti, int layer, EdbColumnInfo ci, EdbPrint ep)
{
	register Doc		doc = ep->doc;

	if(ci==NULL) {
		doc_puts(doc, "<tr>\n");
		doc_printf(doc, "<th style=\"font-size:80%%;\">%s</th>\n", "名前");
		doc_printf(doc, "<th style=\"font-size:80%%;\">%s</th>\n", "XML<br />SQL");
		doc_printf(doc, "<th style=\"font-size:80%%;\">%s</th>\n", "説明");
		doc_printf(doc, "<th style=\"font-size:80%%;\">%s</th>\n", "型");
		doc_printf(doc, "<th style=\"font-size:80%%;\">%s</th>\n", "番");
		doc_printf(doc, "<th style=\"font-size:80%%;\">%s</th>\n", "数");
		doc_printf(doc, "<th style=\"font-size:80%%;\">%s</th>\n", "MAP");
		doc_printf(doc, "<th style=\"font-size:80%%;\">%s</th>\n", "権限<br />委譲");
		doc_printf(doc, "<th style=\"font-size:80%%;\">%s</th>\n", "分類");
		doc_printf(doc, "<th style=\"font-size:80%%;\">%s</th>\n", "同義");
		doc_printf(doc, "<th style=\"font-size:80%%;\">%s</th>\n", "期間<br />限定");
		doc_printf(doc, "<th style=\"font-size:80%%;\">%s</th>\n", "用法");
		doc_printf(doc, "<th style=\"font-size:80%%;\">%s</th>\n", "該当<br />無可");
		doc_printf(doc, "<th style=\"font-size:80%%;\">%s</th>\n", "読権");
		doc_printf(doc, "<th style=\"font-size:80%%;\">%s</th>\n", "作権");
		doc_printf(doc, "<th style=\"font-size:80%%;\">%s</th>\n", "書権");
		doc_printf(doc, "<th style=\"font-size:80%%;\">%s</th>\n", "削権");
		doc_printf(doc, "<th style=\"font-size:80%%;\">%s</th>\n", "見");
		doc_printf(doc, "<th style=\"font-size:80%%;\">%s</th>\n", "順");
		doc_printf(doc, "<th style=\"font-size:80%%;\">%s</th>\n", "巾");
		doc_puts(doc, "</tr>\n");
	} else {
		int	i;
		Link	lm;

		if(ci->obsoleted) {
			doc_puts(doc, "<tr align=\"left\" bgcolor=\"#404040\">\n");
		} else {
			doc_puts(doc, "<tr align=\"left\">\n");
		}
		doc_puts(doc, "<td style=\"font-size:80%;\">");
		for(i=0; i<layer; ++i) doc_puts(doc, "&nbsp;&nbsp;");
		doc_text_puts3(doc, NULL, (char *)ColumnName_ja(ci), "<br />");
		for(i=0; i<layer; ++i) doc_puts(doc, "&nbsp;&nbsp;");
		doc_text_puts3(doc, NULL, (char *)ColumnName_en(ci), "</td>\n");
		doc_puts(doc, "<td style=\"font-size:80%;\">");
		if(!strncmp((char *)ci->xmlname, (char *)ci->ti->xmlname, strlen((char *)ci->ti->xmlname))) {
			doc_text_puts3(doc, "@", (char *)&ci->xmlname[strlen((char *)ci->ti->xmlname)], "<br />\n");
		} else {
			doc_text_puts3(doc, "<td>", (char *)ci->xmlname, "<br />\n");
		}
		doc_text_puts3(doc, NULL, (char *)ColumnSN(ci), "</td>\n");

		doc_text_puts3(doc, "<td style=\"font-size:80%;\">", (char *)ColumnDesc_ja(ci), "<br />\n");
		doc_text_puts3(doc, NULL, (char *)ColumnDesc_en(ci), "</td>\n");
		if(ci->typeInfo) {
			char	*type = ci->typeInfo->name;

			doc_printf(doc, "<td align=\"center\" style=\"font-size:80%%;\">");
			edb_datum_linkto_type_description(ec, ci->typeInfo, doc);
			doc_printf(doc, "</td>\n");
			doc_printf(doc, "<td align=\"right\" style=\"font-size:80%%;\">%d</td>\n", ci->idx);
		} else {
			doc_printf(doc, "<td style=\"font-size:80%%;\">&nbsp;</td>\n");
			doc_printf(doc, "<td style=\"font-size:80%%;\">&nbsp;</td>\n");
		}

#define	TD_ATTENTION	"<td align=\"center\" style=\"background-color:#ffff00;font-size:80%%;\">"
#define	TD_NORMAL		"<td align=\"center\" style=\"font-size:80%%;\">"

		if(ci->elements==1) {
			doc_printf(doc, TD_NORMAL);
			doc_printf(doc, "%d", ci->elements);
			doc_puts(doc, "</td>\n");
		} else if(ci->elements>1) {
			doc_printf(doc, TD_ATTENTION);
			doc_printf(doc, "%d", ci->elements);
			doc_puts(doc, "</td>\n");
		} else {
			doc_printf(doc, TD_ATTENTION);
			doc_puts(doc, "*");
			doc_puts(doc, "</td>\n");
		}

		doc_puts(doc, "<td align=\"center\" style=\"font-size:80%;\">");
		if(ci->l_maplookup) {
			ForLink(lm, ci->l_maplookup) {
				EdbMapLookup	map = (EdbMapLookup)lm->d;
				eid_t	meid;
				EdbTableInfo	mti;

				meid = edb_maplookup_table_eid(ec, map);
				if(EIDisVALID(meid)) doc_printf(doc, "<a href=\"/cgi-bin/edb_browse?EID=%d\">", meid);
				mti = edb_table_get(ec, meid);
				doc_text_puts(doc, (char *)TABLE_Name_ja(mti));
				if(EIDisVALID(meid)) doc_printf(doc, "</a>");
				doc_puts(doc, "<br />");
			}
		} else {
			doc_puts(doc, "&nbsp;");
		}
		doc_puts(doc, "</td>\n");

		if(ci->authorize) {
			doc_printf(doc, TD_ATTENTION "◎</td>\n");
		} else {
			doc_printf(doc, TD_NORMAL "&nbsp;</td>\n");
		}

		if(ci->classify==CLASSIFY_PRIMARY) {
			doc_printf(doc, TD_ATTENTION);
			doc_puts(doc, "◎");
			doc_puts(doc, "</td>\n");
		} else if(ci->classify==CLASSIFY_SECONDARY) {
			doc_printf(doc, TD_ATTENTION);
			doc_puts(doc, "○");
			doc_puts(doc, "</td>\n");
		} else {
			doc_printf(doc, TD_NORMAL);
			doc_puts(doc, "&nbsp;");
			doc_puts(doc, "</td>\n");
		}

		if(ci->alias) {
			doc_printf(doc, TD_ATTENTION);
			doc_puts(doc, "○");
			doc_puts(doc, "</td>\n");
		} else {
			doc_printf(doc, TD_NORMAL);
			doc_puts(doc, "&nbsp;");
			doc_puts(doc, "</td>\n");
		}

		if(ci->chronological) {
			doc_printf(doc, TD_ATTENTION);
			doc_puts(doc, "○");
			doc_puts(doc, "</td>\n");
		} else {
			doc_printf(doc, TD_NORMAL);
			doc_puts(doc, "&nbsp;");
			doc_puts(doc, "</td>\n");
		}

		if(ci->conductor) {
			doc_printf(doc, TD_ATTENTION);
			doc_puts(doc, "[指揮]");
			doc_puts(doc, "</td>\n");
		} else if(ColumnConductorUsage(ci)==USAGE_REQUIRED) {
			doc_printf(doc, TD_ATTENTION);
			doc_puts(doc, "[必須]");
			doc_puts(doc, "</td>\n");
		} else if(ColumnConductorUsage(ci)==USAGE_PREFERRED) {
			doc_printf(doc, TD_ATTENTION);
			doc_puts(doc, "[推奨]");
			doc_puts(doc, "</td>\n");
		} else if(ColumnConductorUsage(ci)==USAGE_OPTIONAL) {
			doc_printf(doc, TD_ATTENTION);
			doc_puts(doc, "[任意]");
			doc_puts(doc, "</td>\n");
		} else if(ColumnConductorUsage(ci)==USAGE_UNUSED) {
			doc_printf(doc, TD_ATTENTION);
			doc_puts(doc, "[不用]");
			doc_puts(doc, "</td>\n");
		} else if(ColumnUsage(ci)==USAGE_REQUIRED) {
			doc_printf(doc, TD_ATTENTION);
			doc_puts(doc, "必須");
			doc_puts(doc, "</td>\n");
		} else if(ColumnUsage(ci)==USAGE_PREFERRED) {
			doc_printf(doc, TD_ATTENTION);
			doc_puts(doc, "推奨");
			doc_puts(doc, "</td>\n");
		} else if(ColumnUsage(ci)==USAGE_OPTIONAL) {
			doc_printf(doc, TD_ATTENTION);
			doc_puts(doc, "任意");
			doc_puts(doc, "</td>\n");
		} else if(ColumnUsage(ci)==USAGE_UNUSED) {
			doc_printf(doc, TD_ATTENTION);
			doc_puts(doc, "不用");
			doc_puts(doc, "</td>\n");
		} else {
			doc_printf(doc, TD_NORMAL);
			doc_puts(doc, "&nbsp;");
			doc_puts(doc, "</td>\n");
		}

		if(ColumnCanbeNothing(ci)) {
			doc_printf(doc, TD_ATTENTION);
			doc_puts(doc, "可");
			doc_puts(doc, "</td>\n");
		} else {
			doc_printf(doc, TD_NORMAL);
			doc_puts(doc, "&nbsp;");
			doc_puts(doc, "</td>\n");
		}

		if(ColumnTupleRead(ci)==EDB_AUTH_INHERIT) {
			edb_print_table_definition_column_permission(ep, TABLE_TupleRead(ti), TRUE);
		} else {
			edb_print_table_definition_column_permission(ep, ColumnTupleRead(ci), FALSE);
		}

		if(ColumnTupleCreate(ci)==EDB_AUTH_INHERIT) {
			edb_print_table_definition_column_permission(ep, TABLE_TupleCreate(ti), TRUE);
		} else {
			edb_print_table_definition_column_permission(ep, ColumnTupleCreate(ci), FALSE);
		}

		if(ColumnTupleWrite(ci)==EDB_AUTH_INHERIT) {
			edb_print_table_definition_column_permission(ep, TABLE_TupleWrite(ti), TRUE);
		} else {
			edb_print_table_definition_column_permission(ep, ColumnTupleWrite(ci), FALSE);
		}

		if(ColumnTupleDelete(ci)==EDB_AUTH_INHERIT) {
			edb_print_table_definition_column_permission(ep, TABLE_TupleDelete(ti), TRUE);
		} else {
			edb_print_table_definition_column_permission(ep, ColumnTupleDelete(ci), FALSE);
		}

		if(ci->caption) {
			doc_printf(doc, TD_ATTENTION "%s</td>\n", edb_caption_string(ec, ci->caption, FALSE));
		} else {
			doc_printf(doc, TD_NORMAL "%s</td>\n", edb_caption_string(ec, ci->caption, FALSE));
		}
		if(ci->order) {
			doc_printf(doc, TD_ATTENTION "%d</td>\n", ci->order);
		} else {
			doc_printf(doc, TD_NORMAL "%d</td>\n", ci->order);
		}
		doc_printf(doc, "<td align=\"right\" style=\"font-size:80%%;\">%d</td>\n", ci->size);
		doc_puts(doc, "</tr>\n");
	}
	return TRUE;
}
static
int
edb_print_table_definition_column(EdbContext ec, EdbTableInfo ti, int layer, EdbColumnInfo ci, EdbTableCBData cbd)
{

	return edb_print_table_definition_column_func(ec, ti, layer, ci, (EdbPrint)cbd->user_data);
}

#define	STRbeVALID(s)	((s) ? (s) : "")

static
int
edb_print_table_definition_column_explain(EdbContext ec, EdbTableInfo ti, int layer, EdbColumnInfo ci, EdbTableCBData cbd)
{
	register EdbPrint	ep = (EdbPrint)cbd->user_data;
	register Doc		doc = ep->doc;

	doc_text_puts3(doc, "<li>", (char *)ColumnName_ja(ci), ": ");
	doc_text_puts3(doc, "(英) ", (char *)ColumnExplain_en(ci), " ");
	doc_text_puts3(doc, "(日) ", (char *)ColumnExplain_ja(ci), " </li>\n");
	return TRUE;
}

static
int
edb_print_table_mapped_from(EdbContext ec, EdbTableInfo ti, int layer, EdbColumnInfo ci, EdbTableCBData cbd)
{
	register EdbPrint	ep = (EdbPrint)cbd->user_data;
	register Doc		doc = ep->doc;
	register EdbTableInfo	to_ti = EpUserData(ep);
	register Link		lm;

	ForLink(lm, ci->l_maplookup) {
		EdbMapLookup	map = lm->d;

		if(TABLE_EID(to_ti)==edb_maplookup_table_eid(ec, map)) {
			doc_printf(doc, "<li>【<a href=\"/cgi-bin/edb_browse?EID=%d&D_CMD=DESCRIPTION\">", TABLE_EID(ti));
			doc_text_puts3(doc, "", (char *)TABLE_Name_ja(ti), "</a>】");
			doc_text_puts3(doc, ".[", (char *)ColumnName_ja(ci), "]</li>\n");
		}
	}
	return TRUE;
}

void
edb_print_table_definition(EdbPrint ep, EdbTableInfo ti)
{
	EdbContext	ec = ep->ec;
	Doc	doc = ep->doc;
	Link	lm;

	doc_puts(doc, "<span style=\"font-size:80%\">\n");
	doc_puts(doc, "<ul>\n");
	doc_printf(doc, "<li>XML: %s (<a href=\"%s%s.xml\">XML</a>, <a href=\"%s%s.dtd\">DTD</a>)</li>\n",
		TABLE_XN(ti), EDB_DTD_URI, TABLE_XN(ti), EDB_DTD_URI, TABLE_XN(ti));
	doc_printf(doc, "<li>SQL: %s</li>\n", TABLE_SN(ti));
	doc_puts(doc, "<li>名前: \n");
	doc_printf(doc, "（英）%s\n", (char *)TABLE_Name_en(ti));
	doc_printf(doc, "（日）%s\n", (char *)TABLE_Name_ja(ti));
	doc_printf(doc, "（読）%s\n", (char *)TABLE_Name_pr(ti));
	doc_puts(doc, "</li>\n");
	doc_puts(doc, "<li>説明: \n");
	doc_printf(doc, "（英）%s\n", (char *)TABLE_Desc_en(ti));
	doc_printf(doc, "（日）%s\n", (char *)TABLE_Desc_ja(ti));
	doc_puts(doc, "</li>\n");
	doc_puts(doc, "<li>解説: \n");
	doc_printf(doc, "（英）%s\n", (char *)TABLE_Explain_en(ti));
	doc_printf(doc, "（日）%s\n", (char *)TABLE_Explain_ja(ti));
	doc_puts(doc, "</li>\n");
	doc_puts(doc, "<li>マップ: \n");
	ForLink(lm, ti->l_maplookup) {
		EdbMapLookup	map = (EdbMapLookup)lm->d;
		eid_t	meid;
		EdbTableInfo mti;

		meid = edb_maplookup_table_eid(ec, map);
		if(EIDisVALID(meid)) doc_printf(doc, "<a href=\"/cgi-bin/edb_browse?EID=%d\">", meid);
		mti = edb_table_get(ec, meid);
		doc_text_puts(doc, (char *)TABLE_Name_ja(mti));
		if(EIDisVALID(meid)) doc_printf(doc, "</a>");
		doc_puts(doc, ", ");
	}
	doc_puts(doc, "</li>\n");
	doc_puts(doc, "<li>属性: \n");
	if(ti->auxiliary) doc_text_puts(doc, "(補助テーブル)");
	if(ti->hierarchical) doc_text_puts(doc, "(階層)");
	if(ti->censorship) doc_text_puts(doc, "(検閲)");
	if(ti->chronological) doc_text_puts(doc, "(期間指定)");
	doc_puts(doc, "</li>\n");
	if(edb_table_has_conductor(ec, ti)) {
		Link	l;
		doc_puts(doc, "<li>指揮:\n");
		ForLink(l, edb_table_get_conductors(ec, ti)) {
			EdbColumnInfo	ci = l->d;
			doc_text_puts(doc, (char *)ColumnName_ja(ci));
			doc_text_puts3(doc, " (", (char *)ColumnXN(ci), ")");
			doc_puts(doc, (l->n!=NULL) ? ", " : ".");
		}
		doc_puts(doc, "</li>\n");
	}
	doc_puts(doc, "<li>タプル権限(既定値): \n");
	doc_printf(doc, "読権: %s, 作権: %s, 書権: %s, 削権: %s.",
		edb_auth_string(ec, TABLE_TupleRead(ti), TRUE),
		edb_auth_string(ec, TABLE_TupleCreate(ti), TRUE),
		edb_auth_string(ec, TABLE_TupleWrite(ti), TRUE),
		edb_auth_string(ec, TABLE_TupleDelete(ti), TRUE));
	doc_puts(doc, "</li>\n");
	doc_puts(doc, "<li>カラム権限(既定値): \n");
	doc_printf(doc, "読権: %s, 作権: %s, 書権: %s, 削権: %s.",
		edb_auth_string(ec, TABLE_ColumnRead(ti), TRUE),
		edb_auth_string(ec, TABLE_ColumnCreate(ti), TRUE),
		edb_auth_string(ec, TABLE_ColumnWrite(ti), TRUE),
		edb_auth_string(ec, TABLE_ColumnDelete(ti), TRUE));
	doc_puts(doc, "</li>\n");
	doc_puts(doc, "<li>登録されているカラム\n");
	doc_puts(doc, "<table border=\"2\" cellspacing=\"0\">\n");
	edb_print_table_definition_column_func(ec, ti, 0, NULL, ep);
	edb_table_column_callback(ec, ti, NULL, edb_print_table_definition_column, ep);
	doc_puts(doc, "</table>\n");
	doc_puts(doc, "</li>\n");
	doc_puts(doc, "<li>カラムの解説文:\n");
	doc_puts(doc, "<ul>\n");
	edb_table_column_callback(ec, ti, NULL, edb_print_table_definition_column_explain, ep);
	doc_puts(doc, "</ul>\n");
	doc_puts(doc, "</li>\n");

	doc_puts(doc, "<li>参照しているカラム:\n");
	doc_puts(doc, "<ul>\n");
	{
		Link	l, l_tables = edb_table_get_all_tables(ec);

		edb_print_push(ep);
		EpUserData(ep) = ti;
		ForLink(l, l_tables) edb_table_column_callback(ec, l->d, NULL, edb_print_table_mapped_from, ep);
		link_free(l_tables);
		edb_print_pop(ep);
	}
	doc_puts(doc, "</ul>\n");
	doc_puts(doc, "</li>\n");

	doc_puts(doc, "</ul>\n");
	doc_puts(doc, "</span>\n");
	doc_puts(doc, "<br />\n");
}


