/*
	$Id: print_tcd.c,v 1.20 2012/04/03 03:51:40 alex Exp alex $
	print_tcd.c--- print TCD
*/

#ifndef	lint
static char *rcsid __attribute__((unused)) = "$Id: print_tcd.c,v 1.20 2012/04/03 03:51:40 alex Exp alex $";
#endif

#include	<stdio.h>
#include	<stdarg.h>
#include	<string.h>
#include	"edb/edb.h"
#include	"edb/tex.h"
#include	"edb/publish.h"

static
int
compare_print_tcd_info(register BiTree_obj_t o1, register BiTree_obj_t o2)
{

	return strcmp(((struct EDB_PRINT_TCD_INFO *)o1)->xmlname, ((struct EDB_PRINT_TCD_INFO *)o2)->xmlname);
}

static
int
edb_print_tcd_duplicate_info_callback(BiTree_obj_t obj, void *data)
{
	register struct EDB_PRINT_TCD_INFO	*src = obj;
	register BiTree						*new_bt = data;
	register struct EDB_PRINT_TCD_INFO	*dst;

	dst = malloc(sizeof(struct EDB_PRINT_TCD_INFO));
	bcopy(src, dst, sizeof(struct EDB_PRINT_TCD_INFO));
	dst->xmlname = strdup(src->xmlname);
	BiTreeAdd(*new_bt, dst, bitree_hint_str(dst->xmlname), compare_print_tcd_info, NULL, NULL);
	return TRUE;
}

BiTree
edb_print_tcd_duplicate_info(BiTree bt)
{
	BiTree	new = NULL;

	bitree_callback(bt, edb_print_tcd_duplicate_info_callback, &new);
	return new;
}

int
edb_print_tcd_register_func(EdbPrint ep, char *xmlname, int style, EdbPrintTCDFunc_t print_tcd)
{
	struct EDB_PRINT_TUPLE_INFO *table_info;
	struct EDB_PRINT_TCD_INFO *tcd_info, tcd_info1;
	char	*xmlname0 = NULL, *p;
	EdbTableInfo	ti;
	EdbColumnInfo	ci;

	if(style<0 || EP_PrintStyle_Variations<=style) return FALSE;
	xmlname0 = STRDUP(xmlname);
	p = strchr(xmlname0, '.');
	if(p) *p = '\0';
	ti = edb_table_get_by_name(ep->ec, (utf8_t *)xmlname0);
	if(ti==NULL) return FALSE;
	ci = edb_table_seek_column_by_xmlname(ep->ec, ti, (utf8_t *)xmlname);
	if(ci==NULL) return FALSE;
	if(!ep->ct->print_tuple_tobe_free) {
		BiTree	bt;

		bt = ep->ct->bt_print_tuple_info;
		ep->ct->bt_print_tuple_info = edb_print_tuple_duplicate_info(bt);
		ep->ct->print_tuple_tobe_free = TRUE;
	}
	table_info = bitree_search(ep->ct->bt_print_tuple_info, NULL, TABLE_EID(ti), NULL);
	if(table_info==NULL) {
		table_info = malloc(sizeof(struct EDB_PRINT_TUPLE_INFO));
		if(table_info==NULL) return FALSE;
		bzero(table_info, sizeof(struct EDB_PRINT_TUPLE_INFO));
#if 0
		table_info->ti = ti;
		BiTreeAdd(ep->ct->bt_print_tuple_info, table_info, TABLE_EID(table_info->ti), NULL, NULL, NULL);
#else
		table_info->teid = TABLE_EID(ti);
		BiTreeAdd(ep->ct->bt_print_tuple_info, table_info, table_info->teid, NULL, NULL, NULL);
#endif
	}
	tcd_info1.xmlname = STRDUP(xmlname);
	tcd_info = bitree_search(table_info->bt_print_tcd_info, &tcd_info1, bitree_hint_str(xmlname), compare_print_tcd_info);
	if(tcd_info==NULL) {
		tcd_info = malloc(sizeof(struct EDB_PRINT_TCD_INFO));
		if(tcd_info==NULL) return FALSE;
		bzero(tcd_info, sizeof(struct EDB_PRINT_TCD_INFO));
		tcd_info->xmlname = strdup(xmlname);
		BiTreeAdd(table_info->bt_print_tcd_info, tcd_info, bitree_hint_str(xmlname), compare_print_tcd_info, NULL, NULL);
	}
	tcd_info->func[style] = print_tcd;
	return TRUE;
}

static
int
edb_print_tcd_default_caption(EdbPrint ep, EdbTupleInfo tuple, int recurse, Link	l_ci)
{
	register Doc	doc = ep->doc;
	int	refed = FALSE;
	Link	l_tc;
	EdbContext	ec = ep->ec;
	int	japanese = LangIsJapanese(ep);
	int	status;

	if(tuple == NULL) return edb_print_alert(ep, EP_ALERT, "!!!");
	if(TupleIsPerson(tuple)) {
		edb_book_print_person_info_name(ep, tuple, japanese, ep->ct->tcd_mode);
		return TRUE;
	}
#if defined(EDB_MANAGE_DNS)
	if(TupleIsInetDomain(tuple) || TupleIsInetNetwork(tuple) || TupleIsInetHost(tuple)) {
		Caption	cap;

		cap = edb_tuple_make_caption(ec, tuple, 0);
		doc_puts(doc, (char *)cap->c_main_english);
		edb_caption_free(ec, cap);
		return TRUE;
	}
#endif
	if(japanese && edb_table_is_hierarchical(ec, TupleTable(tuple)) && recurse>0) {
		eid_t	up_eid;

		up_eid = edb_tuple_hierarchyparent(ec, tuple);
		if(EIDisVALID(up_eid) && !edb_caption_is_omitted(ec, up_eid)) {
			EdbTupleInfo	up_tuple;

			up_tuple = edb_tuple_get(ec, up_eid, 0);
			if(up_tuple) {
				edb_print_tcd_default_caption(ep, up_tuple, recurse-1, NULL);
				doc_puts(doc, japanese ? " " : ", ");
			}
		}
	}
	ForLink(l_tc, tuple->l_column) {
		EdbTupleColumn	tc = (EdbTupleColumn)l_tc->d;
		EdbTupleInfo	rtuple;
		EdbDatum	d;
		int	docap = FALSE;

		if(l_ci==NULL) {
			if(tc->ci->caption==EDB_CAPTION_SHORT) docap = TRUE;
		} else {
			Link	l;

			ForLink(l, l_ci) {
				EdbColumnInfo	ci = l->d;

				if(!strcmp((char *)ColumnXN(tc->ci), (char *)ColumnXN(ci))) docap = TRUE;
				if(docap) break;
			}
		}
		if(!docap) continue;

		ForTCDatum(d, tc) {
			switch(tc->ci->typeInfo->basicTypeID) {
			case EDB_TYPE_EID:
				rtuple = edb_tuple_get(ec, DatumEID(d), 0);
				if(rtuple) {
					Link	lm, l_mci = NULL;

					if(refed) doc_puts(doc, "/");
					ForLink(lm, tc->ci->l_maplookup) {
						EdbMapLookup	map = lm->d;

						if(TABLE_EID(TupleTable(rtuple))==edb_maplookup_table_eid(ec, map)) {
							l_mci = map->l_caption_ci;
							break;
						}
					}
					edb_print_tcd_default_caption(ep, rtuple, recurse-1, l_mci);
					refed = TRUE;
				}
				break;
			case EDB_TYPE_TEXT:
				if(DatumEnglishIsUsable(d)) {
					doc_text_puts(doc, (char *)DatumEnglish(d));
				} else {
					edb_print_alert_column(ep, 0, tc->ci);
				}
				break;
			case EDB_TYPE_NAME:
			case EDB_TYPE_SENTENCE:
				ITEMIZE(ep) {
					if(japanese) {
						if(DatumJapaneseIsUsable(d)) {
							doc_text_puts(doc, (char *)DatumJapanese(d));
						} else {
							edb_print_alert_column(ep, 0, tc->ci);
						}
					} else {	/* English */
						if(DatumEnglishIsUsable(d)) {
							doc_text_puts(doc, (char *)DatumEnglish(d));
						} else {
							edb_print_alert_column(ep, 0, tc->ci);
						}
					}
				} else {
					if(japanese) {
						if(DatumJapaneseIsUsable(d)) {
							doc_text_puts(doc, (char *)DatumJapanese(d));
						} else if(DatumEnglishIsUsable(d)) {
							doc_text_puts(doc, (char *)DatumEnglish(d));
						} else {
							edb_print_alert_column(ep, 0, tc->ci);
						}
					} else {	/* English */
						if(DatumEnglishIsUsable(d)) {
							doc_text_puts(doc, (char *)DatumEnglish(d));
						} else if(DatumJapaneseIsUsable(d)) {
							if(str_isascii((char *)DatumJapanese(d))) {
								doc_text_puts(doc, (char *)DatumJapanese(d));
							} else {
								if(!ep->ct->nocolor) doc_color_begin(ep->doc, WARNING_COLOR);
								if(ep->ct->persist_lang && DocIsTeX(doc)) doc_puts(doc, "\\EdbAlert{");
								doc_text_puts(doc, (char *)DatumJapanese(d));
								if(ep->ct->persist_lang && DocIsTeX(doc)) doc_puts(doc, "}");
								if(!ep->ct->nocolor) doc_color_end(ep->doc);
							}
						} else {
							edb_print_alert_column(ep, 0, tc->ci);
						}
					}
				}
				break;
			default:
				if(japanese) {
					if(DatumJapaneseIsUsable(d)) {
						doc_text_puts(doc, (char *)DatumJapanese(d));
					} else if(DatumEnglishIsUsable(d)) {
						doc_text_puts(doc, (char *)DatumEnglish(d));
					} else {
						edb_print_alert_column(ep, 0, tc->ci);
					}
				} else {	/* English */
					if(DatumEnglishIsUsable(d)) {
						doc_text_puts(doc, (char *)DatumEnglish(d));
					} else if(DatumJapaneseIsUsable(d)) {
						if(str_isascii((char *)DatumJapanese(d))) {
							doc_text_puts(doc, (char *)DatumJapanese(d));
						} else {
							if(!ep->ct->nocolor) doc_color_begin(ep->doc, WARNING_COLOR);
							if(ep->ct->persist_lang && DocIsTeX(doc)) doc_puts(doc, "\\EdbAlert{");
							doc_text_puts(doc, (char *)DatumJapanese(d));
							if(ep->ct->persist_lang && DocIsTeX(doc)) doc_puts(doc, "}");
							if(!ep->ct->nocolor) doc_color_end(ep->doc);
						}
					} else {
						edb_print_alert_column(ep, 0, tc->ci);
					}
				}
				break;
			}
			break;
		}
	}
	if(!japanese && edb_table_is_hierarchical(ec, TupleTable(tuple)) && recurse>0) {
		eid_t	up_eid;

		up_eid = edb_tuple_hierarchyparent(ec, tuple);
		if(EIDisVALID(up_eid) && !edb_caption_is_omitted(ec, up_eid)) {
			EdbTupleInfo	up_tuple;

			up_tuple = edb_tuple_get(ec, up_eid, 0);
			if(up_tuple) {
				doc_puts(doc, japanese ? " " : ", ");
				edb_print_tcd_default_caption(ep, up_tuple, recurse-1, NULL);
			}
		}
	}

	return TRUE;
}

static
int
edb_print_tcd_has_data(EdbContext ec, EdbDatum datum)
{
	Link	l;

	if(datum==NULL) return FALSE;
	if(!DatumAvail(datum)) return FALSE;
	if(DatumEIDisValid(datum)
		 || DatumEnglishIsUsable(datum)
		 || DatumJapaneseIsUsable(datum)) {
		return TRUE;
	}
	return FALSE;
}

int
edb_print_tcd_prefix(EdbPrint ep, EdbTupleInfo tuple, EdbTupleColumn tc, EdbDatum datum)
{
	register Doc	doc = ep->doc;
	register EdbColumnInfo	ci = tc->ci;

	LaTeX(ep) return FALSE;
	if(DocIsXML(doc)) return FALSE;
	HTML(doc) {
		if(ep->ct->tcd_mode&EP_TCD_MODE_BOLD) doc_puts(doc, "<b>");
		if(ep->ct->tcd_mode&EP_TCD_MODE_ITALIC) doc_puts(doc, "<i>");
	}
	if(LangIsJapanese(ep)) {
		doc_text_puts(doc, (char *)ColumnPrefix_ja(ci));
	} else {
		doc_text_puts(doc, (char *)ColumnPrefix_en(ci));
	}
	return TRUE;
}

int
edb_print_tcd_postfix(EdbPrint ep, EdbTupleInfo tuple, EdbTupleColumn tc, EdbDatum datum)
{
	register Doc	doc = ep->doc;
	register EdbColumnInfo	ci = tc->ci;

	LaTeX(ep) return FALSE;
	if(DocIsXML(doc)) return FALSE;
	if(LangIsJapanese(ep)) {
		doc_text_puts(doc, (char *)ColumnSuffix_ja(ci));
	} else {
		doc_text_puts(doc, (char *)ColumnSuffix_en(ci));
	}
	HTML(doc) {
		if(ep->ct->tcd_mode&EP_TCD_MODE_ITALIC) doc_puts(doc, "</i>");
		if(ep->ct->tcd_mode&EP_TCD_MODE_BOLD) doc_puts(doc, "</b>");
	}
	return TRUE;
}

static
int
edb_print_tcd_text(EdbPrint ep, EdbTupleInfo tuple, EdbTupleColumn tc, EdbDatum datum)
{
	EdbContext	ec = ep->ec;
	Doc	doc = ep->doc;
	struct EDB_PRINT_TUPLE_INFO	*tuple_info;
	register EdbPrintTCDFunc_t		print;
	register EdbTableInfo		ti;
	register int	style = PrintStyle(ep);
	int	status = FALSE;

	if(style<0 || EP_PrintStyle_Variations<=style) return FALSE;

	if(tuple == NULL) return edb_print_alert(ep, EP_ALERT, "!!!");
	if(tc == NULL) return edb_print_alert(ep, EP_ALERT, "???");
	if(!edb_print_tcd_has_data(ep->ec, datum)) return edb_print_alert_column(ep, 0, tc->ci);
	ti = TupleTable(tuple);
	if(ti==NULL) return FALSE;

	print = NULL;
	tuple_info = bitree_search(ep->ct->bt_print_tuple_info, NULL, TABLE_EID(ti), NULL);
	if(tuple_info) {
		struct EDB_PRINT_TCD_INFO		*tcd_info, tcd_info1;

		tcd_info1.xmlname = (char *)ColumnXN(tc->ci);
		tcd_info = bitree_search(tuple_info->bt_print_tcd_info, &tcd_info1, bitree_hint_str(tcd_info1.xmlname), compare_print_tcd_info);
		if(tcd_info) print = tcd_info->func[style];
	}
	if(print) {
		status = (*print)(ep, tuple, tc, datum);
	} else if(DatumEIDisValid(datum)) {
		EdbTupleInfo	ref_tuple = edb_tuple_get(ep->ec, DatumEID(datum), 0);
		int	underline = FALSE;

		edb_print_tcd_prefix(ep, tuple, tc, datum);
		if(!ep->ct->nounderline && edb_print_eoi_check(ep, DatumEID(datum))) underline = TRUE;
		if(underline) NotLaTeX(ep) doc_underline_begin(ep->doc);
		if(ref_tuple) {
			Link	lm, l_ci = NULL;

			ForLink(lm, tc->ci->l_maplookup) {
				EdbMapLookup	map = lm->d;

				if(TABLE_EID(TupleTable(ref_tuple))==edb_maplookup_table_eid(ec, map)) {
					l_ci = map->l_caption_ci;
					break;
				}
			}
			status = edb_print_tcd_default_caption(ep, ref_tuple, (ep->ct->tcd_mode&EP_TCD_MODE_NORECURSION) ? 0 : EP_RECURSION_DEPTH, l_ci);
		} else {
			status = edb_print_alert_column(ep, 0, tc->ci);
		}
		if(underline) NotLaTeX(ep) doc_underline_end(ep->doc);
		edb_print_tcd_postfix(ep, tuple, tc, datum);
	} else {
		status = edb_print_type(ep, tuple, tc, datum);
	}
	return status;
}

int
edb_print_tcd(EdbPrint ep, EdbTupleInfo tuple, EdbTupleColumn tc, EdbDatum datum)
{
	register Doc	doc = ep->doc;
	int	status = FALSE;
	char	*cn = NULL;
	char	*bgn=NULL, *end = NULL, *fmt = NULL;

	if(STRisVALID(ep->ct->tcd_idx_prefix)) {
		asprintf(&cn, "%s:%s", ColumnXN(tc->ci), ep->ct->tcd_idx_prefix);
	} else {
		asprintf(&cn, "%s", ColumnXN(tc->ci));
	}

	ITEMIZE(ep) {
		LaTeX(ep) {
			asprintf(&bgn, "\\EdbColDef{%s:%s}{%d}", EP_LaTeX_PREFIX, cn, ep->ct->tcd_idx);
			fmt = "%s{%s}{";
			end = strdup("}%\n");
		} else {
			asprintf(&bgn, "%s:%d", cn, ep->ct->tcd_idx);
			fmt = "%s:%s\t";
			end = strdup("\n");
		}
	}

	ITEMIZE(ep) {
		if(STRisVALID(fmt) && datum && EIDisVALID(DatumEID(datum))) {
			doc_printf(doc, fmt, bgn, "@");
			doc_printf(doc, "%d", DatumEID(datum));
			doc_puts(doc, end);
		}
		edb_print_push(ep);
		EpLang(ep) = LangIsJapanese(ep) ? EP_LANG_English : EP_LANG_Japanese;
		if(STRisVALID(fmt)) doc_printf(doc, fmt, bgn, LangIsJapanese(ep)?"J":"E");
		status |= edb_print_tcd_text(ep, tuple, tc, datum);
		if(STRisVALID(end)) doc_puts(doc, end);
		edb_print_pop(ep);
	}

	if(STRisVALID(fmt)) doc_printf(doc, fmt, bgn, LangIsJapanese(ep)?"J":"E");
	status |= edb_print_tcd_text(ep, tuple, tc, datum);
	if(STRisVALID(end)) doc_puts(doc, end);

	FREE(bgn);
	FREE(end);
	FREE(cn);
	ITEMIZE(ep) return TRUE;
	return status;
}


