/*
$Id$
	survey.c--- statistics
*/

#ifndef lint
static char *rcsid __attribute__((unused)) = "$Id$";
#endif
#include	<stdio.h>
#include	<stdlib.h>
#include	<math.h>
#include	<stdarg.h>
#include	<unistd.h>
#include	<sys/types.h>
#include	<ctype.h>
#include	<string.h>
#include	<signal.h>
#include	"edb/edb.h"
#include	"edb/html.h"
#include	"edb/publish.h"
#include	"edb/javascript.h"

#define	MY_ORGAN_EID	ORGAN_UNIV_EID
#define	TABLE_FONT_SIZE	"-2"

#define	SHOW_HISTORY

#define	THIS_YEAR(ec)	DATE_Y((ec)->present_date)

#define	DEBUG
#undef	DEBUG

#define	PORTRAIT_WIDTH	192
#define	PORTRAIT_HEIGHT	256

#define	PORTRAIT_ICON_WIDTH	48
#define	PORTRAIT_ICON_HEIGHT	64

#define	FOREIGN_EXTENSION		"+ 81 - "

static	EdbPrint	tex_ep = NULL;

static	EdbTableInfo	table_organization = NULL;
static	EdbTableInfo	table_person = NULL;
static	EdbTableInfo	table_article = NULL;
static	EdbTableInfo	table_patent = NULL;
static	EdbTableInfo	table_history = NULL;
static	EdbTableInfo	table_lecture = NULL;
static	EdbTableInfo	table_study = NULL;
static	EdbTableInfo	table_asa = NULL;
static	EdbTableInfo	table_sa = NULL;

static
struct {
	EdbTableInfo	*table;
	char	*xmlname;
} preload_tables[] = {
{	&table_organization,	"organization",				},
{	&table_person,		"person",					},
{	&table_article,		"article",					},
{	&table_patent,		"patent",					},
{	&table_history,		"history",					},
{	&table_lecture,		"lecture",					},
{	&table_study,		"study",					},
{	&table_asa,		"academicsocietyactivity",	},
{	&table_sa,			"socialactivity",				},
};

#define	WORKS_BY_ORGANIZATION_PAGE	"works-o.html"
#define	WORKS_BY_MEMBER_PAGE	"works-m.html"
#define	WORKS_BY_CSV_PAGE	"works.csv"
#define	PERSON_CSV	"person.csv"

static
int
edb_survey_print_tuple_postfix(EdbPrint ep, EdbTupleInfo tuple)
{
	register Doc	doc = ep->doc;

	CSV(ep) return FALSE;
	LaTeX(ep) {
		doc_printf(doc, "\\end{%s:%s}%%\n", EP_LaTeX_PREFIX, TABLE_XN(TupleTable(tuple)));
	}
	HTML(doc) {
		JavaScript_printf(doc, "edb_ref_by_eid(%d);", TupleEID(tuple));
		return TRUE;
	}
	return FALSE;
}

static
Link
get_eid_list(EdbContext ec, EdbTableInfo ti, char *constraint)
{
	char	*query;
	char	*order;
	int	n;
	Link	l_eid;

	order = (char *)edb_table_make_order(ec, ti, (utf8_t *)"t1");
	asprintf(&query,
		"SELECT %s FROM %s AS t1 WHERE %s %s %s %s %s",
		EDB_BASE_EID,
		TABLE_SN(ti),
		EDB_BASE_AVAILABLE,
		constraint ? "AND" : "",
		constraint ? constraint : "",
		STRisVALID(order) ? "ORDER BY" : "", order);
	FREE(order);
	l_eid = edb_pgsql_query_one_field(ec, (utf8_t *)query, &n);
	FREE(query);
	return l_eid;
}

#define	WWW_SURVEY	"/survey"
#define	WWW_PATH	WEB_DOCUMENT_ROOT WWW_SURVEY
static	char	*surveyname = "survey";
static	char	*www_path = WWW_PATH;
#define	DATE_MIN	0
#define	DATE_MAX	99999999
static	int	works_date_from = DATE_MIN;
static	int	works_date_to = DATE_MAX;
static	char	*works_date_string = "";

static
void
survey_begin_HTML(EdbContext ec, char *title, Doc doc)
{

	doc_puts(doc, "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n");
	doc_puts(doc, "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n");
	doc_puts(doc, "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"ja\" lang=\"ja\">\n");
	doc_puts(doc, "	<head>\n");
	doc_puts(doc, "		<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />\n");
	doc_puts(doc, "		<meta http-equiv=\"Content-Style-Type\" content=\"text/css; charset=utf-8\" />\n");
	doc_puts(doc, "		<meta http-equiv=\"Pragma\" content=\"no-cache\" />\n");
	doc_puts(doc, "		<meta http-equiv=\"Cache-Control\" content=\"no-cache\" />\n");
	doc_puts(doc, "		<meta http-equiv=\"Expires\" content=\"Thu, 01 Dec 1994 16:00:00 GMT\" />\n");
	doc_text_puts3(doc, "		<title>", title, "</title>\n");
	doc_puts(doc, "	</head>\n");
	doc_puts(doc, "<body text=\"black\" bgcolor=\"white\">\n");
	doc_puts(doc, "<br />\n");

	doc_puts(doc, "<a name=\"page.top\">\n");
	JavaScript_printf(doc, 
"function\n"
"edb_ref_by_eid(eid)\n"
"{\n"
"	var	doc = self.window.document;\n"
"\n"
"	doc.write('<input type=\"button\" value=\"" NAME_OF_DB "\" onclick=\"window.open(');\n"
"	doc.write(\"'%s%s?EID=\"+eid);\n"
"	doc.write(\"', '_blank');\");\n"
"	doc.writeln('\" style=\"font-size: smaller\">');\n"
"	return true;\n"
"}", WEB_SERVER_URL, EDB_BROWSER_URL_DEFAULT);

	JavaScript_printf(doc,
"function\n"
"moveto_anchor(v)\n"
"{\n"
"\n"
"	if(v) window.location.hash = v;\n"
"	return true;\n"
"}");

	JavaScript_printf(doc,
"function\n"
"moveto_selection(sel)\n"
"{\n"
"\n"
"	if(sel.options[sel.selectedIndex].value) moveto_anchor(sel.options[sel.selectedIndex].value);\n"
"	sel.selectedIndex = 0;\n"
"	return true;\n"
"}");

	doc_puts(doc, "</a>\n");
	doc_puts(doc, "<blockquote>\n");
}


#define	RETURN_TO_PAGE_TOP	"<input type=\"button\" value=\"戻る\" title=\"ページの先頭へ移動\" onclick=\"moveto_anchor('page.top');\" style=\"font-size: smaller\">"

static
void
survey_end_HTML(EdbContext ec, Doc doc)
{

	doc_puts(doc, "<hr /><span style=\"font-size:smaller\">\n");
	doc_text_puts(doc, "本頁に記載されている内容は『");
	doc_printf(doc, "<a href=\"%s\" target=\"_blank\">", WEB_PORTAL_URL);
	doc_text_puts(doc, NAME_OF_DATABASE " (" NAME_OF_DB ")");
	doc_puts(doc, "</a>");
	doc_text_puts(doc, " 』の登録情報をもとに作成されています．");
	doc_puts(doc, "</span><br />\n");
	html_print_postscript(ec, doc->fp);
	doc_puts(doc, "</blockquote>\n");
	doc_puts(doc, "</body>\n");
	doc_puts(doc, "</html>\n");
}

struct SURVEY_PAGE {
	char			*name;
	char			*title;
	char			*desc;
	EdbPrint		ep;
	EdbPrint		csv_ep;
};
typedef	struct SURVEY_PAGE	*SurveyPage;

static
struct SURVEY_PAGE	organization_pages = {
	"index.html",
	"Organization",
	NULL,
};

static
struct SURVEY_PAGE	person_index_pages = {
	"person-idx.html",
	"Index of Person",
	NULL,
};

static
struct SURVEY_PAGE	person_pages = {
	"index.html",
	"Person",
	NULL,
};

struct SURVEY_WORK_CATALOGUE_LIST {
	char			*tablename;
	eid_t			kind;
	char			*anchor;
	char			*caption;
	char			*th_cap;
	char			*tex_cmd;
} catalogue_list[] = {{
	"article",	10442,	"article.book",		"Book",							"Book",					"著書"
},{
	"article",	10443,	"article.paper1",		"Academic Paper (Judged Full Paper)",	"Paper<br />(judged)",		"論文J"
},{
	"article",	60752,	"article.paper2",		"Academic Paper (Unrefereed Paper)",	"Paper<br />(unref.)",		"論文U"
},{
	"article",	10444,	"article.letter",		"Academic Letter",					"Letter",					"レター"
},{
	"article",	10445,	"article.review",	"Review, Commentary",				"Review,<br />comm.",		"総説解説"
},{
	"article",	10446,	"article.intlconf",	"Proceeding of International Conference",	"Proc.<br />int'l<br />conf.",	"国際会議"
},{
	"article",	20015,	"article.domeconf",	"Proceeding of Domestic Conference"	,	"Proc.<br />dome.<br />conf.",	"国内会議"
},{
	"article",	60746,	"article.product",	"Products",						"Pro-<br />ducts",			"作品"
},{
	"article",	10447,	"article.workshop",	"Et cetra, Workshop",					"Etc,<br />work-<br />shop",	"研究会"
},{
	"article",	60753,	"article.report",		"Report",							"Rep.",					"報告書"
},{
	"patent",	10463,	"patent.patent",		"Patent",							"Pat.",					"特許"
},{
	"patent",	10464,	"patent.p-model",	"Practical Model",					"Prac.<br />model",			"実用新案"
},{
	"patent",	373934,	"patent.design",		"Design",							"Design"	,				"意匠"
}};

struct SURVEY_WORK {
	EdbCatalogue	ca[Ns(catalogue_list)];
};
typedef	struct SURVEY_WORK	*SurveyWork;

static
SurveyWork
survey_work_new(int alloc_ca)
{
	SurveyWork	sw;
	int	i;

	sw = (SurveyWork)malloc(sizeof(struct SURVEY_WORK));
	bzero(sw, sizeof(struct SURVEY_WORK));
	if(alloc_ca) {
		for(i=0; i<Ns(catalogue_list); ++i) sw->ca[i] = edb_catalogue_new();
	}
	return sw;
}

static
void
survey_work_free(SurveyWork sw)
{
	int	i;

	for(i=0; i<Ns(catalogue_list); ++i) edb_catalogue_free(sw->ca[i]);
	free(sw);
}

static
void
survey_catalogue_merge(EdbCatalogue dst, EdbCatalogue src)
{
	register Link	l;
	register EdbTupleInfo	tuple;

	ForCatalogueLink(l, src) {
		tuple = CatalogueObject(l);
		if(tuple) edb_catalogue_append(dst, TupleEID(tuple), tuple);
	}
}

static
void
survey_work_merge(SurveyWork dst, SurveyWork src)
{
	int	i;

	for(i=0; i<Ns(catalogue_list); ++i) survey_catalogue_merge(dst->ca[i], src->ca[i]);
}

static
void
survey_print_work_summary(char *prefix, SurveyWork sw, char *postfix, char *refpage, EdbPrint ep)
{
	char	*fmt = "";
	int	i;

	if(PrintStyleIsCSV(ep)) {
		if(STRisVALID(prefix)) doc_printf(ep->doc, "%s", prefix);
		if(sw==NULL) {
			for(i=0; i<Ns(catalogue_list); ++i) doc_printf(ep->doc, "\t%s", catalogue_list[i].caption);
		} else {
			for(i=0; i<Ns(catalogue_list); ++i) {
				doc_printf(ep->doc, "\t%d", edb_catalogue_size(sw->ca[i]));
			}
		}
		if(STRisVALID(postfix)) doc_printf(ep->doc, "\t%s", postfix);
		doc_puts(ep->doc, "\n");
	} else {
		if(STRisVALID(prefix)) doc_printf(ep->doc, "%s\n", prefix);
		if(sw==NULL) {
			fmt = "<th style=\"font-size:90%%;\">%s</th>\n";
			for(i=0; i<Ns(catalogue_list); ++i) doc_printf(ep->doc, fmt, catalogue_list[i].th_cap);
		} else {
			for(i=0; i<Ns(catalogue_list); ++i) {
				if(refpage && edb_catalogue_size(sw->ca[i])>0) {
					doc_printf(ep->doc, "<td><a href=\"%s#%s\">", refpage, catalogue_list[i].anchor);
					doc_printf(ep->doc, "%d", edb_catalogue_size(sw->ca[i]));
					doc_printf(ep->doc, "</a></td>\n");
				} else {
					doc_printf(ep->doc, "<td>%d</td>\n", edb_catalogue_size(sw->ca[i]));
				}
			}
		}
		if(STRisVALID(postfix)) doc_printf(ep->doc, "%s\n", postfix);
	}
}

static	EdbCatalogue	ca_sw_organization = NULL;
static	EdbCatalogue	ca_sw_person = NULL;

static	SurveyWork	survey_organization_page	__P((EdbContext, SurveyPage, eid_t, int));

static
EdbCatalogue
catalogue_get(EdbContext ec, EdbTableInfo table, char *constraint)
{
	register EdbCatalogue	ca;
	register Link	l, l_eid;

	l_eid = get_eid_list(ec, table, constraint);
	ca = edb_catalogue_new();
	ForLink(l, l_eid) {
		register eid_t	eid = ATOI(l->d);
		register EdbTupleInfo	tuple = NULL;

		if(!EIDisVALID(eid)) continue;
		tuple = edb_tuple_get(ec, eid, 0);
		if(tuple) edb_catalogue_append(ca, TupleEID(tuple), tuple);
	}
	link_free(l_eid);
	return ca;
}

static
Link
survey_organization_page_get_child_organ(EdbContext ec, EdbTupleInfo organ)
{
	EdbTupleInfo	child;
	EdbColumnInfo	ci;
	char	*constraint;
	Link	l, l_eid;
	Link	l_child;

	ci = edb_table_seek_column(ec, TupleTable(organ), (utf8_t *)"@.upper");
	if(ci==NULL) return NULL;
	asprintf(&constraint, "%s*=%d", ColumnSN(ci), TupleEID(organ));
	l_eid = get_eid_list(ec, TupleTable(organ), constraint);
	FREE(constraint);

	l_child = NULL;
	for(l=l_eid; l; l=l->n) {
		eid_t	parent_eid;

		child = edb_tuple_get(ec, ATOI(l->d), 0);
		if(child==NULL) continue;
		parent_eid = edb_tuple_hierarchyparent(ec, child);
		if(parent_eid != TupleEID(organ)) continue;
		LinkAppend(l_child, link_atom_new(child, NULL));
	}
	link_free(l_eid);
	return l_child;
}

static
void
tuple_expand_sub(EdbContext ec, EdbCatalogue ca, EdbTupleInfo tuple)
{
	Link	l;
	EdbTupleInfo	rtuple;

	if(tuple==NULL) return;
	if(edb_catalogue_lookup(ca, TupleEID(tuple))) return;
	edb_catalogue_append(ca, TupleEID(tuple), tuple);

	edb_tuple_expand_check(ec, tuple);
	ForLink(l, tuple->expand_map) tuple_expand_sub(ec, ca, (EdbTupleInfo)l->d);
	if(EIDisVALID(TupleMapto(tuple))) {
		rtuple = edb_tuple_get(ec, TupleMapto(tuple), 0);
		if(rtuple) tuple_expand_sub(ec, ca, rtuple);
	}
}

static
EdbCatalogue
tuple_expand(EdbContext ec, EdbTupleInfo tuple)
{
	EdbCatalogue	ca;

	ca = edb_catalogue_new();
	tuple_expand_sub(ec, ca, tuple);
	return ca;
}

static
EdbCatalogue
survey_get_person_history(EdbContext ec, eid_t person, eid_t kind)
{
	EdbCatalogue	ca = NULL;
	char	*condition = NULL;

	condition = (char *)edb_condition_make(ec, table_history, CONDITION_LOGIC_AND,
		ConditionEIDMatch("@.subject", person),
		ConditionEIDMatch("@.attribute", kind),
		ConditionTerminate);
	ca = catalogue_get(ec, table_history, condition);
	FREE(condition);
	return ca;
}

static
EdbCatalogue
survey_get_person_lecture(EdbContext ec, eid_t person)
{
	EdbCatalogue	ca = NULL;
	char	*condition = NULL;

	condition = (char *)edb_condition_make(ec, table_lecture, CONDITION_LOGIC_AND,
		ConditionEIDMatch("@.lecturer", person),
		ConditionIntegerEQ("@.entryyear", THIS_YEAR(ec)),
		ConditionTerminate);
	ca = catalogue_get(ec, table_lecture, condition);
	FREE(condition);
	return ca;
}

static
EdbCatalogue
survey_get_person_study(EdbContext ec, eid_t person)
{
	EdbCatalogue	ca = NULL;
	char	*condition = NULL;

	condition = (char *)edb_condition_make(ec, table_study, CONDITION_LOGIC_AND,
		ConditionEIDMatch("@.researcher", person),
		ConditionTerminate);
	ca = catalogue_get(ec, table_study, condition);
	FREE(condition);
	return ca;
}

static
EdbCatalogue
survey_get_person_asa(EdbContext ec, eid_t person)
{
	EdbCatalogue	ca = NULL;
	char	*condition = NULL;

	condition = (char *)edb_condition_make(ec, table_asa, CONDITION_LOGIC_AND,
		ConditionEIDMatch("@.person", person),
		ConditionTerminate);
	ca = catalogue_get(ec, table_asa, condition);
	FREE(condition);
	return ca;
}

static
EdbCatalogue
survey_get_person_sa(EdbContext ec, eid_t person)
{
	EdbCatalogue	ca = NULL;
	char	*condition = NULL;

	condition = (char *)edb_condition_make(ec, table_sa, CONDITION_LOGIC_AND,
		ConditionEIDMatch("@.person", person),
		ConditionTerminate);
	ca = catalogue_get(ec, table_sa, condition);
	FREE(condition);
	return ca;
}

static
EdbCatalogue
survey_get_person_article_by_kind(EdbContext ec, eid_t person, eid_t kind)
{
	char	*condition = NULL;

	if(works_date_from>DATE_MIN || works_date_to<DATE_MAX) {
		condition = (char *)edb_condition_make(ec, table_article, CONDITION_LOGIC_AND,
			ConditionEIDMatch("@.author", person),
			ConditionEIDMatch("@.kind", kind),
			(works_date_from>DATE_MIN) ? ConditionNoOperation : ConditionSkipNext,
			ConditionDateFrom("@.date", works_date_from),
			(works_date_to<DATE_MAX) ? ConditionNoOperation : ConditionSkipNext,
			ConditionIntegerLE("@.date", works_date_to),
			ConditionTerminate);
	} else {
		condition = (char *)edb_condition_make(ec, table_article, CONDITION_LOGIC_AND,
			ConditionEIDMatch("@.author", person),
			ConditionEIDMatch("@.kind", kind),
			ConditionTerminate);
	}
	return catalogue_get(ec, table_article, condition);
}

static
EdbCatalogue
survey_get_person_patent_by_kind(EdbContext ec, eid_t person, eid_t kind)
{
	char	*condition = NULL;

	if(works_date_from>DATE_MIN || works_date_to<DATE_MAX) {
		condition = (char *)edb_condition_make(ec, table_patent, CONDITION_LOGIC_AND,
			ConditionEIDMatch("@.inventor", person),
			ConditionEIDMatch("@.kind", kind),
			(works_date_from>DATE_MIN) ? ConditionNoOperation : ConditionSkipNext,
			ConditionDateFrom("@.date", works_date_from),
			(works_date_to<DATE_MAX) ? ConditionNoOperation : ConditionSkipNext,
			ConditionIntegerLE("@.date", works_date_to),
			ConditionTerminate);
	} else {
		condition = (char *)edb_condition_make(ec, table_patent, CONDITION_LOGIC_AND,
			ConditionEIDMatch("@.inventor", person),
			ConditionEIDMatch("@.kind", kind),
			ConditionTerminate);
	}
	return catalogue_get(ec, table_patent, condition);
}

static
SurveyWork
survey_get_person_works(EdbContext ec, eid_t person)
{
	SurveyWork	sw;
	int	i;

	sw = survey_work_new(FALSE);
	for(i=0; i<Ns(catalogue_list); ++i) {
		if(!strcmp(catalogue_list[i].tablename, "article")) {
			sw->ca[i] = survey_get_person_article_by_kind(ec, person, catalogue_list[i].kind);
		} else if(!strcmp(catalogue_list[i].tablename, "patent")) {
			sw->ca[i] = survey_get_person_patent_by_kind(ec, person, catalogue_list[i].kind);
		}
	}
	return sw;
}

static
int
survey_work_print_all_by_kind(EdbContext ec, EdbCatalogue ca, EdbCatalogue ca_order, int tex, EdbPrint ep)
{
	int	status = FALSE;

	if(edb_catalogue_size(ca)>0) {
		Link	l;

		doc_puts(ep->doc, "<ol style=\"font-size:90%;\">\n");
		ep->ct->tuple_tag_prefix = "<li>";
		ep->ct->tuple_tag_postfix = "</li>";
		if(tex) {
			tex_ep->ct->tuple_tag_prefix = "\\item ";
			tex_ep->ct->tuple_tag_postfix = "";
		}
		if(ca_order==NULL || edb_catalogue_size(ca)==1) {
			ForCatalogueLink(l, ca) {
				EdbTupleInfo	tuple = CatalogueObject(l);

				edb_print_tuple_caption(ep, tuple, NULL);
				if(tex) edb_print_tuple_caption(tex_ep, tuple, NULL);
			}
		} else {
			ForCatalogueLink(l, ca_order) {
				EdbTupleInfo	tuple = CatalogueObject(l);

				if((tuple=(EdbTupleInfo)edb_catalogue_lookup(ca, TupleEID(tuple)))!=NULL) {
					edb_print_tuple_caption(ep, tuple, NULL);
					if(tex) edb_print_tuple_caption(tex_ep, tuple, NULL);
				}
			}
		}
		doc_puts(ep->doc, "</ol>\n");
		status = TRUE;
	}
	return status;
}

static
int
survey_work_print_all(EdbContext ec, SurveyWork sw, int reorder, int tex, EdbPrint ep)
{
	int	status = FALSE;
	int	i;

	edb_print_push(ep);
	EpLang(ep) = EP_LANG_Auto;
	for(i=0; i<Ns(catalogue_list); ++i) {
		EdbCatalogue	ca_order = NULL;

		if(edb_catalogue_size(sw->ca[i])==0) continue;
		doc_printf(ep->doc, "<dt><a name=\"%s\">%s</a>:</dt>\n<dd>", catalogue_list[i].anchor, catalogue_list[i].caption);
		if(tex) doc_printf(tex_ep->doc, "\\定義{%s}{\n", catalogue_list[i].tex_cmd);
		if(reorder) {
			if(!strcmp(catalogue_list[i].tablename, "article")) {
				ca_order = table_article->ca_all_tuples;
			} else if(!strcmp(catalogue_list[i].tablename, "patent")) {
				ca_order = table_patent->ca_all_tuples;
			}
		}
		status |= survey_work_print_all_by_kind(ec, sw->ca[i], ca_order, tex, ep);
		doc_puts(ep->doc, "</dd>\n");
		if(tex) doc_puts(tex_ep->doc, "}\n");
	}
	edb_print_pop(ep);
	return status;
}

static
char *
survey_person_portrait_name(EdbContext ec, EdbTupleInfo person)
{
	char	*pname = NULL;
	EdbTupleColumn	tc;
	EdbDatum	d;

	tc = edb_tuple_seek_column(ec, person, (utf8_t *)"@.portrait");
	if(tc && (d=tc->datum) && EIDisVALID(d->d_eid)) {
		EdbTupleInfo	ptuple;

		ptuple = edb_tuple_get(ec, d->d_eid, 0);
		if(ptuple) {
			EdbPicture	picture;

			picture = edb_picture_new(ec, ptuple);
			if(picture) {
				if(!strcmp(picture->mimetype, "image/jpeg")) {
					pname = "portrait.jpg";
				} else if(!strcmp(picture->mimetype, "image/png")) {
					pname = "portrait.png";
				}
				edb_picture_free(ec, picture);
			}
		}
	}
	return pname;
}

#define	ICON_BROKEN_PORTRAIT	"icons/broken-portrait.png"

static
void
survey_person_print_name(EdbContext ec, EdbTupleInfo person, int no, EdbPrint ep)
{
	Doc	doc = ep->doc;
	EdbTupleColumn	tc;
	EdbDatum			d, d1, d2, d3;

	tc = edb_tuple_seek_column(ec, person, (utf8_t *)"@.surname");
	if(tc && tc->datum) {
		d1 = tc->datum;
	} else {
		d1 = NULL;
	}
	tc = edb_tuple_seek_column(ec, person, (utf8_t *)"@.middlename");
	if(tc && tc->datum) {
		d2 = tc->datum;
	} else {
		d2 = NULL;
	}
	tc = edb_tuple_seek_column(ec, person, (utf8_t *)"@.givenname");
	if(tc && tc->datum) {
		d3 = tc->datum;
	} else {
		d3 = NULL;
	}
	doc_puts(doc, "<tr align=\"left\">");
	doc_printf(doc, no==0 ? "<td>氏名:</td><td><b>" : "<td>氏名%d:</td><td><b>", no+1);
	if(DatumJapaneseIsValid(d1)) {
		doc_text_puts(doc, (char *)DatumJapanese(d1));
		doc_printf(tex_ep->doc, no==0 ? "\\定義{姓日}{" : "\\定義{姓日%d}{", no+1);
		doc_text_puts(tex_ep->doc, (char *)DatumJapanese(d1));
		doc_puts(tex_ep->doc, "}\n");
	}
	doc_puts(doc, "&nbsp;");
	if(DatumJapaneseIsValid(d2)) {
		doc_text_puts(doc, (char *)DatumJapanese(d2));
		doc_printf(tex_ep->doc, no==0 ? "\\定義{ミ日}{" : "\\定義{ミ日%d}{", no+1);
		doc_text_puts(tex_ep->doc, (char *)DatumJapanese(d2));
		doc_puts(tex_ep->doc, "}\n");
	}
	doc_puts(doc, "&nbsp;");
	if(DatumJapaneseIsValid(d3)) {
		doc_text_puts(doc, (char *)DatumJapanese(d3));
		doc_printf(tex_ep->doc, no==0 ? "\\定義{名日}{" : "\\定義{名日%d}{", no+1);
		doc_text_puts(tex_ep->doc, (char *)DatumJapanese(d3));
		doc_puts(tex_ep->doc, "}\n");
	}
	doc_puts(doc, "&nbsp;");

	doc_puts(doc, "(");
	if(DatumPronounceIsValid(d1)) {
		doc_text_puts(doc, (char *)DatumPronounce(d1));
		doc_printf(tex_ep->doc, no==0 ? "\\定義{姓読}{" : "\\定義{姓読%d}{", no+1);
		doc_text_puts(tex_ep->doc, (char *)DatumPronounce(d1));
		doc_puts(tex_ep->doc, "}\n");
	}
	doc_puts(doc, "&nbsp;");
	if(DatumPronounceIsValid(d2)) {
		doc_text_puts(doc, (char *)DatumPronounce(d2));
		doc_printf(tex_ep->doc, no==0 ? "\\定義{ミ読}{" : "\\定義{ミ読%d}{", no+1);
		doc_text_puts(tex_ep->doc, (char *)DatumPronounce(d2));
		doc_puts(tex_ep->doc, "}\n");
	}
	doc_puts(doc, "&nbsp;");
	if(DatumPronounceIsValid(d3)) {
		doc_text_puts(doc, (char *)DatumPronounce(d3));
		doc_printf(tex_ep->doc, no==0 ? "\\定義{名読}{" : "\\定義{名読%d}{", no+1);
		doc_text_puts(tex_ep->doc, (char *)DatumPronounce(d3));
		doc_puts(tex_ep->doc, "}\n");
	}
	doc_puts(doc, ")");
	doc_puts(doc, "</b></td></tr>\n");

	doc_puts(doc, "<tr align=\"left\">");
	doc_printf(doc, no==0 ? "<td>Name:</td><td><b>" : "<td>Name%d:</td><td><b>", no+1);
	if(DatumEnglishIsValid(d3)) {
		doc_text_puts(doc, (char *)DatumEnglish(d3));
		doc_printf(tex_ep->doc, no==0 ? "\\定義{名英}{" : "\\定義{名英%d}{", no+1);
		doc_text_puts(tex_ep->doc, (char *)DatumEnglish(d3));
		doc_puts(tex_ep->doc, "}\n");
	}
	doc_puts(doc, "&nbsp;");
	if(DatumEnglishIsValid(d2)) {
		doc_text_puts(doc, (char *)DatumEnglish(d2));
		doc_printf(tex_ep->doc, no==0 ? "\\定義{ミ英}{" : "\\定義{ミ英%d}{", no+1);
		doc_text_puts(tex_ep->doc, (char *)DatumEnglish(d2));
		doc_puts(tex_ep->doc, "}\n");
	}
	doc_puts(doc, "&nbsp;");
	if(DatumEnglishIsValid(d1)) {
		doc_text_puts(doc, (char *)DatumEnglish(d1));
		doc_printf(tex_ep->doc, no==0 ? "\\定義{姓英}{" : "\\定義{姓英%d}{", no+1);
		doc_text_puts(tex_ep->doc, (char *)DatumEnglish(d1));
		doc_puts(tex_ep->doc, "}\n");
	}
	doc_puts(doc, "</b></td></tr>\n");
}

static
int
print_date(EdbPrint ep, EdbTupleInfo tuple, EdbDatum super_datum)
{
	register int	status;

	if(super_datum==NULL) return FALSE;
	status = edb_print_tuple_named_column(ep, tuple, super_datum, " (", "", "@.date", NULL, ", ", ")", "", EP_ALERT);
	return status;
}

static
int
print_period(EdbPrint ep, EdbTupleInfo tuple, EdbDatum super_datum)
{
	register int	status;

	if(super_datum==NULL) return FALSE;
	status = edb_print_tuple_named_column(ep, tuple, super_datum, " [", "", "@.period", NULL, ", ", "]", "", EP_ALERT);
	return status;
}

static
int
print_lecture(EdbContext ec, EdbPrint ep, EdbTupleInfo lecture)
{
	EdbTupleColumn	tc;

	edb_print_tuple_named_column(ep, lecture, NULL, "", "", "@.name", NULL, ", ", "", NULL, EP_ALERT);
	edb_print_tuple_named_column(ep, lecture, NULL, " (", "", "@.course", NULL, ", ", ")", "", 0);
	return TRUE;
}

static
int
print_study(EdbContext ec, EdbPrint ep, EdbTupleInfo study)
{

	edb_print_tuple_named_column(ep, study, NULL, "", "", "@.field", NULL, ", ", "", "", EP_ALERT);
	edb_print_tuple_named_column(ep, study, NULL, " (", "", "@.theme", NULL, ", ", ")", "", 0);
	return TRUE;
}

static
int
print_asa(EdbContext ec, EdbPrint ep, EdbTupleInfo asa)
{

	edb_print_tuple_named_column(ep, asa, NULL, "", "", "@.academicsociety", NULL, ", ", "", NULL, EP_ALERT);
	edb_print_tuple_named_column(ep, asa, NULL, " (", "", "@.post", print_period, ", ", ")", "", 0);
	return TRUE;
}

static
int
print_sa(EdbContext ec, EdbPrint ep, EdbTupleInfo sa)
{

	edb_print_tuple_named_column(ep, sa, NULL, "", "", "@.organization", NULL, ", ", "", NULL, EP_ALERT);
	edb_print_tuple_named_column(ep, sa, NULL, " (", "", "@.post", print_period, ", ", ")", "", 0);
	return TRUE;
}

static
SurveyWork
survey_person_page(EdbContext ec, SurveyPage page, eid_t eid, Doc pdoc)
{
	Doc doc;
	char	name[BUFSIZ];
	char	tmpname[BUFSIZ];
	char	pngname[BUFSIZ];
	Link	l;
	int	i;
	EdbTupleInfo	person;
	Caption	cap;
	char	*dir, *title;
	EdbTupleColumn	tc;
	EdbDatum			d;
	EdbCatalogue	ca;
	int	status;
	EdbPrint	ep;
	char	*portrait_name = "../../" ICON_BROKEN_PORTRAIT;
	SurveyWork	sw;
	EdbCatalogue	ca_person;
	int	no;

	sw = (SurveyWork)edb_catalogue_lookup(ca_sw_person, eid);
	if(sw) return sw;

	person = edb_tuple_get(ec, eid, 0);
	if(person==NULL) return NULL;

	if(!TupleIsPerson(person)) return NULL;

	ca_person = tuple_expand(ec, person);
	sw = survey_get_person_works(ec, TupleEID(person));
	edb_catalogue_append(ca_sw_person, eid, sw);

	asprintf(&dir, "%s/%s/%d", www_path, TABLE_XN(TupleTable(person)), eid);
	makedir_hierarchy(dir);
	sprintf(name, "%s/index.html", dir);
	sprintf(tmpname, "%s.tmp", name);
	doc = doc_open(DocML_HTML, tmpname, "w+");
	if(doc==NULL) return NULL;

	ep = edb_print_new(ec, doc, PrintStyle_STANDARD);
	ep->ct->noalert = TRUE;
	ep->ct->nocolor = TRUE;
	EpLang(ep) = EP_LANG_Japanese;
	edb_print_tuple_postfix_register_func(ep, PrintStyle(ep), edb_survey_print_tuple_postfix);
	edb_print_tuple_postfix_register_func(ep, EP_PrintStyle_CAPTION, edb_survey_print_tuple_postfix);
	ForCatalogueLink(l, ca_person) edb_print_eoi_add(ep, TupleEID((EdbTupleInfo)CatalogueObject(l)));

	cap = edb_make_caption(ec, eid, 0);
	asprintf(&title, "%s: %s", page->title, cap->c_main);
	edb_caption_free(ec, cap);
	survey_begin_HTML(ec, title, doc);
	FREE(title);

	doc_puts(tex_ep->doc, "\\begin{個人プロフィール}\n");

	tc = edb_tuple_seek_column(ec, person, (utf8_t *)"@.portrait");
	if(tc && (d=tc->datum) && EIDisVALID(d->d_eid)) {
		EdbTupleInfo	ptuple;

		ptuple = edb_tuple_get(ec, d->d_eid, 0);
		if(ptuple) {
			EdbPicture	picture;

			picture = edb_picture_new(ec, ptuple);
			if(picture) {
				if(!strcmp(picture->mimetype, "image/jpeg")) {
					portrait_name = "portrait.jpg";
				} else if(!strcmp(picture->mimetype, "image/png")) {
					portrait_name = "portrait.png";
				}
				if(portrait_name) {
					FILE	*fp;
					char	*picpath_tmp = NULL, *picpath = NULL;

					asprintf(&picpath, "%s/%s", dir, portrait_name);
					asprintf(&picpath_tmp, "%s.tmp", picpath);
					fp = fopen(picpath_tmp, "w");
					if(fp) {
						fwrite(picture->body, sizeof(char), picture->bytes, fp);
						fclose(fp);
					}
					file_update(picpath_tmp, picpath);
					FREE(picpath_tmp);
				}
				edb_picture_free(ec, picture);
			}
		}
	}
	doc_printf(doc, "<img align=\"right\" src=\"%s\" alt=\"Portrait\" width=\"%d\" height=\"%d\" />", portrait_name, PORTRAIT_WIDTH, PORTRAIT_HEIGHT);
	doc_puts(doc, "<h1>");
	edb_print_tuple_named_column(ep, person, NULL, "", "", "@.title", NULL, ", ", " : ", "", EP_ALERT);
	cap = edb_make_caption(ec, eid, 0);
	doc_text_puts(doc, (char *)cap->c_main);
	JavaScript_printf(doc, "edb_ref_by_eid(%d);", TupleEID(person));
	doc_puts(doc, "</h1>\n");
	edb_caption_free(ec, cap);
	doc_puts(doc, "<hr />\n");
	if(page->desc) {
		doc_printf(doc, "%s\n", page->desc);
		doc_puts(doc, "<hr />\n");
	}

	doc_puts(doc, "<table border=\"0\" cellspacing=\"0\">\n");
	no = 0;
	ForCatalogueLink(l, ca_person) survey_person_print_name(ec, CatalogueObject(l), no++, ep);

	doc_puts(doc, "<tr align=\"left\"><td>肩書:</td><td>");
	edb_print_tuple_named_column(ep, person, NULL, "", "", "@.title", NULL, ", ", "", "( <i>not registered</i> )", EP_ALERT);
	doc_puts(doc, "</td></tr>\n");

	doc_puts(doc, "<tr align=\"left\"><td>学位:</td><td>");
	edb_print_tuple_named_column(ep, person, NULL, "", "", "@.degree", NULL, ", ", "", "( <i>not registered</i> )", EP_ALERT);
	doc_puts(doc, "</td></tr>\n");

	edb_caption_omit_save(ec);
	edb_caption_omit_clear(ec);
	doc_puts(doc, "<tr align=\"left\"><td>出身大学:</td><td>");
	edb_print_tuple_named_column(ep, person, NULL, "", "", "@.almamater", print_date, ", ", "", "( <i>not registered</i> )", EP_WARNING);
	doc_puts(doc, "</td></tr>\n");

	doc_puts(doc, "<tr align=\"left\"><td>出身大学院(修士):</td><td>");
	edb_print_tuple_named_column(ep, person, NULL, "", "", "@.almamater2", print_date, ", ", "", "( <i>not registered</i> )", EP_WARNING);
	doc_puts(doc, "</td></tr>\n");

	doc_puts(doc, "<tr align=\"left\"><td>出身大学院(博士):</td><td>");
	edb_print_tuple_named_column(ep, person, NULL, "", "", "@.almamater3", print_date, ", ", "", "( <i>not registered</i> )", EP_WARNING);
	doc_puts(doc, "</td></tr>\n");
	edb_caption_omit_restore(ec);

	doc_puts(doc, "<tr align=\"left\"><td>性別:</td><td>");
	edb_print_tuple_named_column(ep, person, NULL, "", "", "@.sex", NULL, ", ", "", "( <i>not registered</i> )", EP_WARNING);
	doc_puts(doc, "</td></tr>\n");

#if 0
	doc_puts(doc, "<tr align=\"left\"><td>生年月:</td><td>");
	tc = edb_tuple_seek_column(ec, person, (utf8_t *)"@.birthday");
	if(tc && (d=tc->datum) && DatumEnglishIsValid(d)) {
		int	ymd = ATOI((char *)DatumEnglish(d));
		int	m;
		static char	*mname[] = {
			"January", "February", "March", "April", "May", "June",
			"July", "August", "September", "October", "November", "December"
		};

		doc_puts(doc, "<b>");
		doc_puts(tex_ep->doc, "\\定義{生年月日}{");
		doc_text_printf(tex_ep->doc, "%d年%d月%d日", DATE_Y(ymd), DATE_M(ymd), DATE_D(ymd));
		doc_puts(tex_ep->doc, "}\n");
		m = DATE_M(ymd);
		if(1<=m && m<=12) {
			doc_printf(doc, "%s, ", mname[m-1]);
		}
		doc_printf(doc, "%d.", DATE_Y(ymd));
		doc_puts(doc, "</b>");
	} else {
		doc_puts(doc, "( <i>not registered</i> )");
	}
	doc_puts(doc, "</td></tr>\n");
#endif

	doc_puts(doc, "<tr align=\"left\"><td>E-mail:</td><td>");
	tc = edb_tuple_seek_column(ec, person, (utf8_t *)"@.email");
	if(tc && (d=tc->datum) && DatumEnglishIsValid(d)) {
		char	*s = (char *)DatumEnglish(d);

		doc_text_puts3(tex_ep->doc, "\\定義{電子メール}{", (char *)DatumEnglish(d), "}\n");
		edb_print_emailaddress_in_HTML_avoid_SPAM(doc, s);
	} else {
		doc_puts(doc, "( <i>not registered</i> )");
	}
	doc_puts(doc, "</td></tr>\n");

#if 0
	doc_puts(doc, "<tr align=\"left\"><td>電話:</td><td>");
	tc = edb_tuple_seek_column(ec, person, (utf8_t *)"@.phone");
	if(tc && (d=tc->datum) && x(char *)DatumEnglishIsValid(d)) {
		char	*s = (char *)DatumEnglish(d);

		doc_text_puts3(tex_ep->doc, "\\定義{電話}{", (char *)DatumEnglish(d), "}\n");
		doc_puts(doc, "<b>");
		while(*s) {
			switch(*s) {
			case '-':
				doc_puts(doc, "&nbsp;-&nbsp;");
				break;
			default:
				doc_putc(doc, *s);
				break;
			}
			++s;
		}
		doc_puts(doc, "</b>");

		s = (char *)DatumEnglish(d);
		if(*s=='0') {
			doc_puts(doc, " ( <b>");
			doc_puts(doc, FOREIGN_EXTENSION);
			++s;
			while(*s) {
				switch(*s) {
				case '-':
					doc_puts(doc, "&nbsp;-&nbsp;");
					break;
				default:
					doc_putc(doc, *s);
					break;
				}
				++s;
			}
			doc_puts(doc, "</b> )");
		}
	} else {
		doc_puts(doc, "( <i>not registered</i> )");
	}
	doc_puts(doc, "</td></tr>\n");
#endif

#if 0
	doc_puts(doc, "<tr align=\"left\"><td>Fax:</td><td>");
	tc = edb_tuple_seek_column(ec, person, (utf8_t *)"@.fax");
	if(tc && (d=tc->datum) && DatumEnglishIsValid(d)) {
		char	*s = (char *)DatumEnglish(d);

		doc_text_puts3(tex_ep->doc, "\\定義{ファックス}{", (char *)DatumEnglish(d), "}\n");
		doc_puts(doc, "<b>");
		while(*s) {
			switch(*s) {
			case '-':
				doc_puts(doc, "&nbsp;-&nbsp;");
				break;
			default:
				doc_putc(doc, *s);
				break;
			}
			++s;
		}
		doc_puts(doc, "</b>");

		s = (char *)DatumEnglish(d);
		if(*s=='0') {
			doc_puts(doc, " ( <b>");
			doc_puts(doc, FOREIGN_EXTENSION);
			++s;
			while(*s) {
				switch(*s) {
				case '-':
					doc_puts(doc, "&nbsp;-&nbsp;");
					break;
				default:
					doc_putc(doc, *s);
					break;
				}
				++s;
			}
			doc_puts(doc, "</b> )");
		}
	} else {
		doc_puts(doc, "( <i>not registered</i> )");
	}
	doc_puts(doc, "</td></tr>\n");
#endif

	doc_puts(doc, "<tr align=\"left\"><td>WebPage:</td><td>");
	tc = edb_tuple_seek_column(ec, person, (utf8_t *)"@.url");
	if(tc && (d=tc->datum) && DatumEnglishIsValid(d)) {
		doc_puts(doc, "<a href=\"");
		doc_puts(doc, (char *)DatumEnglish(d));
		doc_text_puts3(doc, "\" target=\"_blank\">", (char *)DatumEnglish(d), "</a>");
	} else {
		doc_puts(doc, "( <i>not registered</i> )");
	}
	doc_puts(doc, "</td></tr>\n");
	doc_puts(doc, "</table>\n");

	doc_puts(doc, "<select onchange=\"moveto_selection(this);\">\n");
	doc_puts(doc, "<option value=\"\" selected=\"selected\">ページ内の移動</option>\n");
#ifdef	SHOW_HISTORY
	doc_puts(doc, "<option value=\"history.academic\">⇒ 学歴</option>\n");
	doc_puts(doc, "<option value=\"history.work\">⇒ 職歴</option>\n");
#endif
	doc_printf(doc, "<option value=\"lecture\">⇒ 担当授業</option>\n");
	doc_printf(doc, "<option value=\"study\">⇒ 研究</option>\n");
	doc_printf(doc, "<option value=\"academicsocietyactivity\">⇒ 所属学会</option>\n");
	doc_printf(doc, "<option value=\"socialactivity\">⇒ 社会活動</option>\n");
	doc_printf(doc, "<option value=\"work\">⇒ 研究業績</option>\n");
	for(i=0; i<Ns(catalogue_list); ++i) {
		if(edb_catalogue_size(sw->ca[i])==0) continue;
		doc_printf(doc, "<option value=\"%s\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;%s</option>\n", catalogue_list[i].anchor, catalogue_list[i].caption);
	}
	doc_puts(doc, "</select>\n");

#ifdef	SHOW_HISTORY
	doc_puts(doc, "<dl>\n");
	doc_puts(doc, "<dt><a name=\"history.academic\">学歴:</a> " RETURN_TO_PAGE_TOP "</dt>\n");
	ca = survey_get_person_history(ec, TupleEID(person), 10357);
	if(ca && edb_catalogue_size(ca)>0) {
		Link	l;

		doc_puts(tex_ep->doc, "\\定義{学歴}{\n");
		doc_puts(doc, "<dd><table border=\"0\" cellspacing=\"0\">");
		ForCatalogueLink(l, ca) {
			EdbTupleInfo	history = CatalogueObject(l);
			EdbTupleColumn	tc;
			EdbDatum			datum;

			doc_puts(doc, "<tr valign=\"top\">");
			tc = edb_tuple_seek_column(ec, history, (utf8_t *)"@.date");
			doc_puts(tex_ep->doc, "\\履歴{");
			if(tc && (datum=tc->datum) && DatumEnglishIsValid(datum)) {
				int	ymd = ATOI((char *)DatumEnglish(datum));

				doc_printf(doc, "<td align=\"right\">%04d.</td><td align=\"right\">%2d.</td>", DATE_Y(ymd), DATE_M(ymd));
				doc_printf(tex_ep->doc, "%04d年%2d日", DATE_Y(ymd), DATE_M(ymd));
			} else {
				doc_printf(doc, "<td align=\"right\">????.</td><td align=\"right\">??.</td>");
			}
			doc_puts(tex_ep->doc, "}{");
			doc_puts(doc, "<td align=\"left\">");
			tc = edb_tuple_seek_column(ec, history, (utf8_t *)"@.event");
			if(tc && (datum=tc->datum)) {
				if(DatumJapaneseIsValid(datum)) {
					doc_text_puts(doc, (char *)DatumJapanese(datum));
					doc_text_puts(tex_ep->doc, (char *)DatumJapanese(datum));
				} else if(DatumEnglishIsValid(datum)) {
					doc_text_puts(doc, (char *)DatumEnglish(datum));
					doc_text_puts(tex_ep->doc, (char *)DatumEnglish(datum));
				} else {
					doc_printf(doc, "??????");
				}
			} else {
				doc_printf(doc, "??????");
			}
			doc_puts(doc, "</td>");
			doc_puts(tex_ep->doc, "}\n");
			doc_puts(doc, "</tr>\n");
		}
		doc_puts(doc, "</table></dd>\n");
		doc_puts(tex_ep->doc, "}\n");
	} else {
		doc_puts(doc, "<dd>( <i>not registered</i> )</dd>\n");
	}
	edb_catalogue_free(ca);

	doc_puts(doc, "<dt><a name=\"history.work\">職歴:</a> " RETURN_TO_PAGE_TOP "</dt>\n");
	ca = survey_get_person_history(ec, TupleEID(person), 10358);
	if(ca && edb_catalogue_size(ca)>0) {
		Link	l;

		doc_puts(tex_ep->doc, "\\定義{職歴}{\n");
		doc_puts(doc, "<dd><table border=\"0\" cellspacing=\"0\">");
		ForCatalogueLink(l, ca) {
			EdbTupleInfo	history = CatalogueObject(l);
			EdbTupleColumn	tc;
			EdbDatum			datum;

			doc_puts(doc, "<tr valign=\"top\">");
			tc = edb_tuple_seek_column(ec, history, (utf8_t *)"@.date");
			doc_puts(tex_ep->doc, "\\履歴{");
			if(tc && (datum=tc->datum) && DatumEnglishIsValid(datum)) {
				int	ymd = ATOI((char *)DatumEnglish(datum));

				doc_printf(doc, "<td align=\"right\">%04d.</td><td align=\"right\">%2d.</td>", DATE_Y(ymd), DATE_M(ymd));
				doc_printf(tex_ep->doc, "%04d年%2d月", DATE_Y(ymd), DATE_M(ymd));
			} else {
				doc_printf(doc, "<td align=\"right\">????.</td><td align=\"right\">??.</td>");
			}
			doc_puts(tex_ep->doc, "}{");
			doc_puts(doc, "<td align=\"left\">");
			tc = edb_tuple_seek_column(ec, history, (utf8_t *)"@.event");
			if(tc && (datum=tc->datum)) {
				if(DatumJapaneseIsValid(datum)) {
					doc_text_puts(doc, (char *)DatumJapanese(datum));
					doc_text_puts(tex_ep->doc, (char *)DatumJapanese(datum));
				} else if(DatumEnglishIsValid(datum)) {
					doc_text_puts(doc, (char *)DatumEnglish(datum));
					doc_text_puts(tex_ep->doc, (char *)DatumEnglish(datum));
				} else {
					doc_printf(doc, "??????");
				}
			} else {
				doc_printf(doc, "??????");
			}
			doc_puts(doc, "</td>");
			doc_puts(tex_ep->doc, "}\n");
			doc_puts(doc, "</tr>\n");
		}
		doc_puts(doc, "</table></dd>\n");
		doc_puts(tex_ep->doc, "}\n");
	} else {
		doc_puts(doc, "<dd>( <i>not registered</i> )</dd>\n");
	}
	edb_catalogue_free(ca);
#endif

	doc_printf(doc, "<dt><a name=\"lecture\">担当授業(%d年度入学生分):</a> " RETURN_TO_PAGE_TOP "</dt>\n", THIS_YEAR(ec));
	ca = survey_get_person_lecture(ec, TupleEID(person));
	if(ca && edb_catalogue_size(ca)>0) {
		Link	l;

		ForCatalogueLink(l, ca) {
			EdbTupleInfo	lecture = CatalogueObject(l);

			doc_puts(doc, "<dd>");
			print_lecture(ec, ep, lecture);
			doc_puts(doc, "</dd>\n");
		}
	} else {
		doc_puts(doc, "<dd>( <i>not registered</i> )</dd>\n");
	}
	edb_catalogue_free(ca);

	doc_puts(doc, "<dt><a name=\"study\">研究:</a></dt>\n");
	ca = survey_get_person_study(ec, TupleEID(person));
	if(ca && edb_catalogue_size(ca)>0) {
		Link	l;

		ForCatalogueLink(l, ca) {
			EdbTupleInfo	study = CatalogueObject(l);

			doc_puts(doc, "<dd>");
			print_study(ec, ep, study);
			doc_puts(doc, "</dd>\n");
		}
	} else {
		doc_puts(doc, "<dd>( <i>not registered</i> )</dd>\n");
	}
	edb_catalogue_free(ca);

	doc_puts(doc, "<dt><a name=\"academicsocietyactivity\">所属学会:</a> " RETURN_TO_PAGE_TOP "</dt>\n");
	ca = survey_get_person_asa(ec, TupleEID(person));
	if(ca && edb_catalogue_size(ca)>0) {
		Link	l;

		ForCatalogueLink(l, ca) {
			EdbTupleInfo	asa = CatalogueObject(l);

			doc_puts(doc, "<dd>");
			print_asa(ec, ep, asa);
			doc_puts(doc, "</dd>\n");
		}
	} else {
		doc_puts(doc, "<dd>( <i>not registered</i> )</dd>\n");
	}
	edb_catalogue_free(ca);

	doc_puts(doc, "<dt><a name=\"socialactivity\">社会活動:</a> " RETURN_TO_PAGE_TOP "</dt>\n");
	ca = survey_get_person_sa(ec, TupleEID(person));
	if(ca && edb_catalogue_size(ca)>0) {
		Link	l;

		ForCatalogueLink(l, ca) {
			EdbTupleInfo	sa = CatalogueObject(l);

			doc_puts(doc, "<dd>");
			print_sa(ec, ep, sa);
			doc_puts(doc, "</dd>\n");
		}
	} else {
		doc_puts(doc, "<dd>( <i>not registered</i> )</dd>\n");
	}
	edb_catalogue_free(ca);

	doc_printf(doc, "<dt><a name=\"work\">研究業績%s:</a> " RETURN_TO_PAGE_TOP "</dt>\n", works_date_string);
	status = survey_work_print_all(ec, sw, FALSE, TRUE, ep);
	if(!status) doc_puts(doc, "<dd>( <i>not registered</i> )</dd>\n");
	doc_puts(doc, "</dl>\n");

	doc_puts(doc, "<table border=\"1\" cellspacing=\"0\" align=\"center\" width=\"100%\">\n");
	survey_print_work_summary("<tr>", NULL, "</tr>", NULL, ep);
	survey_print_work_summary("<tr align=\"right\">", sw, "</tr>", "", ep);
	doc_puts(doc, "</table><br /> " RETURN_TO_PAGE_TOP "\n");

	doc_puts(tex_ep->doc, "\\end{個人プロフィール}\n");
	doc_puts(tex_ep->doc, "\n");
	survey_end_HTML(ec, doc);
	edb_catalogue_free(ca_person);

	edb_print_free(ep);
	doc_close(doc);
	file_update(tmpname, name);
	FREE(dir);
	return sw;
}

static
EdbCatalogue
survey_get_organization_article_by_kind(EdbContext ec, eid_t organization, eid_t kind)
{
	char	*condition = NULL;

	if(works_date_from>DATE_MIN || works_date_to<DATE_MAX) {
		condition = (char *)edb_condition_make(ec, table_article, CONDITION_LOGIC_AND,
			ConditionEIDMatch("@.organization", organization),
			ConditionEIDMatch("@.kind", kind),
			(works_date_from>DATE_MIN) ? ConditionNoOperation : ConditionSkipNext,
			ConditionDateFrom("@.date", works_date_from),
			(works_date_to<DATE_MAX) ? ConditionNoOperation : ConditionSkipNext,
			ConditionIntegerLE("@.date", works_date_to),
			ConditionTerminate);
	} else {
		condition = (char *)edb_condition_make(ec, table_article, CONDITION_LOGIC_AND,
			ConditionEIDMatch("@.organization", organization),
			ConditionEIDMatch("@.kind", kind),
			ConditionTerminate);
	}
	return catalogue_get(ec, table_article, condition);
}

static
EdbCatalogue
survey_get_organization_patent_by_kind(EdbContext ec, eid_t organization, eid_t kind)
{
	char	*condition = NULL;

	if(works_date_from>DATE_MIN || works_date_to<DATE_MAX) {
		condition = (char *)edb_condition_make(ec, table_patent, CONDITION_LOGIC_AND,
			ConditionEIDMatch("@.organization", organization),
			ConditionEIDMatch("@.kind", kind),
			(works_date_from>DATE_MIN) ? ConditionNoOperation : ConditionSkipNext,
			ConditionDateFrom("@.date", works_date_from),
			(works_date_to<DATE_MAX) ? ConditionNoOperation : ConditionSkipNext,
			ConditionIntegerLE("@.date", works_date_to),
			ConditionTerminate);
	} else {
		condition = (char *)edb_condition_make(ec, table_patent, CONDITION_LOGIC_AND,
			ConditionEIDMatch("@.organization", organization),
			ConditionEIDMatch("@.kind", kind),
			ConditionTerminate);
	}
	return catalogue_get(ec, table_patent, condition);
}

static
SurveyWork
survey_get_organization_works(EdbContext ec, eid_t organization)
{
	SurveyWork	sw;
	int	i;

	sw = survey_work_new(FALSE);
	for(i=0; i<Ns(catalogue_list); ++i) {
		if(!strcmp(catalogue_list[i].tablename, "article")) {
			sw->ca[i] = survey_get_organization_article_by_kind(ec, organization, catalogue_list[i].kind);
		} else if(!strcmp(catalogue_list[i].tablename, "patent")) {
			sw->ca[i] = survey_get_organization_patent_by_kind(ec, organization, catalogue_list[i].kind);
		}
	}
	return sw;
}

static
eid_t
organization_attribute(EdbContext ec, EdbTupleInfo organization)
{
	register EdbTupleColumn	tc;
	register EdbDatum	d;

	tc = edb_tuple_seek_column(ec, organization, (utf8_t *)"@.attribute");
	if(tc && (d=tc->datum)) return d->d_eid;
	return 0;
}

static
int
survey_organization_page_sub		__P((EdbContext ec, SurveyPage page, SurveyWork sw, EdbTupleInfo organ, int layer, int layer2));

static
int
survey_organization_page_sub_child(EdbContext ec, SurveyPage page, SurveyWork sw, EdbTupleInfo organ, int layer, int layer2)
{
	EdbPrint	ep = page->ep;
	Doc	doc = ep->doc;
	EdbTupleColumn	tc;
	SurveyWork	sw_sub;

	tc = edb_tuple_seek_column(ec, organ, (utf8_t *)"@.member");
	if(tc && tc->datum) {
		EdbDatum	d;
		int	num = 0;

		ForTCDatum(d, tc) {
			EdbTupleInfo	child;
			Caption	cap;
			int	i;
			struct SURVEY_PAGE	sp;
			eid_t	attribute;
			SurveyWork	sw_organization = NULL;
			EdbCatalogue	ca_child;

			if(!EIDisVALID(d->d_eid)) continue;
			if(!edb_datum_at_present(ec, d)) continue;
			child = edb_tuple_get(ec, d->d_eid, 0);
			if(!TupleIsOrganization(child)) continue;

			if(TupleMapto(child)) continue;
			attribute = organization_attribute(ec, child);
			if(attribute == 10366) continue;		/* 教授会 */
			if(attribute == 10367) continue;		/* 委員会 */
			if(attribute == 10369) continue;		/* 教官グループ */
			if(attribute == 10400) continue;		/* 学会 */

			if(num==0 && layer>0) {
				doc_puts(doc, "<dt>\n");
				doc_puts(doc, "<ul>\n");
			}

			bcopy(page, &sp, sizeof(struct SURVEY_PAGE));
			sp.ep = NULL;
			sp.csv_ep = NULL;

			cap = edb_tuple_make_caption(ec, child, CAP_norecurse);
			sw_sub = survey_organization_page(ec, &sp, TupleEID(child), layer+1);
			if(sw_sub) survey_work_merge(sw, sw_sub);
			if(layer==0) {
				doc_puts(doc, "<tr align=\"right\"><td align=\"left\" rowspan=\"2\"><b>");
			} else {
				doc_printf(doc, "<li><b>");
			}
			doc_printf(doc, "<a href=\"../../%s/%d/index.html\">", TABLE_XN(TupleTable(child)), TupleEID(child));
			if(STRisVALID(cap->c_main)) {
				doc_text_printf(doc, "%s", cap->c_main);
			} else {
				doc_text_printf(doc, "[%d]", TupleEID(child));
			}
			doc_puts(doc, "</a>");

			if(layer==0 || layer2==0) {
				ca_child = tuple_expand(ec, child);
				sw_organization = survey_get_organization_works(ec, TupleEID(child));
				edb_catalogue_free(ca_child);
			}
			if(layer==0) {
				char	*workspage;

				asprintf(&workspage, "../../%s/%d/%s", TABLE_XN(TupleTable(child)), TupleEID(child), WORKS_BY_ORGANIZATION_PAGE);
				survey_print_work_summary("</b></td>", sw_organization, "<td align=\"left\">organization</td></tr>", workspage, ep);
				FREE(workspage);
				asprintf(&workspage, "../../%s/%d/%s", TABLE_XN(TupleTable(child)), TupleEID(child), WORKS_BY_MEMBER_PAGE);
				survey_print_work_summary("<tr bgcolor=\"#e0e0e0\" align=\"right\">", sw_sub, "<td align=\"left\">member</td></tr>", workspage, ep);
				FREE(workspage);
			} else {
				doc_puts(doc, "</b></li>\n");
			}
			if(layer2==0) {
				survey_print_work_summary((char *)cap->c_main, sw_organization, "organization", NULL, page->csv_ep);
				survey_print_work_summary((char *)cap->c_main, sw_sub, "member", NULL, page->csv_ep);
			}

			edb_caption_free(ec, cap);
			if(layer>0) survey_organization_page_sub(ec, page, sw, child, layer+1, layer2+1);
			++num;
		}
		if(num>0) if(layer>0) doc_puts(doc, "</ul></dt>\n");
	}

	return TRUE;
}

static
int
survey_organization_page_sub_person(EdbContext ec, SurveyPage page, SurveyWork sw, EdbTupleInfo organ, int layer, int layer2)
{
	EdbPrint	ep = page->ep;
	Doc	doc = ep->doc;
	EdbTupleColumn	tc;
	SurveyWork	sw_sub;

	tc = edb_tuple_seek_column(ec, organ, (utf8_t *)"@.member");
	if(tc && tc->datum) {
		EdbDatum	d;
		int	num = 0;
#define	NUM_OF_PORTRAITs_PER_COLUMN	10

		doc_puts(doc, "<dt>\n");
		doc_puts(doc, "<table border=\"0\">\n");

		ForTCDatum(d, tc) {
			EdbTupleInfo	person;
			Caption	cap;

			if(!EIDisVALID(d->d_eid)) continue;
			if(!edb_datum_at_present(ec, d)) continue;
			person = edb_tuple_get(ec, d->d_eid, 0);
			if(!TupleIsPerson(person)) continue;

			if(num%NUM_OF_PORTRAITs_PER_COLUMN==0) doc_puts(doc, "<tr align=\"center\">\n");
			cap = edb_tuple_make_caption(ec, person, CAP_norecurse);
			doc_printf(doc, "<td width=\"9%%\">");
			if((sw_sub=survey_person_page(ec, &person_pages, TupleEID(person), doc))!=NULL) {
				char	*pname;

				survey_work_merge(sw, sw_sub);
				pname = survey_person_portrait_name(ec, person);
				doc_printf(doc, "<a href=\"../../%s/%d/index.html\">", TABLE_XN(TupleTable(person)), TupleEID(person));
				if(pname) {
					doc_printf(doc, "<img src=\"../../%s/%d/%s\" alt=\"Portrait\" width=\"%d\" height=\"%d\" border=\"0\" /><br />", TABLE_XN(TupleTable(person)), TupleEID(person), pname, PORTRAIT_ICON_WIDTH, PORTRAIT_ICON_HEIGHT);
				} else {
					doc_printf(doc, "<img src=\"../../%s\" alt=\"Not Available\" width=\"%d\" height=\"%d\" border=\"0\" /><br />", ICON_BROKEN_PORTRAIT, PORTRAIT_ICON_WIDTH, PORTRAIT_ICON_HEIGHT);
				}
				edb_print_tuple_named_column(ep, organ, d, "", "", "@.title", NULL, ", ", "<br />", "", 0);
				if(STRisVALID(cap->c_main)) {
					doc_text_printf(doc, "%s", cap->c_main);
				} else {
					doc_text_printf(doc, "[%d]", TupleEID(person));
				}
				doc_puts(doc, "</a>\n");
			} else {
				edb_print_tuple_named_column(ep, organ, d, "", "", "@.title", NULL, ", ", "<br />", "", 0);
				if(STRisVALID(cap->c_main)) {
					doc_text_printf(doc, "%s", cap->c_main);
				} else {
					doc_text_printf(doc, "[%d]", TupleEID(person));
				}
			}
			doc_puts(doc, "</td>\n");
			++num;
			if(num%NUM_OF_PORTRAITs_PER_COLUMN==0) doc_puts(doc, "</tr>\n");
		}
		doc_puts(doc, "</table></dt>\n");
	}
	return TRUE;
}

static
int
survey_organization_page_sub(EdbContext ec, SurveyPage page, SurveyWork sw, EdbTupleInfo organ, int layer, int layer2)
{
	EdbPrint	ep = page->ep;
	Doc	doc = ep->doc;
	EdbTupleColumn	tc;
	SurveyWork	sw_sub;

	if(layer>10) return FALSE;
	if(layer>0) doc_puts(doc, "<dl>\n");

	if(layer==0) {
		survey_organization_page_sub_child(ec, page, sw, organ, layer, layer2);
	} else {
		survey_organization_page_sub_child(ec, page, sw, organ, layer, layer2);
		survey_organization_page_sub_person(ec, page, sw, organ, layer, layer2);
	}

	if(layer>0) doc_puts(doc, "</dl>\n");
	return TRUE;
}

static
void
survey_organization_make_survey_page(EdbContext ec, SurveyPage page, char *namehtml, SurveyWork sw, int reorder, EdbTupleInfo organization, Doc pdoc)
{
	Doc doc;
	EdbPrint	ep;
	char	name[BUFSIZ];
	char	tmpname[BUFSIZ];
	Caption	cap;
	char	*dir;
	char	*title;
	int	status;
	Link	l;
	EdbCatalogue	ca_organization;

	ca_organization = tuple_expand(ec, organization);

	asprintf(&dir, "%s/%s/%d", www_path, TABLE_XN(TupleTable(organization)), TupleEID(organization));
	makedir_hierarchy(dir);
	sprintf(name, "%s/%s", dir, namehtml);
	sprintf(tmpname, "%s.tmp", name);
	doc = doc_open(DocML_HTML, tmpname, "w+");
	if(doc==NULL) return;
	ep = edb_print_new(ec, doc, PrintStyle_STANDARD);
	ep->ct->noalert = TRUE;
	ep->ct->nocolor = TRUE;
	EpLang(ep) = EP_LANG_Japanese;
	edb_print_tuple_postfix_register_func(ep, PrintStyle(ep), edb_survey_print_tuple_postfix);
	edb_print_tuple_postfix_register_func(ep, EP_PrintStyle_CAPTION, edb_survey_print_tuple_postfix);
	ForCatalogueLink(l, ca_organization) edb_print_eoi_add(ep, TupleEID((EdbTupleInfo)CatalogueObject(l)));

	cap = edb_make_caption(ec, TupleEID(organization), 0);
	asprintf(&title, "%s: %s", page->title, cap->c_main);
	survey_begin_HTML(ec, title, doc);
	FREE(title);
	doc_printf(doc, "<h1>%s: ", TupleTable(organization)->name.japanese);
	doc_text_puts(doc, (char *)cap->c_main);
#if 0
	doc_printf(doc, "[<a href=\"%s%s?EID=%d\" target=\"_blank\">%s</a>]", WEB_SERVER_URL, EDB_BROWSER_URL_DEFAULT, TupleEID(organization), NAME_OF_DB);
#else
	JavaScript_printf(doc, "edb_ref_by_eid(%d);", TupleEID(organization));
#endif
	doc_puts(doc, "</h1>\n");
	edb_caption_free(ec, cap);
	doc_puts(doc, "<hr />\n");

	doc_puts(doc, "<dl>\n");
	doc_printf(doc, "<dt>Works%s:</dt>\n", works_date_string);
	status = survey_work_print_all(ec, sw, reorder, FALSE, ep);
	if(!status) doc_puts(doc, "<dd>( <i>not registered</i> )</dd>\n");
	doc_puts(doc, "</dl>\n");

	survey_end_HTML(ec, doc);
	edb_catalogue_free(ca_organization);
	edb_print_free(ep);
	doc_close(doc);
	file_update(tmpname, name);
	FREE(dir);
}

static
SurveyWork
survey_organization_page(EdbContext ec, SurveyPage page, eid_t eid, int layer)
{
	Doc				doc, csv_doc;
	EdbPrint			ep;
	char				name[BUFSIZ];
	char				tmpname[BUFSIZ];
	char				csv_name[BUFSIZ];
	char				csv_tmpname[BUFSIZ];
	Link				l;
	int				i;
	EdbTupleInfo		organization;
	Caption			cap;
	char				*dir;
	char				*title;
	EdbTupleColumn	tc;
	SurveyWork		sw;
	SurveyWork		sw_by_organization;
	EdbCatalogue		ca_organization;
	EdbDatum			datum;

	sw = (SurveyWork)edb_catalogue_lookup(ca_sw_organization, eid);
	if(sw) return sw;

	organization = edb_tuple_get(ec, eid, 0);
	if(organization==NULL) return NULL;

	ca_organization = tuple_expand(ec, organization);
	sw = survey_work_new(TRUE);
	edb_catalogue_append(ca_sw_organization, eid, sw);

	asprintf(&dir, "%s/%s/%d", www_path, TABLE_XN(TupleTable(organization)), eid);
	makedir_hierarchy(dir);
	sprintf(name, "%s/index.html", dir);
	sprintf(tmpname, "%s.tmp", name);
	doc = doc_open(DocML_HTML, tmpname, "w+");
	if(doc==NULL) return NULL;
	page->ep = ep = edb_print_new(ec, doc, PrintStyle_STANDARD);
	ep->ct->noalert = TRUE;
	ep->ct->nocolor = TRUE;
	EpLang(ep) = EP_LANG_Japanese;
	edb_print_tuple_postfix_register_func(ep, PrintStyle(ep), edb_survey_print_tuple_postfix);
	edb_print_tuple_postfix_register_func(ep, EP_PrintStyle_CAPTION, edb_survey_print_tuple_postfix);
	ForCatalogueLink(l, ca_organization) edb_print_eoi_add(ep, TupleEID((EdbTupleInfo)CatalogueObject(l)));

	sprintf(csv_name, "%s/%s", dir, WORKS_BY_CSV_PAGE);
	sprintf(csv_tmpname, "%s.tmp", csv_name);
	csv_doc = doc_open(DocML_PLAIN, csv_tmpname, "w+");
	if(csv_doc==NULL) return NULL;
	page->csv_ep= edb_print_new(ec, csv_doc, PrintStyle_CSV);
	page->csv_ep->ct->noalert = TRUE;
	page->csv_ep->ct->nocolor = TRUE;
	EpLang(page->csv_ep) = EP_LANG_Japanese;

	cap = edb_make_caption(ec, eid, 0);
	asprintf(&title, "%s: %s", page->title, cap->c_main);
	survey_begin_HTML(ec, title, doc);
	FREE(title);
	doc_printf(doc, "<h1>%s: ", TupleTable(organization)->name.japanese);
	doc_text_puts(doc, (char *)cap->c_main);
#if 0
	doc_printf(doc, "[<a href=\"%s%s?EID=%d\" target=\"_blank\">%s</a>]", WEB_SERVER_URL, EDB_BROWSER_URL_DEFAULT, TupleEID(organization), NAME_OF_DB);
#else
	JavaScript_printf(doc, "edb_ref_by_eid(%d);", TupleEID(organization));
#endif
	doc_puts(doc, "</h1>\n");
	edb_caption_free(ec, cap);
	doc_puts(doc, "<hr />\n");
	if(page->desc) {
		doc_printf(doc, "%s\n", page->desc);
		doc_puts(doc, "<hr />\n");
	}

	doc_puts(doc, "<dl>\n");
	tc = edb_tuple_seek_column(ec, organization, (utf8_t *)"@.chief");
	doc_puts(doc, "<dt>長: </dt>\n");
	if(tc && (datum=tc->datum) && EIDisVALID(datum->d_eid)) {
		EdbTupleInfo	chief;

		i=10;
		while((chief=edb_tuple_get(ec, datum->d_eid, 0))!=NULL) {
			if(TupleIsPerson(chief)) {
				char	*pname;
				Caption	cap = edb_tuple_make_caption(ec, chief, CAP_norecurse);

				doc_puts(doc, "<dd>");
				if(survey_person_page(ec, &person_pages, TupleEID(chief), doc)) {
					pname = survey_person_portrait_name(ec, chief);
					doc_printf(doc, "<a href=\"../../%s/%d/index.html\">", TABLE_XN(TupleTable(chief)), TupleEID(chief));
					if(pname) {
						doc_printf(doc, "<img src=\"../../%s/%d/%s\" alt=\"Portrait\" width=\"%d\" height=\"%d\" border=\"0\" /><br />", TABLE_XN(TupleTable(chief)), TupleEID(chief), pname, PORTRAIT_ICON_WIDTH, PORTRAIT_ICON_HEIGHT);
					} else {
						doc_printf(doc, "<img src=\"../../%s\" alt=\"Not Available\" width=\"%d\" height=\"%d\" border=\"0\" /><br />", ICON_BROKEN_PORTRAIT, PORTRAIT_ICON_WIDTH, PORTRAIT_ICON_HEIGHT);
					}
					if(STRisVALID(cap->c_main)) {
						edb_print_tuple_named_column(ep, chief, NULL, "", "", "@.title", NULL, ", ", "<br />", "", 0);
						doc_text_printf(doc, "%s", cap->c_main);
					} else {
						doc_text_printf(doc, "[%d]", TupleEID(chief));
					}
					doc_puts(doc, "</a>\n");
				} else {
					if(STRisVALID(cap->c_main)) {
						edb_print_tuple_named_column(ep, chief, NULL, "", "", "@.title", NULL, ", ", "<br />", "", 0);
						doc_text_printf(doc, "%s", cap->c_main);
					} else {
						doc_text_printf(doc, "[%d]", TupleEID(chief));
					}
				}
				edb_caption_free(ec, cap);
				doc_puts(doc, "</dd>\n");
				break;
			} else if(TupleIsPersonification(chief)) {
				tc = edb_tuple_seek_column(ec, chief, (utf8_t *)"@.person");
				if(!(tc && (datum=tc->datum) && EIDisVALID(datum->d_eid))) break;
				--i;
			} else {
				break;
			}
			if(i<0) break;
		}
	} else {
		doc_puts(doc, "<dd>( <i>not registered</i> )</dd>\n");
	}

	doc_puts(doc, "<dt>概要: </dt>\n");
	edb_print_tuple_named_column(ep, organization, NULL, "<dd>", "<dd>", "@.outline", NULL, "</dd>", "</dd>", "<dd>( <i>not registered</i> )</dd>", 0);
	if(layer==0) survey_organization_page_sub_person(ec, page, sw, organization, layer, 0);
	doc_puts(doc, "</dl>\n");

	sw_by_organization = survey_get_organization_works(ec, TupleEID(organization));
	survey_organization_make_survey_page(ec, page, WORKS_BY_ORGANIZATION_PAGE, sw_by_organization, FALSE, organization, doc);
	if(layer==0) {
		doc_puts(doc, "<table border=\"1\" cellspacing=\"0\" align=\"center\" width=\"100%\">\n");
		survey_print_work_summary("<tr><th>Organization</th>", NULL, "<th>Collected by</th></tr>", NULL, ep);
	}
	survey_print_work_summary("Organization", NULL, "Collected by", NULL, page->csv_ep);
	survey_organization_page_sub(ec, page, sw, organization, layer, 0);
	survey_organization_make_survey_page(ec, page, WORKS_BY_MEMBER_PAGE, sw, TRUE, organization, doc);
	if(layer==0) {
		survey_print_work_summary("<tr align=\"right\"><th rowspan=\"2\">Total</th>", sw_by_organization, "<td align=\"left\">organization</td></tr>", WORKS_BY_ORGANIZATION_PAGE, ep);
		survey_print_work_summary("<tr bgcolor=\"#e0e0e0\" align=\"right\">", sw, "<td align=\"left\">member</td></tr>", WORKS_BY_MEMBER_PAGE, ep);
		doc_puts(doc, "</table>\n");
	} else {
		doc_puts(doc, "<table border=\"1\" cellspacing=\"0\" align=\"center\" width=\"100%\">\n");
		survey_print_work_summary("<tr>", NULL, "<th>Collected by</th></tr>", NULL, ep);
		survey_print_work_summary("<tr align=\"right\">", sw_by_organization, "<td align=\"left\">organization</td></tr>", WORKS_BY_ORGANIZATION_PAGE, ep);
		survey_print_work_summary("<tr bgcolor=\"#e0e0e0\" align=\"right\">", sw, "<td align=\"left\">member</td></tr>", WORKS_BY_MEMBER_PAGE, ep);
		doc_puts(doc, "</table>\n");
	}
	doc_printf(doc, "<a href=\"%s\">CSV (TAB Separated Values) formatted file</a> for spreadsheet softwares (e.g. MicroSoft Excel).\n", WORKS_BY_CSV_PAGE);
	survey_print_work_summary("Total", sw_by_organization, "organization", NULL, page->csv_ep);
	survey_print_work_summary("Total", sw, "member", NULL, page->csv_ep);

	survey_end_HTML(ec, doc);
	edb_catalogue_free(ca_organization);
	edb_print_free(page->csv_ep);
	edb_print_free(ep);
	doc_close(doc);
	doc_close(csv_doc);
	file_update(tmpname, name);
	file_update(csv_tmpname, csv_name);
	FREE(dir);
	return sw;
}

static
void
survey_person_index_page(EdbContext ec, SurveyPage page)
{
	Doc doc, csv_doc;
	EdbPrint	ep, csv_ep;
	char	name[BUFSIZ];
	char	tmpname[BUFSIZ];
	char	csv_name[BUFSIZ];
	char	csv_tmpname[BUFSIZ];
	Link	l;
	Caption	cap;
	char	*dir;
	int	num;

	asprintf(&dir, "%s", www_path);
	makedir_hierarchy(dir);
	sprintf(name, "%s/%s", dir, page->name);
	sprintf(tmpname, "%s.tmp", name);
	doc = doc_open(DocML_HTML, tmpname, "w+");
	if(doc==NULL) return;
	page->ep = ep = edb_print_new(ec, doc, PrintStyle_STANDARD);
	ep->ct->noalert = TRUE;
	ep->ct->nocolor = TRUE;
	EpLang(ep) = EP_LANG_Japanese;
	edb_print_tuple_postfix_register_func(ep, PrintStyle(ep), edb_survey_print_tuple_postfix);
	edb_print_tuple_postfix_register_func(ep, EP_PrintStyle_CAPTION, edb_survey_print_tuple_postfix);

	sprintf(csv_name, "%s/%s", dir, PERSON_CSV);
	sprintf(csv_tmpname, "%s.tmp", csv_name);
	csv_doc = doc_open(DocML_HTML, csv_tmpname, "w+");
	if(csv_doc==NULL) return;
	csv_ep= edb_print_new(ec, csv_doc, PrintStyle_CSV);
	csv_ep->ct->noalert = TRUE;
	csv_ep->ct->nocolor = TRUE;
	EpLang(csv_ep) = EP_LANG_Japanese;

	survey_begin_HTML(ec, page->title, doc);
	doc_printf(doc, "<h1>%s</h1>", page->title);
	doc_puts(doc, "<hr />\n");

	doc_puts(doc, "<table border=\"0\" width=\"100%\">\n");

	num = 0;
	survey_print_work_summary("Person", NULL, NULL, NULL, csv_ep);
	ForCatalogueLink(l, table_person->ca_all_tuples) {
		EdbTupleInfo	person = CatalogueObject(l);
		Caption	cap;
		SurveyWork	sw_p;

		if(!edb_catalogue_lookup(ca_sw_person, TupleEID(person))) continue;
		if(num%NUM_OF_PORTRAITs_PER_COLUMN==0) doc_puts(doc, "<tr align=\"center\">\n");
		cap = edb_tuple_make_caption(ec, person, CAP_norecurse);
		doc_printf(doc, "<td>");
		if((sw_p=survey_person_page(ec, &person_pages, TupleEID(person), doc))!=NULL) {
			char	*pname;

			pname = survey_person_portrait_name(ec, person);
			doc_printf(doc, "<a href=\"%s/%d/index.html\">", TABLE_XN(TupleTable(person)), TupleEID(person));
			if(pname) {
				doc_printf(doc, "<img src=\"%s/%d/%s\" alt=\"Portrait\" width=\"%d\" height=\"%d\" border=\"0\" /><br />", TABLE_XN(TupleTable(person)), TupleEID(person), pname, PORTRAIT_ICON_WIDTH, PORTRAIT_ICON_HEIGHT);
			} else {
				doc_printf(doc, "<img src=\"%s\" alt=\"Not Available\" width=\"%d\" height=\"%d\" border=\"0\" /><br />", ICON_BROKEN_PORTRAIT, PORTRAIT_ICON_WIDTH, PORTRAIT_ICON_HEIGHT);
			}
			if(STRisVALID(cap->c_main)) {
				edb_print_tuple_named_column(ep, person, NULL, "", "", "@.title", NULL, ", ", "<br />", "", 0);
				doc_text_printf(doc, "%s", cap->c_main);
			} else {
				doc_text_printf(doc, "[%d]", TupleEID(person));
			}
			doc_puts(doc, "</a>\n");
			survey_print_work_summary((char *)cap->c_main, sw_p, NULL, NULL, csv_ep);
		} else {
			if(STRisVALID(cap->c_main)) {
				edb_print_tuple_named_column(ep, person, NULL, "", "", "@.title", NULL, ", ", "<br />", "", 0);
				doc_text_printf(doc, "%s", cap->c_main);
			} else {
				doc_text_printf(doc, "[%d]", TupleEID(person));
			}
		}
		doc_puts(doc, "</td>\n");
		edb_caption_free(ec, cap);
		++num;
		if(num%NUM_OF_PORTRAITs_PER_COLUMN==0) doc_puts(doc, "</tr>\n");
	}
	doc_puts(doc, "</table></dt>\n");
	doc_printf(doc, "<a href=\"%s\">CSV (TAB Separated Values) formatted file</a> for spreadsheet softwares (e.g. MicroSoft Excel).\n", PERSON_CSV);

	survey_end_HTML(ec, doc);
	edb_print_free(csv_ep);
	edb_print_free(ep);
	doc_close(doc);
	file_update(tmpname, name);
	file_update(csv_tmpname, csv_name);
	FREE(dir);
}

static
int
survey_initialize(EdbContext ec)
{
	int	i;

	for(i=0; i<Ns(preload_tables); ++i) {
		*preload_tables[i].table = edb_table_get_by_name(ec, (utf8_t *)preload_tables[i].xmlname);
		if(*preload_tables[i].table==NULL) return FALSE;
		edb_table_prefetch_tuples(ec, *preload_tables[i].table);
	}

	ca_sw_organization = edb_catalogue_new();
	ca_sw_person = edb_catalogue_new();

	return TRUE;
}

int	edb_make_survey	__P((EdbContext, int, char *[], FILE *));

int
edb_make_survey(EdbContext ec, int ac, char *av[], FILE *fp)
{
	char	name[BUFSIZ];
	char	tmpname[BUFSIZ];
	char	tex_name[BUFSIZ];
	char	tex_tmpname[BUFSIZ];
	Doc	doc, tex_doc;
	struct tm tm;

	if(ac>0 && STRisVALID(av[0])) {
		asprintf(&www_path, "%s", av[0]);
		++av;
		--ac;
	}
	if(ac>0 && STRisVALID(av[0])) {
		surveyname = strdup(av[0]);
		++av;
		--ac;
	}
	if(ac>0) {
		works_date_from = ATOI(av[0]);
		asprintf(&works_date_string, " (%04d/%02d-)",
			DATE_Y(works_date_from), DATE_M(works_date_from));
		++av;
		--ac;
	}
	if(ac>0) {
		works_date_to = ATOI(av[0]);
		asprintf(&works_date_string, " (%04d/%02d-%04d/%02d)",
			DATE_Y(works_date_from), DATE_M(works_date_from),
			DATE_Y(works_date_to), DATE_M(works_date_to));
		++av;
		--ac;
	}

	if(chdir(www_path)<0) return FALSE;

	edb_begin(ec, FALSE);
	localtime_r(&ec->now, &tm);
	ec->present_date = YMD_DATE(tm.tm_year + 1900, tm.tm_mon+1, tm.tm_mday);
	ec->present_wday = tm.tm_wday;
	ec->present_time = HMS_TIME(tm.tm_hour, tm.tm_min, tm.tm_sec);
//	fprintf(stderr, "THIS_YEAR = %d\n", THIS_YEAR(ec));

	if(!survey_initialize(ec)) {
		edb_end(ec);
		return FALSE;
	}

	edb_caption_omit_append(ec, MY_ORGAN_EID);

	sprintf(name, "index.html");
	sprintf(tmpname, "%s.tmp", name);
	doc = doc_open(DocML_HTML, tmpname, "w+");
	if(doc==NULL) return FALSE;

	sprintf(tex_name, "contents.tex");
	sprintf(tex_tmpname, "%s.tmp", tex_name);
	tex_doc = doc_open(DocML_TeX, tex_tmpname, "w+");
	if(tex_doc==NULL) return FALSE;
	tex_ep = edb_print_new(ec, tex_doc, PrintStyle_STANDARD);
	tex_ep->ct->nounderline=TRUE;
	tex_ep->ct->nocolor=TRUE;
	tex_ep->ct->noalert=TRUE;
	EpLang(tex_ep) = EP_LANG_Japanese;

	survey_begin_HTML(ec, "Survey on " NAME_OF_DB, doc);
	if(works_date_from>DATE_MIN || works_date_to<DATE_MAX) {
		doc_printf(doc, "<h1>Survey on " NAME_OF_DB " %s</h1>\n", works_date_string);
	} else {
		doc_printf(doc, "<h1>Survey on " NAME_OF_DB "</h1>\n");
	}
	doc_puts(doc, "<hr />\n");
	doc_puts(doc, "<blockquote>\n");
	doc_puts(doc, "<h2>Contents</h2>\n");
	doc_puts(doc, "<ul>\n");

	fprintf(fp, "%s\n", organization_pages.title);
	doc_printf(doc, "<li> <a href=\"%s/%d/index.html\"><b>%s</b></a>\n",
		TABLE_XN(table_organization),
		MY_ORGAN_EID,
		organization_pages.title);
	survey_organization_page(ec, &organization_pages, MY_ORGAN_EID, 0);

	fprintf(fp, "%s\n", person_index_pages.title);
	doc_printf(doc, "<li> <a href=\"%s\"><b>%s</b></a>\n",
		person_index_pages.name,
		person_index_pages.title);
	survey_person_index_page(ec, &person_index_pages);
	doc_puts(doc, "</ul>\n");

	doc_puts(doc, "<a href=\"contents.tex\">LaTeX version (person)</a><br />\n");
	doc_printf(doc, "<a href=\"%s.tar.gz\">%s.tar.gz</a>\n", surveyname, surveyname);
	doc_puts(doc, "</blockquote>\n");
	survey_end_HTML(ec, doc);
	edb_end(ec);

	edb_print_free(tex_ep);
	doc_close(tex_doc);
	file_update(tex_tmpname, tex_name);
	doc_close(doc);
	file_update(tmpname, name);

	return TRUE;
}


